Package: svUnit
Type: Package
Version: 1.0.8
Title: 'SciViews::R' - Unit, Integration and System Testing
Description: A complete unit test system.
Authors@R: c(
  person("Philippe", "Grosjean", role = c("aut", "cre"),
    email = "phgrosjean@sciviews.org",
    comment = c(ORCID = "0000-0002-2694-9471")))
Maintainer: Philippe Grosjean <phgrosjean@sciviews.org>
Depends: R (>= 1.9.0)
Imports: utils (>= 1.9.0)
Suggests: svGUI (>= 1.0.0), datasets (>= 1.9.0), XML (>= 3.99.0.10),
        RUnit (>= 0.4.30), covr (>= 3.5.0), knitr (>= 1.42), rmarkdown
        (>= 2.21), spelling (>= 2.2.1)
License: GPL-2
URL: https://github.com/SciViews/svUnit,
        https://www.sciviews.org/svUnit/,
        https://sciviews.r-universe.dev/svUnit
BugReports: https://github.com/SciViews/svUnit/issues
RoxygenNote: 7.3.2
VignetteBuilder: knitr
Encoding: UTF-8
Language: en-US
ByteCompile: yes
NeedsCompilation: no
Packaged: 2025-08-26 10:27:46 UTC; phgrosjean
Author: Philippe Grosjean [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-2694-9471>)
Repository: CRAN
Date/Publication: 2025-08-26 11:00:02 UTC
Built: R 4.6.0; ; 2026-04-25 16:34:16 UTC; unix

Package: svUnit
Type: Package
Version: 1.0.8
Title: 'SciViews::R' - Unit, Integration and System Testing
Description: A complete unit test system.
Authors@R: c(
  person("Philippe", "Grosjean", role = c("aut", "cre"),
    email = "phgrosjean@sciviews.org",
    comment = c(ORCID = "0000-0002-2694-9471")))
Maintainer: Philippe Grosjean <phgrosjean@sciviews.org>
Depends: R (>= 1.9.0)
Imports: utils (>= 1.9.0)
Suggests: svGUI (>= 1.0.0), datasets (>= 1.9.0), XML (>= 3.99.0.10),
        RUnit (>= 0.4.30), covr (>= 3.5.0), knitr (>= 1.42), rmarkdown
        (>= 2.21), spelling (>= 2.2.1)
License: GPL-2
URL: https://github.com/SciViews/svUnit,
        https://www.sciviews.org/svUnit/,
        https://sciviews.r-universe.dev/svUnit
BugReports: https://github.com/SciViews/svUnit/issues
RoxygenNote: 7.3.2
VignetteBuilder: knitr
Encoding: UTF-8
Language: en-US
ByteCompile: yes
NeedsCompilation: no
Packaged: 2025-08-26 10:27:46 UTC; phgrosjean
Author: Philippe Grosjean [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-2694-9471>)
Repository: CRAN
Date/Publication: 2025-08-26 11:00:02 UTC
Built: R 4.6.0; ; 2026-04-26 11:30:55 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(makeUnit,default)
S3method(makeUnit,svSuite)
S3method(makeUnit,svTest)
S3method(metadata,svSuiteData)
S3method(print,svSuite)
S3method(print,svSuiteData)
S3method(print,svTest)
S3method(print,svTestData)
S3method(protocol,default)
S3method(protocol,svSuiteData)
S3method(protocol_junit,svSuiteData)
S3method(protocol_junit,svTestData)
S3method(protocol_text,svSuiteData)
S3method(runTest,default)
S3method(runTest,list)
S3method(runTest,svSuite)
S3method(runTest,svTest)
S3method(stats,svSuiteData)
S3method(stats,svTestData)
S3method(summary,svSuiteData)
S3method(summary,svTestData)
export("test<-")
export(DEACTIVATED)
export(Log)
export(as.svSuite)
export(as.svTest)
export(checkEquals)
export(checkEqualsNumeric)
export(checkException)
export(checkIdentical)
export(checkTrue)
export(clearLog)
export(createLog)
export(errorLog)
export(guiSuiteAutoList)
export(guiSuiteList)
export(guiTestFeedback)
export(guiTestReport)
export(is.svSuite)
export(is.svSuiteData)
export(is.svTest)
export(is.svTestData)
export(is.test)
export(koUnit_isAutoTest)
export(koUnit_runTest)
export(koUnit_setAutoTest)
export(koUnit_showRUnitPane)
export(koUnit_version)
export(lastSuite)
export(lastTest)
export(makeTestListFromExamples)
export(makeUnit)
export(metadata)
export(protocol)
export(protocol_junit)
export(protocol_text)
export(runTest)
export(stats)
export(svSuite)
export(svSuiteList)
export(svTest)
export(test)
importFrom(utils,sessionInfo)
importFrom(utils,str)

#' Report or give feedback to the GUI client about running test units
#'
#' These functions are usually not called from the command line. They return
#' data to compatible GUI clients, like Komodo Edit with the SciViews-K
#' extension.
#'
#' @param object a 'svUnitData' object.
#' @param sep Field separator to use in the results.
#' @param path Path where to write a 'Suites.txt' file with the list of
#' currently available test suites (to be used by the GUI client). If `NULL`,
#' no file is written (by default).
#' @param ... Not used currently.
#' @param compare Do we compare the list of available test suite and return
#' something to the GUI client only if there are changes in the list? This is
#' used (when `TRUE`) to avoid unnecessary multiple processing of the same list
#' by the GUI client.
#'
#' @return
#' [guiSuiteList()] returns the list of available test suites invisibly.
#' [guiSuiteAutoList()] is used to establish a callback to automatically list
#' the available test suites in the GUI. It is not intended to be called
#' directly by the user. The other functions just return `TRUE` invisibly.They
#' are used for their side effect of sending data to compatible GUI clients.
#'
#' @export
#' @author Philippe Grosjean
#' @seealso [svTest()], [svSuite()], [koUnit_version()]
#' @keywords utilities
#' @concept unit testing
guiTestReport <- function(object, sep = "\t", path = NULL, ...) {
  # Report the results of a test to the GUI client
  if (!is.svSuiteData(object))
    stop("'object' must be a 'svSuiteData' object")

  # For all 'svTestData' objects, create a table with test results for the GUI
  # Indicate global results of the Unit Test
  Tests <- ls(object)
  if (length(Tests) == 0) {
    Res <- "<<<svUnitSummary>>>|||0|||0|||0|||0"
  } else {
    # Get general information about the tests
    Stats <- stats(object)
    Tests <- rownames(Stats)  # To make sure we use the same!
    Stats$label <- paste(">", sub("^test", "", Tests), " (",
      round(Stats$timing, 3), " sec)", sep = "")
    State <- table(Stats$kind)
    Res <- paste("<<<svUnitSummary>>>|||", State[1], "|||", State[2],
      "|||", State[3], "|||", State[4], sep = "")
    Kinds <- as.numeric(Stats$kind)
    Kinds[Kinds == 4] <- 0   # Use 0 instead of 4 for deactivated tests
    Stats$kind <- Kinds
    # Get the type for the objects
    Units <- Stats$unit
    Types <- rep("units in packages", length(Units))
    Types[Units == ""] <- "other objects"
    # TODO: include also dirs!
    Dir1 <- gsub("\\\\", "/", dirname(Units))
    Dir2 <- dirname(Dir1)
    Dir3 <- dirname(Dir2)
    TempDir <- gsub("\\\\", "/", tempdir())
    Types[Dir1 == TempDir] <- "objects in .GlobalEnv"
    Types[tolower(basename(Dir2)) == "inst" ||
      tolower(basename(Dir3)) == "inst"] <- "units in sources"
    # Keep only "*" in Units
    Units <- basename(Units)
    Units[regexpr("^runit.+\\.[rR]$", Units) == -1] <- ""
    Units[Dir1 == TempDir] <- "" # No second level for objects in .GlobalEnv
    Units <- sub("^runit(.+)\\.[rR]$", "\\1", Units)
    change <- Units != ""
    Units[change] <- paste(">unit", Units[change])
    # Complete label is Type<Unit<Test (x.xxx sec)
    Stats$label <- paste(Types, Units, Stats$label, sep = "")
    # Sort Tests and Stats according to label alphabetically
    ord <- order(Stats$label)
    Stats <- Stats[ord, ]
    Tests <- Tests[ord]
    # Get detailed information about each test
    lastUnit <- ""
    for (Test in Tests) {
      Data <- Stats[Test, ]
      # Calculate Info
      tData <- Log()[[Test]]
      tStats <- stats(tData)
      Info <- paste(c("Pass:", "Fail:", "Errors:"), tStats$kind[1:3],
        collapse = " ")
      # Don't print tests that succeed if !all
      tData <- tData[tData$kind != "OK", ]
      # Get info about each individual filtered test
      if (nrow(tData) > 0) {
        Result <- ifelse(tData$res == "", "",
          paste("\n", tData$res, sep = ""))
        Info <- paste(Info, "\n", paste("* ", tData$msg, ": ",
          tData$call, .formatTime(tData$timing, secDigits = 3),
          " ... ", as.character(tData$kind), Result, sep = "",
          collapse = "\n"), sep = "")
      }
      # Calculate URI (currently, the name of the unit file
      # and the name of the test function)
      if (Data$unit == "") {
        URI <- Data$unit
      } else {
        URI <- paste(Data$unit, Test, sep = "#")
      }
      if (Data$unit != lastUnit) {
        lastUnit <- Data$unit
        Res <- c(Res, paste("<<<svUnitFile>>>|||", Data$unit,
          "|||||||||", sep = ""))
      }
      Res <- c(Res, paste("<<<svUnitTest>>>|||", Data$label, "|||",
        Data$kind, "|||", Info, "|||", URI, sep = ""))
    }
  }
  Res <- paste(gsub("\t", "    ", Res), collapse = sep)
  if (is.null(path)) {
    return(Res)
  } else {
    cat(Res, file = path)
  }
  path
}

#' @export
#' @rdname guiTestReport
guiSuiteList <- function(sep = "\t", path = NULL, compare = TRUE) {
  Suites <- svSuiteList()

  if (compare) {
    oldSuites <- .getTemp(".guiSuiteListCache", default = "")
    # Compare both versions
    if (!identical(Suites, oldSuites)) {
      # Keep a copy of the last version in SciViews:TempEnv
      .assignTemp(".guiSuiteListCache", Suites)
      Changed <- TRUE
    } else Changed <- FALSE
  } else {
    Changed <- TRUE
    .assignTemp(".guiSuiteListCache", Suites)
  }

  if (is.null(path)) {# Return result, as a single character string with sep
    if (Changed) {
      if (!is.null(sep))
        Suites <- paste(Suites, collapse = sep)
      return(Suites)
    } else {
      return(NULL)
    }
  } else {# Write to a file called 'Suites.txt' in this path
    file <- file.path(path, "Suites.txt")
    if (Changed) {
      if (is.null(sep))
        sep <- "\n"
      cat(Suites, sep = sep, file = file)
    }
    return(invisible(Changed))
  }
}

#' @export
#' @rdname guiTestReport
guiSuiteAutoList <- function(...) {
  # Is koCmd() available?
  if (!exists("koCmd", mode = "function"))
    return(TRUE)

  # Is it something changed in the unit list?
  res <- guiSuiteList(sep = ",", path = NULL, compare = TRUE)
  if (!is.null(res))
    ret <- get("koCmd")('sv.r.unit.getRUnitList_Callback("<<<data>>>");',
      data = res)
  return(TRUE)
}

#' @export
#' @rdname guiTestReport
guiTestFeedback <- function(object, path = NULL, ...) {
  # Give feedback to client about the currently running tests
  # TODO: feedback about test run
}

#' @export
#' @rdname svTest
makeTestListFromExamples <- function(packageName, manFilesDir,
skipFailing = FALSE) {
  manPageFiles <- list.files(manFilesDir, pattern = "\\.Rd$")
  manPages <- sapply(manPageFiles,
    function(filename) {
      lines <- readLines(paste(manFilesDir, filename, sep = "/"))
      lines <- lines[grep("^\\\\name[ ]*\\{(.*)\\}", lines)]
      sub("^\\\\name[ ]*\\{(.*)\\}", lines, replacement = "\\1")
    }
  )
  manPages <- manPages[manPages != paste(packageName, "package", sep = "-")]
  names(manPages) <- manPages

  lapply(manPages, function(x) {
    testCall <- call("example", x, packageName)
    if (skipFailing) {
      onFailure <- function(w) {
        DEACTIVATED(paste(w, collapse = "\n"))
        checkTrue(FALSE)
      }
    } else {
      onFailure <- function(w)
        checkIdentical(NULL, w)
    }
    result <- svTest(function() {
      tryCatch(
        withCallingHandlers({
          eval(testCall)
          checkTrue(TRUE)
        },
        warning = function(w)
          onFailure(w)),
        error = function(w)
          onFailure(w))
    })
    attr(result, 'unit') <- 'rcheck.examples'
    result
  })
}

#' SciViews-R log management functions
#'
#' These functions define the code of test functions. They are designed to check
#' the result of some test calculation.
#'
#' @param description A (short) character string describing this test suite log.
#' @param deleteExisting Do we delete an existing a `.Log` object already
#' defined in `.GlobalEnv` (no, by default)?
#' @param stopit Do we issue an error ([stop()] in case of any error or failure?
#' This is particularly useful if you want to interrupt R CMD check on packages,
#' when you included one or more test suites in examples (see `?unitTests`).
#' @param summarize Should the summary of the log be printed in case we stop
#' execution of the code when an error is found (see `stopit =` argument. It is,
#' indeed, useful to indicate at this time which tests failed or raised an
#' error. So, this argument should usually be left at its default value.
#'
#' @return
#' [Log()] and [createLog()] return the `.Log` object defined in `.GlobalEnv` by
#' reference (it is indeed an environment). So, you can use its content (and
#' change it, if you write functions to manipulate this log).
#'
#' [clearLog()] return invisibly `TRUE` or `FALSE`, depending if an existing log
#' object was deleted or not.
#'
#' [errorLog()] is mainly used for its side-effect of stopping code execution
#' and/or printing a summary of the test runs in the context of example
#' massaging in R CMD check (see the "Writing R extensions" manual). However,
#' this function also returns invisibly a contingency table with the number of
#' successes, failures, errors and deactivated tests recorded so far.
#'
#' [lastTest()] and [lastSuite()] recall results of last test and last suite
#' run, respectively.
#'
#' @details
#' svUnit records results of assertions (using the checkxxx() functions) in a
#' 'svSuiteData' object named `.Log` and located in .GlobalEnv. Hence, this log
#' is easy to access. However, in order to avoid errors in your code in case
#' this object was deleted, or not created, it is better to access it using
#' [Log()] which take care to create the object if it is missing.
#'
#' @export
#' @importFrom utils sessionInfo str
#' @author Philippe Grosjean
#' @seealso [svSuiteData()], [svSuite()], [svTest()], [checkEquals()]
#' @keywords utilities
#' @concept unit testing
#' @examples
#' clearLog()           # Clear the svUnit log
#'
#' # Two correct tests
#' (checkTrue(1 < 2))
#' (checkException(log("a")))
#' errorLog()           # Nothing, because there is no error
#'
#' \dontrun{
#' (checkTrue(1 > 2))    # This test fails
#' lastTest()            # Print results of last test
#' errorLog()            # Stop and summarize the tests run so far
#' }
#'
#' clearLog()
Log <- function(description = NULL) {
  if (!exists(".Log", envir = .GlobalEnv, inherits = FALSE))
    createLog(description = description)
  get(".Log", envir = .GlobalEnv, inherits = FALSE)
}

#' @export
#' @rdname Log
createLog <- function(description = NULL, deleteExisting = FALSE) {
  # Create a log consisting in an environment with class svSuiteData
  if (isTRUE(deleteExisting) && exists(".Log", envir = .GlobalEnv,
    inherits = FALSE)) rm(.Log, envir = .GlobalEnv)
  if (!exists(".Log", envir = .GlobalEnv, inherits = FALSE)) {
    .Log <<- structure(new.env(parent = .GlobalEnv),
      class = c("svSuiteData", "environment"))
    # Add general informations and variables to it
    .Log$.time <- Sys.time()            # Creation time of the log
    .Log$.R.version <- R.version        # R version
    .Log$.sessionInfo <- sessionInfo()  # Information about current session
    .Log$.description <- description    # Optional description of this log
    # Create ..xxx variables used for test context
    # Note: never delete or put NULL in these variables, use "" instead
    .Log$..Unit <- ""
    .Log$..Msg <- ""
    .Log$..Obj <- ""
    .Log$..File <- ""
    .Log$..Tag <- ""
    # Create .lastTest that contains details from last check...()
    naChr <- as.character(NA)
    .Log$.lastTest <- structure(
      data.frame(msg = naChr, call = naChr,
        timing = as.numeric(NA), kind = .kind(NA), res = naChr,
        obj = naChr, file = naChr, tag = naChr,
        stringsAsFactors = FALSE),
      class = c("svTestData", "data.frame"))
    # Create .lastSuite with an empty list of test units to run
    .Log$.lastSuite <- list()
  }
}

#' @export
#' @rdname Log
clearLog <- function() {
  if (exists(".Log", envir = .GlobalEnv, inherits = FALSE)) {
    rm(list = ".Log", envir = .GlobalEnv)
    res <- TRUE
  } else {
    res <- FALSE
  }
  invisible(res)
}

#' @export
#' @rdname Log
errorLog <- function(stopit = TRUE, summarize = TRUE) {
  .Log <- Log()
  Res <- table(stats(.Log)$kind)
  if (isTRUE(stopit) && any(Res[2:3] > 0)) {
    if (isTRUE(summarize))
      summary(.Log)
    msg <- paste(Res[2], "failure(s) and", Res[3], "error(s)")
    stop(msg)
  } else if (interactive()) {
    cat("Summary statistics on all tests run:\n")
    print(Res)
  }
  invisible(Res)
}

#' @export
#' @rdname Log
lastTest <- function() {
  # Return a svTestData object with data from last recorded test
  Log()$.lastTest
}

#' @export
#' @rdname Log
lastSuite <- function() {
  # Return data about last suite run
  Log()$.lastSuite
}

#' Interact with the test unit GUI in Komodo/SciViews-K
#'
#' These functions allow controlling the test unit module (R Unit tab at right)
#' in Komodo with SciViews-K and SciViews-K Unit extensions. R must be correctly
#' connected to Komodo, meaning that the 'svGUI' package must be loaded with
#' proper configuration of client/server socket connections between R and
#' Komodo. See the manual about SciViews-K for more information. The functions
#' defined here are the same as JavaScript functions defined in the 'sv.r.unit'
#' namespace in Komodo/SciViews-K Unit. For instance, [koUnit_runTest()] is
#' equivalent to `sv.r.unit.runTest();` in a Javascript macro in Komodo.
#'
#' @param state `TRUE` or `FALSE`, or missing for `koUnit_showRUnitPane()`, in
#' this case, the R Unit pane visibility is toggled.
#'
#' @return
#' [koUnit_isAutoTest()] returns `TRUE` if the test unit is in auto mode in
#' Komodo (the selected tests are run automatically each time a .R file
#' edited in Komodo is saved).
#'
#' [koUnit_version()] returns the version for which the SciViews-K Unit
#' extension was designed for. This allow to check if this version is compatible
#' with current 'svUnit' R package version, and to propose to update the Komodo
#' extension if needed (this mechanism is not running currently, but it will be
#' implemented in the future to avoid or limit incompatibilities between
#' respective R and Komodo extensions).
#'
#' The other functions are invoked for their side-effect and they return
#' nothing. Note, however, that correct execution of this code in Komodo is
#' verified, and the functions issue an error in R if they fail to execute
#' correctly in Komodo.
#'
#' @export
#' @name koUnit
#' @author Philippe Grosjean
#' @seealso [guiTestReport()]
#' @keywords utilities
#' @concept unit testing
#' @examples
#' \dontrun{
#' # Make sure R is communicating with Komodo before use, see ?koCmd in svGUI
#' koUnit_version()
#'
#' # Toggle visibility of the R Unit pane in Komodo twice
#' koUnit_showRUnitPane()
#' koUnit_showRUnitPane()
#'
#' # Make sure that the R Unit pane is visible
#' koUnit_showRUnitPane(TRUE)
#'
#' # Is the test unit running in auto mode?
#' koUnit_isAutoTest()
#'
#' # Toggle auto test mode off
#' koUnit_setAutoTest(FALSE)
#'
#' # Run the test units from within Komodo
#' koUnit_runTest()
#' }
koUnit_setAutoTest <- function(state) {
  if (isTRUE(state)) state <- "true" else state <- "false"
  res <- .koUnit('sv.r.unit.setAutoTest(<<<data>>>);', data = state)
}

.koUnit <- function(cmd, warn = FALSE, ...) {
  # Look if koCmd() exists, otherwise, we are probably not connected to Komodo
  if (exists("koCmd", mode = "function")) {
    res <- get("koCmd")(cmd, ...)
    if (isTRUE(warn) & inherits(res, "try-error"))
      warning("Komodo is not available or did not process this command correctly")
    return(res)
  } else {
    if (isTRUE(warn))
      warning("You must establish a connection with Komodo/SciViews-K to use this function")
  }
}

#' @export
#' @rdname koUnit
koUnit_isAutoTest <- function() {
  res <- .koUnit('sv.socket.serverWrite(sv.r.unit.isAutoTest());')
  return(res == "true")
}

#' @export
#' @rdname koUnit
koUnit_runTest <- function()
  res <- .koUnit('sv.r.unit.runTest();')

#' @export
#' @rdname koUnit
koUnit_showRUnitPane <- function(state) {
  if (missing(state)) {
    state <- ""
  } else if (isTRUE(state)) {
    state <- "true"
  } else {
    state <- "false"
  }
  res <- .koUnit('sv.r.unit.showRUnitPane(<<<data>>>);', data = state)
}

#' @export
#' @rdname koUnit
koUnit_version <- function()
  .koUnit('sv.socket.serverWrite(sv.r.unit.version + "." + sv.r.unit.release);')

#' Unit tests for the package 'svUnit'
#'
#' Performs unit tests defined in this package by running
#' `example(unitTests.svUnit)`. Tests are in `runit*.R` files located in the
#' '/unitTests' subdirectory or one of its subdirectories ('/inst/unitTests' and
#' subdirectories in package sources).
#'
#' @name unitTests.svUnit
#' @author Philippe Grosjean
#' @keywords utilities
#' @examples
#' if (require(svUnit)) {
#'   clearLog()
#'   runTest(svSuite("package:svUnit"), "svUnit")
#'   \donttest{
#'   # Tests to run with example() but not with R CMD check
#'   runTest(svSuite("package:svUnit (VirtualClass)"), "VirtualClass")
#'   }
#'   \dontrun{
#'   # Tests to present in ?unitTests.svUnit but not run automatically
#'   # Run all currently loaded packages test cases and test suites
#'   runTest(svSuiteList(), "AllTests")
#'   }
#'   \dontshow{
#'   # Put here test units you want to run during R CMD check but
#'   # don't want to show or run with example(unitTests.svUnit)
#'   }
#'   # Check errors at the end (needed to interrupt R CMD check)
#'   errorLog()
#' }
NULL

.onLoad <- function(lib, pkg) { # nocov start
  # The default exclusion list, if it is not defined yet
  # Although there are unit tests defined in these packages (as examples),
  # we don't want to include them, by default, in our test suite!
  if (is.null(getOption("svUnit.excludeList")))
    options(svUnit.excludeList = c("package:sv", "package:RUnit"))
  # Look if the SciViews-K Unit Komodo extension is installed
  # TODO: this causes more problems than solutions => temporarily deactivated!
  #.installUpgradeKomodoExtension()
  # Install a callback to update the list of units automatically in the GUI
  # Use the mechanism introduced in svSocket 0.9-48 to allow execution of
  # this task callback from a socket client too
  h <- .getTemp(".svTaskCallbackManager", default = NULL)
  if (!is.null(h))
    h$add(guiSuiteAutoList, name = "guiSuiteAutoList")
} # nocov end

.onUnload <- function(libpath) { # nocov start
  # Delete the task callback
  h <- .getTemp(".svTaskCallbackManager", default = NULL)
  if (!is.null(h))
    h$remove("guiSuiteAutoList")
  # Clear the list of units in the GUI client
  if (exists("koCmd", mode = "function"))
    get("koCmd")('sv.r.unit.getRUnitList_Callback("");')
} # nocov end

.packageName <- "svUnit"

.komodoExtensionMinVersion <- "0.7.3"

.installUpgradeKomodoExtension <- function() { # nocov start
  if (!exists("koCmd", mode = "function"))
    return()
  # Look if the SciViews-K Unit Komodo extension is installed and is of the
  # right version. Otherwise, propose to install, or update it
  xpiFile <- system.file("komodo", "sciviewskunit-ko.xpi", package = "svUnit")
  # Bug: sometimes this fails, preventing svUnit to load, despite it would
  # work well past this point. So, I put this in a try() to silently catch
  # the error and continue loading svUnit anyway (thanks Claudia Beleites)
  koVersion <- try(
    get("koCmd")('sv.socket.serverWrite(sv.r.unit.version + "." + sv.r.unit.release);'),
    silent = TRUE)
  if (inherits(koVersion, "try-error"))
    return()
  if (koVersion == "undefined.undefined") {
    # We need to install the extension
    cmd <- 'var res = ko.dialogs.okCancel("The SciViews-K Unit extension is required by svUnit",'
    cmd <- paste(cmd, '"OK", "Do you want to install the SciViews-K Unit extension now in Komodo?')
    cmd <- paste(cmd, 'You will be prompted for confirmation (click \'Install Now\')')
    cmd <- paste(cmd, 'and will have to restart Komodo at the end of the installation')
    cmd <- paste(cmd, '(click \'Restart Komodo\').", "svUnit");')
    cmd <- paste(cmd, ' if (res == "OK") { ko.open.URI("<<<data>>>"); }', sep = "")
    get("koCmd")(cmd, data = xpiFile)
  } else if (.compareVersion(koVersion, .komodoExtensionMinVersion) == -1) {
    # We need to upgrade the extension
    cmd <- 'var res = ko.dialogs.okCancel("A newer SciViews-K Unit extension is required by svUnit",'
    cmd <- paste(cmd, '"OK", "Your SciViews-K Unit extension is too old for this version of svUnit.')
    cmd <- paste(cmd, 'Do you want to upgrade it now?')
    cmd <- paste(cmd, 'You will be prompted for confirmation (click \'Install Now\')')
    cmd <- paste(cmd, 'and will have to restart Komodo at the end of the installation')
    cmd <- paste(cmd, '(click \'Restart Komodo\').", "svUnit");')
    cmd <- paste(cmd, ' if (res == "OK") { ko.open.URI("<<<data>>>"); }', sep = "")
    get("koCmd")(cmd, data = xpiFile)
  }
} # nocov end

.compareVersion <- function(a, b) {
  # This is the same as utils::compareVersion(), but we don't want
  # dependencies on utils
  if (is.na(a))
    return(-1)
  if (is.na(b))
    return(1)
  a <- as.integer(strsplit(a, "[\\.-]")[[1]])
  b <- as.integer(strsplit(b, "[\\.-]")[[1]])
  for (k in 1:length(a)) {
    if (k <= length(b)) {
      if (a[k] > b[k]) {
        return(1)
      } else if (a[k] < b[k]) {
        return(-1)
      }
    } else {
      return(1)
    }
  }
  if (length(b) > length(a)) {
    return(-1)
  } else {
    return(0)
  }
}

.kindLevels <- c("OK", "**FAILS**", "**ERROR**", "DEACTIVATED")

.kind <- function(val = TRUE) {
  # TRUE or 1 -> 1 = "OK"
  # FALSE or 0 -> 2 = "**FAILS**"
  # -1 -> 3 = "**ERROR**"
  # -2 -> 4 = "DEACTIVATED"
  factor(.kindLevels[-(as.integer(val) - 2)], levels = .kindLevels)
}

.kindMax <- function(kinds) {
  # If there are no record, must be because all tests succeed!
  if (length(kinds) == 0)
    return(.kind(TRUE))
  Kinds <- as.numeric(kinds)
  if (sum(Kinds, na.rm = TRUE) == 0)
    return(.kind(NA))
  factor(.kindLevels[max(as.numeric(kinds), na.rm = TRUE)],
    levels = .kindLevels)
}

.formatTime <- function(x, secDigits = 0, minSec = 10^-secDigits,
prepend = " run in") {
  # x is given in seconds, and it returns a pretty formatted string with time
  if (is.null(x) || is.na(x)[1])
    return("")
  x <- as.numeric(x)
  Sec <- round(x %% 60, digits = secDigits)
  Min <- x %/% 60
  Hour <- Min %/% 60
  Min <- Min %% 60
  Time <- prepend
  Time <- ifelse(Hour > 0, paste(Time, Hour, "h"), Time)
  Time <- ifelse(Min > 0 | Time != prepend, paste(Time, Min, "min"), Time)
  Time <- ifelse(Sec > minSec | Time != prepend, paste(Time, Sec, "sec"), Time)
  Time <- ifelse(Time == prepend, paste(prepend, "less than", minSec, "sec"), Time)
  Time[is.na(Time)] <- ""
  Time
}
# Test: .formatTime((0:10) * 400 + 0.56)

.formatResult <- function(result, level = getOption("svUnit.strLevel")) {
  if (is.null(level)) {
    level <- 1
  } else {
    level <- as.integer(level[1])
  }
  if (level < 1)
    return("")
  # Capture the report returned by the str() function
  capture.str <- function(data, level) {
    rval <- NULL
    file <- textConnection("rval", "w", local = TRUE)
    sink(file, type = "output")
    sink(file, type = "message")
    on.exit({
      sink(type = "output")
      sink(type = "message")
      close(file)
    })
    str(data, max.level = level)
    cat("\n")
    rval
  }
  Str <- capture.str(result, level)
  paste(Str, collapse = "\n")
}

.logTest <- function(timing, test, msg = "", description = NULL) {
  .Log <- Log(description = description)
  # Determine the name of the test
  if (missing(test)) {# Is it defined globally?
    if (exists("..Test", envir = .Log, inherits = FALSE)) {
      test <- .Log$..Test
    } else {# Try to guess it from the call
      ret <- try(test <- as.character(sys.call(-2))[1], silent = TRUE)
      if (inherits(ret, "try-error") || is.na(test)) {
        # check...() probably called directly at the command line
        test <- "eval"
        # Convenient for collecting these tests together with tests run
        # inside runit*.R files (not embedded in test functions)
      } else if (test == "runTest") {
        # Special case for runTest(myTest) or runTest(test(foo))
        test <- as.character(sys.call(-2))[2]
      } else if (test == "eval.with.vis") {
        test <- "eval"
      }
    }
  }
  # Do we need to create 'test'?
  if (!exists(test, envir = .Log, inherits = FALSE)) {
    if (msg == "")
      msg <- .Log$..Msg
    .Log[[test]] <- structure(
      data.frame(
        msg = character(),
        call = character(),
        timing = numeric(),
        kind = .kind(logical()),
        res = character(),
        obj = character(),
        file = character(),
        tag = character(),
        stringsAsFactors = FALSE),
      time = Sys.time(),
      stats = c(tests = 1, timing = timing),
      context = c(unit = as.character(.Log$..Unit), test = test,
      msg = paste(msg, collapse = "\n")),
      class = c("svTestData", "data.frame"))
  } else {
    # Just update stats
    attr(.Log[[test]], "stats") <- attr(.Log[[test]], "stats") + c(1, timing)
  }
  test
}

.logTestData <- function(test, msg, call, timing, val, kind = .kind(val), res,
obj = .Log$..Obj, file = .Log$..File, tag = .Log$..Tag,
printTest = getOption("svUnit.printTest")) {
  # Add these data to .lastTest
  .Log$.lastTest <- structure(data.frame(
    msg = msg, call = call, timing = timing, kind = kind, res = res,
    obj = obj, file = file, tag = tag, stringsAsFactors = FALSE),
    class = c("svTestData", "data.frame"),
    row.names = as.character(attr(.Log[[test]], "stats")["tests"]))
  # Add them also to the log of my test
  .Log[[test]][nrow(.Log[[test]]) + 1, ] <- .Log$.lastTest
  # Do we print detailed results for this test?
  if (is.null(printTest))
    printTest <- !interactive() # Guess it from context
  if (isTRUE(printTest))
    print(.Log$.lastTest)
}

.prepareUnit <- function(name, dir) {
  # Prepare for writing a test unit file
  dir <- gsub("\\\\", "/", as.character(dir)[1])
  # Check that dir exists (do not create it!)
  if (!file.exists(dir) || !file.info(dir)$isdir)
    stop("'dir' must be an existing directory")
  # If dir is tempdir(), then, make sure there are no other runit*.R files
  # left (should not!) - One can store only one unit at a time in tempdir()!
  if (dir == gsub("\\\\", "/", tempdir())) {
    runitfiles <- list.files(dir, pattern = "^runit.*\\.[r|R]$",
      full.names = TRUE)
    unlink(runitfiles)
  }
  Unit <- file.path(dir, paste("runit", name, ".R", sep = ""))
  cat("## Test unit '", name, "'\n", sep = "", file = Unit)
  Unit
}

.writeSetUp <- function(unit, file = "", msg = "", tag = "", code = NULL) {
  # Write the .setUp() function in the test unit file
  # Here, we write a context to localize tested objects and test unit files
  catUnit <- function(...)
    cat(..., sep = "", file = unit, append = TRUE)
  catUnit('\n.setUp <-\n')
  catUnit('function () {\n')
  catUnit('\t## Specific actions for svUnit: prepare context\n')
  catUnit('\tif ("package:svUnit" %in% search()) {\n')
  catUnit('\t\t.Log <- Log() ## Make sure .Log is created\n')
  catUnit('\t\t.Log$..Unit <- "', unit, '"\n')
  catUnit('\t\t.Log$..File <- "', as.character(file)[1], '"\n')
  catUnit('\t\t.Log$..Obj <- ""\n')
  catUnit('\t\t.Log$..Tag <- "', tag, '"\n')
  catUnit('\t\t.Log$..Msg <- "', paste(msg, collapse = "\n"), '"\n')
  catUnit('\t\trm(..Test, envir = .Log)\n')
  catUnit('\t}\n')
  if (!is.null(code))
    catUnit(paste("\t", code, collapse = "\n"))
  catUnit('}\n')
}

.writeTearDown <- function(unit, code = NULL, rm.unit = TRUE, rm.file = TRUE) {
  # Write the .tearDown() function in the test unit file
  # Here, we undo what was done in .setUp()
  catUnit <- function(...)
    cat(..., sep = "", file = unit, append = TRUE)
  catUnit('\n.tearDown <-\n')
  catUnit('function () {\n')
  if (!is.null(code))
    catUnit(paste("\t", code, collapse = "\n"))
  catUnit('\t## Specific actions for svUnit: clean up context\n')
  catUnit('\tif ("package:svUnit" %in% search()) {\n')
  catUnit('\t\t.Log$..Unit <- ""\n')
  catUnit('\t\t.Log$..File <- ""\n')
  catUnit('\t\t.Log$..Obj <- ""\n')
  catUnit('\t\t.Log$..Tag <- ""\n')
  catUnit('\t\t.Log$..Msg <- ""\n')
  catUnit('\t\trm(..Test, envir = .Log)\n')
  catUnit('\t}\n')
  catUnit('}\n')
}

.writeTest <- function(unit, objname, pos = .GlobalEnv, obj = NULL) {
  # Make sure that the name of a test function is syntactically correct
  # and starts with 'test'
  if (regexpr("^test", objname) > -1) {
    testname <- objname
  } else {
    testname <- paste("test", objname, sep = "")
  }
  # Write the first line in the file
  catUnit <- function(...)
    cat(..., file = unit, append = TRUE)
  catUnit('\n"', testname, '" <-\n', sep = "")
  # Get the object
  if (missing(obj)) {
    # Look for 'objname' in 'pos'
    if (!exists(objname, where = pos)) {
      Test <- ""
    } else {
      Test <- test(get(objname, pos = pos))
    }
  } else {
    Test <- test(obj)
  }
  if (is.character(Test)) {
    # Create a dummy test with DEACTIVATED entry indicating missing object
    body <- c(
      'function() {',
      paste('\tDEACTIVATED("Object', objname, 'not found!")'),
      '}\n')
  } else if (is.null(Test)) {
    # Create a dummy test with DEACTIVATED entry indicating missing test
    body <- c(
      'function() {',
      paste('\tDEACTIVATED("Object', objname, 'has no tests!")'),
      '}\n')
  } else {
    # Get the body of the test function
    capture.body <- function(Data) {
      rval <- NULL
      File <- textConnection("rval", "w", local = TRUE)
      sink(File)
      on.exit({
        sink()
        close(File)
      })
      dput(Data, file = File, control = "useSource")
      on.exit()
      sink()
      close(File)
      rval
    }
    body <- capture.body(Test)
  }
  # Write the body of the test function in the file
  catUnit(body, sep = "\n")
}

.runTest <- function(x, envir, test, objfile = "", unit = "", tag = "",
msg = "") {
  # Run one test in a protected environment catching errors and warnings
  # and preparing a suitable context
  name <- sub("^test\\.(.+)\\.$", "\\1", test)

  # The context is written in the .Log, but previous context is saved
  # and restored on return
  .Log <- Log()  # Make sure that .Log exists, or create a new one
  oContext <- c(Unit = .Log$..Unit, Obj = .Log$..Obj, File = .Log$..File,
    Msg = .Log$..Msg, Tag = .Log$..Tag)
  on.exit({
    .Log$..Unit <- as.character(oContext[1])
    .Log$..Obj <- as.character(oContext[2])
    .Log$..File <- as.character(oContext[3])
    .Log$..Msg <- as.character(oContext[4])
    .Log$..Tag <- as.character(oContext[5])
  })
  .Log$..Unit <- unit          # The unit file
  .Log$..Obj <- name           # Name of the tested object
  .Log$..File <- objfile       # Where is the code source of 'name'?
  .Log$..Msg <- paste(msg, collapse = "\n") # Message for this test
  .Log$..Tag <- tag            # A tag in objfile to locate code
  # Define the test and save possible existing definition to restore it
  if (exists("..Test", envir = .Log, inherits = FALSE)) {
    oTest <- .Log$..Test
    on.exit(.Log$..Test <- oTest, add = TRUE)
  } else on.exit(rm("..Test", envir = .Log), add = TRUE)
  .Log$..Test <- test          # Define the name of the test

  if (missing(envir)) {
    # The environment where the test is run
    envir <- new.env(parent = .GlobalEnv)
    envir[[test]] <- x                # A copy of the test code
    envir$.setUp <- function() {}     # Fake .setUp
    envir$.tearDown <- function() {}  # Fake .tearDown
  }
  # We need this installed in our sandbox .TestEnv to run the test
  envir$.LogWarnings <- list()        # A list to collect warnings

  # Clear the corresponding log, if it exists
  if (exists(test, envir = .Log, inherits = FALSE))
    rm(list = test, envir = .Log)

  # Evaluate the test function in .testEnv, catching errors
  owarn <- getOption("warn")
  on.exit(options(warn = owarn), add = TRUE)
  if (isTRUE(getOption("svUnit.StopOnWarning"))) {
    nwarn <- 2
  } else {
    nwarn <- -1
  }
  options(warn = nwarn)

  # Evaluate the test function in the .TestEnv environment
  cmd <- paste("evalq(.LogRes <- try( { .setUp(); `", test,
    "`(); .tearDown() }, silent = TRUE), envir = envir)", sep = "")
  eval(parse(text = cmd))

  # Analyze error => is it a deactivation or error in the code?
  Res <- envir$.LogRes
  if (inherits(Res, "try-error")) {
    # We record this as a test returning **ERROR** or DEACTIVATED
    .logTest(0, test)
    # Did we encountered a DEACTIVATED() command or something else?
    if (regexpr("DEACTIVATED\\(", Res) > -1) {
      Msg <- sub("^[^:]+: *", "", as.character(Res))
      Msg <- sub("\n$", "", Msg)
      .logTestData(test, msg = Msg, call = "", timing = NA,
        val = -2, res = "\n")
    } else {
      # This is an error (something wrong with the code in the test fun)
      .logTestData(test, msg = "", call = deparse(sys.call()),
        timing = NA, val = -1, res = paste(Res, collapse = "\n"))
    }
  }
  test
}

.assignTemp <- function(x, value)
    assign(x, value, envir = .TempEnv())

.getTemp <- function(x, default = character(0)) {
  if  (exists(x, envir = .TempEnv(), inherits = FALSE)) {
    return(get(x, envir = .TempEnv(), inherits = FALSE))
  } else {# Variable not found, return the default value
    return(default)
  }
}

.TempEnv <- function() {
  pos <-  match("SciViews:TempEnv", search())
  if (is.na(pos)) {# Must create it
    `SciViews:TempEnv` <- list()
    Attach <- function(...)
      get("attach", mode = "function")(...)
    Attach(`SciViews:TempEnv`, pos = length(search()) - 1)
    rm(`SciViews:TempEnv`)
    pos <- match("SciViews:TempEnv", search())
  }
  pos.to.env(pos)
}

#' Create and run test suites by collecting together unit tests and function
#' tests defined in objects
#'
#' A 'svSuite' object is essentially a list of test units directories (or
#' packages, in this case, corresponding directories are PKG/unitTests and its
#' subdirectories), and of object names containing tests to add temporarily to
#' the test suite. These must be formatted in a concise way as described for the
#' 'tests' argument.
#'
#' [svSuiteList()] lists all loaded packages having /unitTests/runit*.R files
#' (or similar files in subdirectories), and all objects in the user workspace
#' that have a 'test' attribute, or are 'svTest' objects (by default). It is a
#' rather exhaustive list of all test items currently available in the current R
#' session, but restricted by `getOption("svUnit.excludeList")`.
#'
#' [makeUnit()] writes a test unit on disk with the tests from the objects
#' listed in the 'svSuite' object that do not belong yet to a test unit.
#' [runTest()] runs all the test in packages, directories and objects listed in
#' the 'svSuite' object.
#'
#' @param tests A character string with items to include in the test suite. It
#' could be 'package:PKG' for including test units located in the /unitTests
#' subdirectory of the package PGK, or 'package:PKG (SUITE)' for test units
#' located in the subdirectory /unitTests/SUITE of package PKG, or 'dir:MYDIR'
#' for including test units in MYDIR, or 'test(OBJ)' for tests embedded in an
#' object, or 'OBJ' for 'svTest' object directly.
#' @param x Any kind of object.
#' @param packages Do we list test units available in loaded packages?
#' Alternatively one can provide a character vector of package names, and it
#' will be used to filter packages (take care: in this case it will look at
#' installed packages, not only loaded packages)!
#' @param objects Do we list test available in objects? Alternatively, one can
#' provide a character vector of object names, and it will filter objects in
#' 'pos' according to this vector.
#' @param dirs An additional list of directories where to look for more test
#' units. For convenience, this list can simply be saved as an 'svUnit.dirs'
#' options.
#' @param excludeList A list of items to exclude from the listing. The function
#' uses regular expression to match the exclusions. So, for instance, specifying
#' `"package:MYPKG"` will exclude all items from package 'MYPKG', while using
#' `"package:MYPKG$"` will exclude only tests suites defined in the
#' .../MYPKG/unitTests directory, bur not in its subdirectories. For
#' convenience, it can be saved in a 'svUnit.excludeList' option. By default,
#' all tests for packages whose name start with 'sv' or 'RUnit' are excluded,
#' that is, `c("package:sv", "package:RUnit")`.
#' @param pos The environment to look for 'objects' (environment, character
#' string with name of an environment, or integer with position of the
#' environment in the search path.
#' @param loadPackages In the case a list of packages is provided in
#' `packages =`, do we make sure that these packages are loaded? If yes, the
#' function will try to load all packages in that list that are not loaded yet
#' and will issue a warning for the packages not found. Default, `FALSE`.
#' @param name The name of the test suite to build.
#' @param dir The directory where to create the test unit file.
#' @param objfile The path to the file containing the original source code of
#' the object being tested. This argument is used to bring a context for a
#' test and allow a GUI to automatically open the source file for edition when
#' the user clicks on a test that failed or raised an error.
#' @param codeSetUp An expression with some code you want to add to the
#' `.setUp()` function in your unit file (this function is executed before each
#' test.
#' @param codeTearDown An expression with some code you want to add to the
#' `.tearDown()` function in your unit file (this function is executed after
#' each test.
#' @param unitname The name of a unit to run inside the suite. If `NULL` (by
#' default), all units are run.
#' @param ... Further arguments to pass to [makeUnit()] or [runTest()] (not used
#' yet).
#'
#' @return
#' [svSuite()], [as.svSuite()] and [svSuiteList()] return a 'svSuite' object.
#' [is.svSuite()] returns `TRUE` if the object is an 'svSuite'.
#'
#' [makeUnit()] creates a test unit file on disk, and [runTest()] runs the tests
#' in such a file. They are used for their side-effect, but the first one also
#' returns the file created, and the second one returns invisibly the list of
#' all test unit files that where sourced ans run.
#'
#' @details
#' Thanks to the variety of sources allowed for tests, it is possible to define
#' these tests in a structured way, inside packages, like for the 'RUnit'
#' package (but with automatic recognition of test units associated to packages,
#' in the present case). It is also easy to define tests more loosely by just
#' attaching those tests to the objects you want to check. Whenever there objects
#' are loaded in the user's workspace, their tests are available. In both cases,
#' a test unit file on disk is sourced in a local environment and test functions
#' are run (same approach as in the 'RUnit' package, and the same test unit files
#' should be compatibles with both 'RUnit' and 'svUnit' packages), but in the
#' case of a loosely definition of the tests by attachment to objects, the test
#' unit file is created on the fly in the temporary directory (by default).
#'
#' At any time, you can transform a series of tests loosely attached to objects
#' into a test unit file by applying [makeUnit()] to a 'svSuite' object,
#' probably specifying another directory than the (default) temporary dir for
#' more permanent storage of your test unit file. The best choice is the
#' '/inst/unitTests' directory of a package source, or one of its
#' subdirectories. That way, your test unit file(s) will be automatically listed
#' and available each time you load the compiled package in R (if you list them
#' using [svSuiteList()]). Of course, you still can exclude tests from given
#' packages by adding 'package:PKG' in the exclusion list with something
#' like: `options(svUnit.excludeList = c(getOption("svUnit.excludeList"), "package:PKG"))`.
#'
#' @export
#' @author Philippe Grosjean
#' @seealso [svSuiteData()], [svTest()], [Log()], [checkEquals()], [RUnit::checkEquals()]
#' @keywords utilities
#' @concept unit testing
#' @examples
#' svSuiteList() # List all currently available test units and test cases
#' # Exclusion list is used (regular expression filtering!). It contains:
#' (oex <- getOption("svUnit.excludeList"))
#' # Clear it, and relist available test units
#' options(svUnit.excludeList = NULL)
#' svSuiteList()
#'
#' # Two functions that include their test cases
#' Square <- function(x)
#'   return(x^2)
#' test(Square) <- function() {
#'   checkEquals(9, Square(3))
#'   checkEquals(c(1, 4, 9), Square(1:3))
#'   checkException(Square("xx"))
#' }
#'
#' Cube <- function(x)
#'   return(x^3)
#' test(Cube) <- function() {
#'   checkEquals(27, Cube(3))
#'   checkEquals(c(1, 8, 28), Cube(1:3))
#'   checkException(Cube("xx"))
#' }
#'
#' # A separate test case object (not attached to a particular object)
#' # This is the simplest way to define quick and durty integration tests
#' test_Integrate <- svTest(function() {
#'   checkTrue(1 < 2, "check1")
#'   v <- 1:3 	# The reference
#'   w <- 1:3 	# The value to compare to the reference
#'   checkEquals(v, w)
#' })
#'
#' # A function without test cases (will be filtered out of the suite list)
#' foo <- function(x)
#'   return(x)
#'
#' # Look now which tests are available
#' svSuiteList()
#'
#' # Only objects, no package units
#' svSuiteList(packages = FALSE)
#'
#' \dontrun{
#' # Create the test unit file for all objects with tests in .GlobalEnv
#' myunit <- makeUnit(svSuiteList(), name = "AllTests")
#' file.show(myunit, delete.file = TRUE)
#' }
#'
#' # Filter objects using a list (object with/without tests and a nonexisting obj)
#' svSuiteList(packages = FALSE, objects = c("Cube", "foo", "bar"))
#'
#' # Create another svSuite object with selected test items
#' (mysuite <- svSuite(c("package:svUnit (VirtualClass)", "test(Cube)")))
#' is.svSuite(mysuite)	# Should be!
#'
#' \dontrun{
#' # Run all the tests currently available
#' (runTest(svSuiteList(), name = "AllTests"))
#' summary(Log())
#' }
#'
#' # Restore previous exclusion list, and clean up the environment
#' options(svUnit.excludeList = oex)
#' rm(Square, Cube, foo, test_Integrate, mysuite, myunit, oex)
svSuite <- function(tests) {
  # Check provided tests and build a 'svSuite' object
  tests <- as.character(tests)
  # Remove NAs and empty strings ("") from tests
  tests <- tests[!is.na(tests) & !(tests == "")]
  if (length(tests) > 0) {
    # Tests must be character strings like:
    # * package:PKG
    # * package:PKG (TESTSUITE)
    # * dir:MYDIR
    ## * test(OBJ) where OBJ is any object with a 'test' attribute
    # * OBJ being a 'svTest' object (with non "exotic" name!),
    # Syntax is checked, but not existence/validity of corresponding tests!
    check1 <- (regexpr("^package:[a-zA-Z][a-zA-Z._0-9]*$", tests) > -1)
    check2 <- (regexpr("^package:[a-zA-Z][a-zA-Z._0-9]* *\\(.+\\)$",
      tests) > -1)
    check3 <- (regexpr("^dir:.+", tests) > -1)
    check4 <- (regexpr("^test\\(.+\\)$", tests) > -1)
    check5 <- (regexpr("^[a-zA-Z0-9_.]+$", tests) > -1)
    wrong <- ((check1 + check2 + check3 + check4 + check5) == 0)
    if (any(wrong))
      stop("Wrong 'tests' data: must be 'package:PKG', 'package:PKG (SUITE)',\n'dir:MYDIR', 'test(OBJ)' or 'OBJ'")
  }
  # This is a 'svSuite' object subclassing 'character'
  class(tests) <- c("svSuite", "character")
  tests
}

#' @export
#' @rdname svSuite
as.svSuite <- function(x)
  svSuite(x)

#' @export
#' @rdname svSuite
is.svSuite <- function(x)
  inherits(x, "svSuite")

#' @export
#' @rdname svSuite
print.svSuite <- function(x, ...) {
  if (!is.svSuite(x))
    stop("'x' must be a 'svSuite' object")
  if (length(x) < 1) {
    cat("An empty svUnit test suite\n")
  } else {
    cat("A svUnit test suite definition with:\n")
    # Separate unit tests from tests embedded in objects
    isSuite <- regexpr("^[package:|dir:]", x) > -1
    if (any(isSuite)) {
      Suites <- x[isSuite]
      msg <- ifelse(length(Suites) == 1, "\n- Test suite:\n",
        "\n- Test suites:\n")
      cat(msg)
      print(Suites)
    }

    if (any(!isSuite)) {
      Objs <- x[!isSuite]
      msg <- ifelse(length(Objs) == 1, "\n- Test function:\n",
        "\n- Test functions:\n")
      cat(msg)
      print(Objs)
    }
  }
  invisible(x)
}

#' @export
#' @rdname svSuite
svSuiteList <- function(packages = TRUE, objects = TRUE,
dirs = getOption("svUnit.dirs"), excludeList = getOption("svUnit.excludeList"),
pos = .GlobalEnv, loadPackages = FALSE) {
  # List unit test (1) in loaded packages (2) in objects in pos and (3) in
  # directories, possibly filtering them using an exclusion list
  # Note: Komodo should list test unit files in loaded projects too!
  if (length(packages) < 1)
    stop("'package' cannot have zero length")
  if (length(objects) < 1)
    stop("'objects' cannot have zero length")

  items <- character()

  # 1) Unit test files in loaded packages
  if (packages[1] != FALSE) {
    if (is.character(packages)) {# We assume it is a list of packages
      Pkgs <- packages
    } else {# We use the list of all loaded packages
      Pkgs <- .packages()
    }
    for (Pkg in Pkgs) {
      # Look for test units in the package
      path <- system.file(package = Pkg, "unitTests")
      if (path != "" && file.info(path)$isdir) {
        pkgname <- paste("package", Pkg, sep = ":")
        items <- c(items, pkgname)
        Files <- list.files(path = path, full.names = TRUE)
        for (File in Files) {# Add all subdirectories too
          if (file.info(File)$isdir)
            items <- c(items, paste(pkgname, " (", basename(File),
              ")", sep = ""))
        }
      }
    }
  }

  # 2) Tests embedded in objects located in 'pos' environment
  if (objects[1] != FALSE) {
    envir = as.environment(pos)
    if (is.character(objects)) {
      tests <- character()
      for (Oname in objects) {
        if (exists(Oname, envir = envir, inherits = FALSE)) {
          Obj <- get(Oname, envir = envir, inherits = FALSE)
          if (is.svTest(Obj)) {
            tests <- c(tests, Oname)
          } else if (is.test(Obj)) {
            tests <- c(tests, paste("test(", Oname, ")", sep = ""))
          }
        }
      }
    } else {# We list all objects in pos
      Objs <- mget(ls(envir = envir), envir = envir)
      Onames <- names(Objs)
      tests <- character()
      if (length(Objs) > 0) {
        for (i in 1:length(Objs)) {
          if (is.svTest(Objs[[i]])) {
            tests <- c(tests, Onames[i])
          } else if (is.test(Objs[[i]])) {
            tests <- c(tests, paste("test(", Onames[i], ")", sep = ""))
          }
        }
      }
    }
    items <- c(items, sort(tests))
  }

  # 3) Additional directories (check that they are valid and existing dirs)
  if (!is.null(dirs)) {
    # Check if each entry exists as a directory, exclude it if not
    # Prepend "dir:" to tag them as additional directories
    Dirs <- character()
    for (Dir in dirs)
      if (file.exists(Dir) && file.info(Dir)$isdir)
        Dirs <- c(Dirs, paste("dir", Dir, sep = ":"))
    items <- c(items, sort(Dirs))
  }

  # Filter the resulting list with 'excludeList'
  if (!is.null(excludeList)) {
    for (pattern in excludeList)
      items <- items[regexpr(pattern, items) == -1]
  }

  # Do we load the package?
  if (loadPackages) {
    # Get a list of packages we need for the suite
    Pkgs <- items[regexpr("^package:", items)]
    PkgsSrch <- unique(sub(" +\\(.+$", "", Pkgs))
    l <- length(PkgsSrch)
    if (l > 0) {
      PkgsName <- sub("^package:", "", PkgsSrch)
      Search <- search()
      for (i in 1:l) {
        if (!PkgsSrch[i] %in% Search) {
          res <- try(library(PkgsName[i], character.only = TRUE),
            silent = TRUE)
          if (inherits(res, "try-error"))
            warning("Cannot load package '", PkgsName[i], "'")
        }
      }
    }
  }

  # Make it a 'svSuite' object subclassing 'character'
  class(items) <- c("svSuite", "character")
  items
}

#' @export
#' @rdname svSuite
makeUnit.svSuite <- function(x, name = make.names(deparse(substitute(x))),
dir = tempdir(), objfile = "", codeSetUp = NULL, codeTearDown = NULL,
pos = .GlobalEnv, ...) {
  # Take an 'svSuite' object and make a unit from its function tests
  # that are not written yet in a test unit in a file
  # They are saved in a file named runit<name>.R in 'dir'
  if (!is.svSuite(x))
    stop("'x' must be a 'svSuite' object")
  name <- as.character(name)[1]
  # Under Windows, we transform \\ into /
  dir <- gsub("\\\\", "/", as.character(dir)[1])
  # Collect all items that are not 'package:...' or 'dir:...'
  isObj <- regexpr("^[package:|dir:]", x) == -1
  Objs <- sub("^test[(](.+)[)]$", "\\1", x[isObj])
  if (length(Objs) == 0) {# No objects, return NULL
    return(NULL)
  } else {# Make a sourceable test unit file with tests collected in Objs
    Unit <- .prepareUnit(name, dir)
    .writeSetUp(unit = Unit, file = objfile, code = codeSetUp)
    .writeTearDown(unit = Unit, code = codeTearDown)
    for (objname in Objs)
      .writeTest(unit = Unit, objname = objname, pos = pos)
  }
  Unit
}

#' @export
#' @rdname svSuite
runTest.svSuite <- function(x, name = make.names(deparse(substitute(x))),
unitname = NULL, ...) {
  # Compile and run the test for this 'svSuite' object
  if (!is.svSuite(x))
    stop("'x' must be a 'svSuite' object")
  name <- as.character(name[1])

  # Decode tests contained in x
  tests <- as.character(x)
  dirs <- character()
  # Package suites...
  isPkg <- regexpr("^package:", tests) > -1
  if (any(isPkg)) {
    Pkgs <- tests[isPkg]
    Subdirs <- sub("^.+[(](.+)[)] *$", "\\1", Pkgs)
    Subdirs[Subdirs == Pkgs] <- ""
    Pkgs <- sub("^package:([^ ]+).*$", "\\1", Pkgs)
    for (i in 1:length(Pkgs)) {
      if (Subdirs[i] == "") {
      dir <- system.file(package = Pkgs[i], "unitTests")
    } else {
      dir <- system.file(package = Pkgs[i], "unitTests", Subdirs[i])
    }
      if (dir != "")
        dirs <- c(dirs, dir)
    }
  }

  # Add directories, and possibly make a temporary unit for test objects
  if (any(!isPkg)) {
    tests <- tests[!isPkg]
    # Directories
    isDir <- regexpr("^dir:", tests) > -1
    if (any(isDir))
      dirs <- c(sub("^dir:", "", tests[isDir]), dirs)
    # Objects
    if (any(!isDir)) {
      # Make a temporary unit for the tests of these objects
      if (!is.null(Unit <- makeUnit(x, name = name))) {
        # Add this path to dirs
        dirs <- c(dirname(Unit), dirs)
      }
    }
  }

  # Now, list all files in these dirs with name being runit*.R
  files <- character()
  for (dir in dirs)
    files <- c(files, list.files(dir, pattern = "^runit.+\\.[rR]$",
      full.names = TRUE))
  if (length(files) == 0) return(NULL)  # Nothing to run!
  # Under Windows, transform all \\ into / in the file names
  files <- gsub("\\\\", "/", files)
  # Added by Thomas Wurtzler to control which unit test to run
  if (!is.null(unitname)) {
    unitname <- deparse(substitute(unitname))
    testNames <- gsub("^.*runit(.+)\\.[rR]$", "\\1", files)
    keep <- which(testNames == unitname)
    files <- files[keep]
    if (length(files) == 0) {
      warning("Test unit ", unitname, " not found")
      return(NULL)
    }
  }
  # Run this test suite now, that is, source each file in .TestSuiteEnv
  # and run each testxxx function in it, using .setUp and .tearDown too
  # Record the list of tests
  .lastSuite <- list()
  for (file in files)
    .lastSuite[[basename(file)]] <- list(file = file)
  .Log <- Log()
  .Log$.lastSuite <- .lastSuite

  # Source each runit*.R file in turn
  for (unit in names(.lastSuite)) {
    # Create a new environment for this suite.
    .ThisTestSuiteEnv <- new.env(parent = .GlobalEnv)
    # Store it in SciViews:TempEnv so that we can inspect it in case of
    # stop on error. But please do not remove the local alias.  #1327
    .assignTemp(".TestSuiteEnv", .ThisTestSuiteEnv)
    # Source the corresponding file
    Unit <- .lastSuite[[unit]]$file
    sys.source(Unit, envir = .ThisTestSuiteEnv)
    # Make sure there are .setUp() and .tearDown() functions
    if (!exists(".setUp", envir = .ThisTestSuiteEnv, mode = "function",
      inherits = FALSE))
      .ThisTestSuiteEnv$.setUp <- function() {}
    if (!exists(".tearDown", envir = .ThisTestSuiteEnv, mode = "function",
      inherits = FALSE))
      .ThisTestSuiteEnv$.tearDown <- function() {}
    # List all test files in the unit
    tests <- ls(.ThisTestSuiteEnv, pattern = "^test.+$")
    # Keep only 'test*' objects that are function
    keep <- unlist(lapply(tests, function(n) exists(n,
      envir = .ThisTestSuiteEnv, mode = "function", inherits = FALSE)))
    tests <- tests[keep]
    .Log$.lastSuite[[unit]]$tests <- tests
    # Run each test in turn
    for (test in tests)
      .runTest(envir = .ThisTestSuiteEnv, test = test, unit = Unit)
  }
  invisible(files)
}

#' SciViews-R Unit assertions (check functions)
#'
#' These functions define the assertions in test functions. They are designed to
#' check the result of some test calculation.
#'
#' @param target a target object as reference for comparison.
#' @param current An object created for comparison (not an S4 class object).
#' @param msg An optional (short!) message to document a test. This message is
#' stored in the log and printed in front of each test report.
#' @param tolerance numeric >= 0. A numeric check does not fail if differences
#' are smaller than 'tolerance'.
#' @param checkNames Flag, if `FALSE` the names attributes are set to `NULL` for
#' both `current` and `target` before performing the check.
#' @param expr Syntactically valid R expression which can be evaluated and must
#' return a logical vector (`TRUE`|`FALSE`). A named expression is also allowed
#' but the name is disregarded. In [checkException()], expr is supposed to
#' generate an error to pass the test.
#' @param silent Flag passed on to try, which determines if the error message
#' generated by the checked function is displayed at the R console. By default,
#' it is `FALSE`.
#' @param ... Optional arguments passed to [all.equal()] or
#' [all.equal.numeric()].
#'
#' @return
#' These function return `TRUE` if the test succeeds, `FALSE` if it fails,
#' possibly with a 'result' attribute containing more information about the
#' problem. This is very different from corresponding functions in 'RUnit' that
#' stop with an error in case of test failure. Consequently, current functions
#' do not require the complex evaluation framework designed in 'RUnit' for that
#' reason.
#'
#' @details
#' These check functions are equivalent to various methods of the class
#' junit.framework.Assert of Java junit framework. They should be
#' code-compatible with functions of same name in 'RUnit' 0.4.17, except
#' for [checkTrue()] that is vectorized here, but accept only a scalar result in
#' 'RUnit'. For scalar test, the behavior of the function is the same in both
#' packages.
#' See [svTest()] for examples of use of these functions in
#' actual test cases attached to R objects.
#' See also the note about S4 objects in the [RUnit::checkTrue()] online help of the
#' 'RUnit' package.
#'
#' @export
#' @name check
#' @author Written by Ph. Grosjean, inspired from the general design of the
#' 'RUnit' package by Thomas Konig, Klaus Junemann & Matthias Burger.
#' @seealso [svTest()], [Log()], [guiTestReport()], [RUnit::checkTrue]
#' @keywords utilities
#' @concept unit testing
#' @examples
#' clearLog()     # Clear the svUnit log
#'
#' # All these tests are correct
#' (checkEquals(c("A", "B", "C"), LETTERS[1:3]))
#' (checkEqualsNumeric(1:10, seq(1, 10)))
#' (checkIdentical(iris[1:50, ], iris[iris$Species == "setosa",]))
#' (checkTrue(1 < 2))
#' (checkException(log("a")))
#' Log()    # See what's recorded in the log
#'
#' # ... but these ones fail
#' (checkEquals("A", LETTERS[1:3]))
#' (checkEqualsNumeric(2:11, seq(1, 10)))
#' (checkIdentical(iris[1:49, ], iris[iris$Species == "setosa",]))
#' (checkTrue(1 > 2))
#' (checkException(log(1)))
#' Log()    # See what's recorded in the log
#'
#' # Create a test function and run it
#' foo <- function(x, y = 2)
#'   return(x * y)
#' test(foo) <- function() {
#'   #DEACTIVATED()
#'   checkEqualsNumeric(5, foo(2))
#'   checkEqualsNumeric(6, foo(2, 3))
#'   checkTrue(is.test(foo))
#'   checkTrue(is.test(test(foo)))
#'   checkIdentical(test(foo), attr(foo, "test"))
#'   checkException(foo("b"))
#'   checkException(foo(2, "a"))
#' }
#' (runTest(foo))
#'
#' # Of course, everything is recorded in the log
#' Log()
#'
#' clearLog()
checkEquals <- function(target, current, msg = "",
tolerance = .Machine$double.eps^0.5, checkNames = TRUE, ...) {
  val <- FALSE
  timing <- as.numeric(system.time({
    ret <- try({
      # Run the test
      if (isTRUE(checkNames)) {
        cn <- ""	# Since this is the default value
      } else {
        cn <- ", checkNames = FALSE"
        names(target) <- NULL
        names(current) <- NULL
      }
      if (!is.numeric(tolerance))
        stop("tolerance has to be a numeric value")
      if (length(tolerance) != 1)
        stop("tolerance has to be a scalar")
      res <- all.equal(target, current, tolerance = tolerance, ...)
      val <- isTRUE(res)
    }, silent = TRUE)
  }, gcFirst = FALSE)[3])

  # Log this test
  test <- .logTest(timing)

  # Decide if recording more info or not
  minTiming <- getOption("svUnit.minTiming")
  if (is.null(minTiming))
    minTiming <- 0.1
  if (!isTRUE(getOption("svUnit.recordAll"))  && isTRUE(timing < minTiming)
    && val)
    return(invisible(TRUE))

  # Check for error
  if (inherits(ret, "try-error")) {
    val <- NA
    .logTestData(test, msg = msg, call =
    deparse(sys.call()[1:3], nlines = 1), timing = timing, val = -1,
      res = as.character(ret))
  } else {
    .logTestData(test, msg = msg, call =
      deparse(sys.call()[1:3], nlines = 1), timing = timing, val = val,
      res = if (val) "" else paste(c(res, .formatResult(current)),
      collapse = "\n"))
  }
  invisible(val)
}

#' @export
#' @rdname check
checkEqualsNumeric <- function(target, current, msg = "",
tolerance = .Machine$double.eps^0.5, ...) {
  val <- FALSE
  timing <- as.numeric(system.time({
    ret <- try({
      # Run the test
      if (!is.numeric(tolerance))
        stop("tolerance has to be a numeric value")
      if (length(tolerance) != 1)
        stop("tolerance has to be a scalar")
      res <- all.equal.numeric(as.vector(target), as.vector(current),
        tolerance = tolerance, ...)
      val <- isTRUE(res)
    }, silent = TRUE)
  }, gcFirst = FALSE)[3])

  # Log this test
  test <- .logTest(timing)

  # Decide if recording more info or not
  minTiming <- getOption("svUnit.minTiming")
  if (is.null(minTiming))
    minTiming <- 0.1
  if (!isTRUE(getOption("svUnit.recordAll"))  && isTRUE(timing < minTiming)
    && val)
    return(invisible(TRUE))

  # Check for error
  if (inherits(ret, "try-error")) {
    val <- NA
    .logTestData(test, msg = msg, call =
      deparse(sys.call()[1:3], nlines = 1), timing = timing, val = -1,
      res = as.character(ret))
  } else {
    .logTestData(test, msg = msg, call =
      deparse(sys.call()[1:3], nlines = 1), timing = timing, val = val,
      res = if (val) "" else paste(c(res, .formatResult(current)),
      collapse = "\n"))
  }
  invisible(val)
}

#' @export
#' @rdname check
checkIdentical <- function(target, current, msg = "") {
  val <- FALSE
  timing <- as.numeric(system.time({
    ret <- try({
      # Run the test
      val <- identical(target, current)
    }, silent = TRUE)
  }, gcFirst = FALSE)[3])

  # Log this test
  test <- .logTest(timing)

  # Decide if recording more info or not
  minTiming <- getOption("svUnit.minTiming")
  if (is.null(minTiming))
    minTiming <- 0.1
  if (!isTRUE(getOption("svUnit.recordAll"))  && isTRUE(timing < minTiming)
    && val)
    return(invisible(TRUE))

  # Check for error
  if (inherits(ret, "try-error")) {
    val <- NA
    .logTestData(test, msg = msg, call =
      deparse(sys.call()[1:3], nlines = 1), timing = timing, val = -1,
      res = as.character(ret))
  } else {
    .logTestData(test, msg = msg, call =
      deparse(sys.call()[1:3], nlines = 1), timing = timing, val = val,
      res = .formatResult(current))
  }
  invisible(val)
}

#' @export
#' @rdname check
checkTrue <- function(expr, msg = "") {
  val <- FALSE
  timing <- as.numeric(system.time({
    ret <- try({
      # Run the test
      val <- isTRUE(all(expr == TRUE))
    }, silent = TRUE)
  }, gcFirst = FALSE)[3])

  # Log this test
  test <- .logTest(timing)

  # Decide if recording more info or not
  minTiming <- getOption("svUnit.minTiming")
  if (is.null(minTiming))
    minTiming <- 0.1
  if (!isTRUE(getOption("svUnit.recordAll"))  && isTRUE(timing < minTiming)
    && val)
    return(invisible(TRUE))

  # Get call, without msg
  call <- sys.call()
  call <- deparse(call[names(call) != "msg"])

  # Check for error
  if (inherits(ret, "try-error")) {
    val <- NA
    .logTestData(test, msg = msg, call =
      deparse(sys.call()[1:2], nlines = 1), timing = timing, val = -1,
      res = as.character(ret))
  } else {
    .logTestData(test, msg = msg, call =
      deparse(sys.call()[1:2], nlines = 1), timing = timing, val = val,
      res = .formatResult(expr))
  }
  invisible(val)
}

#' @export
#' @rdname check
checkException <- function(expr, msg = "",
silent = getOption("svUnit.silentException")) {
  val <- FALSE
  timing <- as.numeric(system.time({
    ret <- try({
      # Run the test
      silent <- (is.null(silent) || isTRUE(silent))
      val <- inherits(res <- try(expr, silent = silent), "try-error")
    }, silent = TRUE)
  }, gcFirst = FALSE)[3])

  # Log this test
  test <- .logTest(timing)

  # Decide if recording more info or not
  minTiming <- getOption("svUnit.minTiming")
  if (is.null(minTiming))
    minTiming <- 0.1
  if (!isTRUE(getOption("svUnit.recordAll"))  && isTRUE(timing < minTiming)
    && val)
    return(invisible(TRUE))

  # Check for error
  if (inherits(ret, "try-error")) {
    val <- NA
    .logTestData(test, msg = msg, call =
      deparse(sys.call()[1:2], nlines = 1), timing = timing, val = -1,
      res = as.character(ret))
  } else {
    .logTestData(test, msg = msg, call =
      deparse(sys.call()[1:2], nlines = 1), timing = timing, val = val,
      res = if (val) paste(res, collapse = "\n") else
      "No exception generated!\n")
  }
  invisible(val)
}

#' @export
#' @rdname check
DEACTIVATED <- function(msg = "")
  stop(msg)

#' Objects of class 'svTestData' contain results from running a test
#'
#' The 'svTestData' contains results of test run. The `checkxxx()` functions and
#' the `runTest()` method generate one such object which is located in the
#' `.Log` object in `.GlobalEnv`. It is then possible to display and report
#' information it contains in various ways to analyze the results.
#'
#' @param x Any kind of object, or a 'svTestData' object in the case of
#' [print()].
#' @param object A 'svTestData' object.
#' @param all Do we print concise report for all test, or only for the tests
#' that fail or produce an error?
#' @param header Do we print a header or not?
#' @param file Character. The path to the file where to write the report. If
#' `file = ""`, the report is output to the console.
#' @param append Do we append to this file?
#' @param ... Further arguments to pass to methods. Not used yet.
#'
#' @return
#' [is.svTestData()] returns `TRUE` if the object is an 'svTestData'. The
#' various methods serve to extract or print content in the object.
#'
#' @export
#' @name svTestData
#' @author Philippe Grosjean
#' @seealso [svTest()], [is.svSuiteData()], [Log()], [checkEquals()]
#' @keywords utilities
#' @concept unit testing
#' @examples
#' foo <- function(x, y = 2)
#'   return(x * y)
#' is.test(foo)  # No
#' # Create test cases for this function
#' test(foo) <- function() {
#'   checkEqualsNumeric(4, foo(2))
#'   checkEqualsNumeric(5, foo(2, 3))
#'   checkEqualsNumeric(5, foo(nonexists))
#' }
#' # Generate a 'svTestData' object by running the test
#' obj <- runTest(foo)  # Equivalent to runTest(test(foo)), but shorter
#' obj
#' summary(obj)
#' stats(obj)
#' is.svTestData(obj)
#'
#' rm(foo, obj)
is.svTestData <- function(x) {
  # It this a svTestData object
  inherits(x, "svTestData")
}

#' @export
#' @rdname svTestData
stats <- function(object, ...)
  UseMethod("stats")

#' @export
#' @rdname svTestData
stats.svTestData <- function(object, ...) {
  if (!is.svTestData(object))
    stop("'object' must inherit from 'svTestData'")
  Stats <- attr(object, "stats")
  Table <- table(object$kind)
  # Update the table with the total number of test
  Kinds <- c(Stats["tests"] - sum(Table[2:4], na.rm = TRUE), Table[2:4])
  names(Kinds) <- names(Table)
  # Return a list with the table of kinds and the total timing
  list(kind = Kinds, timing = Stats["timing"])
}

#' @export
#' @rdname svTestData
print.svTestData <- function(x, all = FALSE, header = TRUE, file = "",
append = FALSE, ...) {
  # If there is a context attribute, print info about the tests
  cat("", file = file, append = append)
  Context <- attr(x, "context")
  if (!is.null(Context)) {
    unitStr <- if (Context["unit"] == "") "" else
      paste(" (in ", basename(Context["unit"]), ")", sep = "")
    Stats <- stats(x)
    if (isTRUE(header)) {
      cat("\n== ", Context["test"], unitStr, .formatTime(Stats$timing,
        secDigits = 1), ": ", as.character(.kindMax(x$kind)), "\n",
        Context["msg"], "\n", sep = "", file = file, append = TRUE)
    }
    cat(paste(c("//Pass:", "Fail:", "Errors:"), Stats$kind[1:3],
      collapse = " "), "//\n\n", sep = "", file = file, append = TRUE)
    # Don't print tests that succeed if !all
    if (!isTRUE(all)) {
      X <- x[x$kind != "OK", ]
    } else {
      X <- x
    }
  } else {
    X <- x
  }
  # Print info about each individual filtered test
  if (nrow(X) > 0) {
    Res <- ifelse(X$res == "", "", paste("\n", X$res, sep = ""))
    cat(paste("* ", X$msg, ": ", X$call, .formatTime(X$timing,
      secDigits = 3), " ... ", as.character(X$kind), Res, sep = "",
      collapse = "\n"), file = file, append = TRUE)
  }
  invisible(x)
}

#' @export
#' @rdname svTestData
summary.svTestData <- function(object, header = TRUE, file = "",
append = FALSE, ...) {
  # If there is a context attribute, print info about the tests
  cat("", file = file, append = append)
  Context <- attr(object, "context")
  if (!is.null(Context)) {
    unitStr <- if (Context["unit"] == "") "" else
      paste(" (in ", basename(Context["unit"]), ")", sep = "")
    Stats <- stats(object)
    if (isTRUE(header)) {
      cat("\n== ", Context["test"], unitStr, .formatTime(Stats$timing,
          secDigits = 1), ": ", as.character(.kindMax(object$kind)), "\n",
          Context["msg"], "\n", sep = "", file = file, append = TRUE)
    }
    cat(paste(c("//Pass:", "Fail:", "Errors:"), Stats$kind[1:3],
      collapse = " "), "//\n\n", sep = "", file = file, append = TRUE)
  }
  # List tests that failed
  Items <- rownames(object)
  Fail <- object$kind == "**FAILS**"
  if (any(Fail)) {
    cat("=== Failures\n", file = file, append = TRUE)
    cat(paste("[", Items[Fail], "] ", object$msg[Fail], ": ",
      object$call[Fail], collapse = "\n", sep = ""), "\n\n",
      sep = "", file = file, append = TRUE)
  }
  # List tests that produce errors
  Err <- object$kind == "**ERROR**"
  if (any(Err)) {
    cat("=== Errors\n", file = file, append = TRUE)
    cat(paste("[", Items[Err], "] ", object$msg[Err], ": ",
      object$call[Err], collapse = "\n", sep = ""), "\n\n",
      sep = "", file = file, append = TRUE)
  }
}

#' @export
#' @rdname svTestData
protocol_junit.svTestData <- function(object, ...) {
  #if (!require(XML, quietly = TRUE))
  #  return(invisible(FALSE))
  requireNamespace("XML")

  toValidXmlString <- function(s) {
    s <- gsub("&", "&amp;", s)
    s <- gsub("<", "&lt;", s)
    s <- gsub(">", "&gt;", s)
    s <- gsub('"', "&quot;", s)
    s <- gsub("'", "&apos;", s)
    s
  }

  basename <- function(s)
    sub(".*/", "", s)

  Context <- attr(object, "context")
  Stats <- attr(object, "stats")
  .xmlNode <- XML::xmlNode
  .addChildren <- XML::addChildren
  result <- .xmlNode('testcase', attrs = c(
    'classname' = basename(Context[['unit']]),
    'name' = toValidXmlString(Context[['test']]),
    'time' = object$timing))
  kind <- as.numeric(.kindMax(object$kind))  # TODO: use accessor
  elementName <- c(NA, 'failure', 'error', NA)[kind]
  if (!is.na(elementName)) {
    failureNode <- .xmlNode(elementName, attrs = c(
      'type' = elementName,
      'message' = toValidXmlString(object$res)))  # TODO: use accessor
    result <- .addChildren(result, kids = list(failureNode))
  }
  if (kind == 4)
    result <- .addChildren(result, kids = list(.xmlNode('skipped')))

  result
}

#' Create, attach to and manipulate test functions in R objects
#'
#' Test functions are functions without arguments with class 'svTest' containing
#' one or more assertions using \code{checkxxx()} functions. They can be
#' attached to any object as a 'test' attribute. They can also be transferred
#' into a more formal test unit file on disk by applying the [makeUnit()] method.
#'
#' @param testFun A function without arguments defining assertions (using
#' `checkxxx()` functions) for tests to be transformed into a 'svTest' object.
#' @param x Any kind of object.
#' @param value The tests to place in the object (as 'test' attribute); could be
#' a 'svTest' object, or a function without arguments with
#' assertions (`checkxxx()` functions).
#' @param name The name of a test.
#' @param dir The directory where to create the test unit file.
#' @param objfile The path to the file containing the original source code of
#' the object being tested. This argument is used to bring a context for a test
#' and allow a GUI to automatically open the source file for edition when the
#' user clicks on a test that failed or raised an error.
#' @param codeSetUp An expression with some code you want to add to the
#' `.setUp()` function in your unit file (this function is executed before each
#' test.
#' @param codeTearDown An expression with some code you want to add to the
#' `.tearDown()` function in your unit file (this function is executed after
#' each test.
#' @param tag A tag is a character string identifying a location in source code
#' files (either a test unit file, or the original source code of the tested
#' objects defined in `objfile =`. This character string will be searched by the
#' text editor for easy location of the cursor near the corresponding test
#' command, or near the location in the original object that is concerned by
#' this test. Use any string you want to uniquely identify your tag, both in
#' your files, and in this argument.
#' @param msg A message you want to associate with this test run.
#' @param packageName A character string identifying the package from which to
#' extract examples.
#' @param manFilesDir A character string identifying the directory holding the
#' manual pages and examples.
#' @param skipFailing A logical indicating whether missing or failing
#' documentation examples should be marked as `skipped` instead of as `failure`.
#' @param  ... Further arguments to the method (not used yet).
#'
#' @return
#' A 'svTest' object for [svTest()], [as.svTest()] and [test()]. Function
#' [is.svTest()] returns `TRUE` if 'x' is a 'svTest' object, and `is.test()`
#' does the same but also looks in the 'test' attribute if the class of 'x' is
#' not 'svTest' and returns `TRUE` if it finds something there.
#'
#' [makeUnit()] takes an object, extract its test function and write it in a
#' sourceable test unit on the disk (it should be compatible with 'RUnit' test
#' unit files too).
#'
#' [runTest()] returns invisibly a 'svTestData' object with all results after running specified tests.
#'
#' @export
#' @author Philippe Grosjean
#' @seealso [svSuite()], [is.svTestData()], [Log()], [checkEquals()]
#' @keywords utilities
#' @concept unit testing
#' @examples
#' clearLog()    # Clear the log file
#'
#' foo <- function(x, y = 2)
#'   return(x * y)
#' is.test(foo)    # No
#' # Create test cases for this function
#' test(foo) <- function() {
#'   checkEqualsNumeric(4, foo(2))
#'   checkEqualsNumeric(6, foo(2, 3))
#'   checkTrue(is.test(foo))
#'   checkTrue(is.test(test(foo)))
#'   checkIdentical(attr(foo, "test"), test(foo))
#'   checkException(foo(2, "aa"))
#'   checkException(foo("bb"))
#' }
#' is.test(foo)    # Yes
#'
#' \dontrun{
#' # Create a test unit on disk and view it
#' unit <- makeUnit(foo)
#' file.show(unit, delete.file = TRUE)
#' }
#'
#' # Run the test
#' (runTest(foo))
#' # Same as...
#' bar <- test(foo)
#' (runTest(bar))
#'
#' # How fast can we run 100 times such kind of tests (700 test in total)?
#' # (just an indication because in real situation with test unit files, we
#' # have also the time required to source the units!)
#' system.time(for (i in 1:100) runTest(foo))[3]
#'
#' is.svTest(test(foo))    # Yes, of course!
#' # When an object without associated test is passed to runTest(),
#' # a simple test containing only a DEACTIVATED entry is build
#' x <- 1:10
#' summary(runTest(x))
#'
#' summary(Log())
#'
#' rm(foo, bar, x)
svTest <- function(testFun) {
  # Create a 'svTest' object, using testFun: a function without arguments
  # that contains one or more checkXX() assertions
  if (!is.function(testFun))
    stop("'testFun' must be a function or a 'svTest' object")
  # Check that there are no arguments
  if (length(formals(testFun)) > 0)
    stop("'testFun' must be a function without any arguments")
  # This is a S3 object of class 'svTest', subclassing 'function'
  class(testFun) <- c("svTest", "function")
  testFun
}

#' @export
#' @rdname svTest
print.svTest <- function(x, ...) {
  cat("svUnit test function:\n")
  print(body(x))
  invisible(x)
}

#' @export
#' @rdname svTest
as.svTest <- function(x) {
  # Coercion to a 'svTest' object
  svTest(x)
}

#' @export
#' @rdname svTest
is.svTest <- function(x) {
  # It this a svTest object
  inherits(x, "svTest")
}

#' @export
#' @rdname svTest
is.test <- function(x) {
  # Is this a 'svTest'object
  # or do this object contain a non NULL 'test' attribute?
  return(is.svTest(x) || !is.null(attr(x, "test")))
}

#' @export
#' @rdname svTest
test <- function(x) {
  # If x is a 'svTest' object, return it, otherwise,
  # get the 'test' attribute from the object, if it exists
  if (is.svTest(x)) {
    return(x)
  } else {
    res <- attr(x, "test")
    if (is.null(res)) {
      # Create a dummy test with only a DEACTIVATED entry
      res <- svTest(function() DEACTIVATED("Object has no tests!"))
    }
    return(res)
  }
}

#' @export
#' @rdname svTest
`test<-` <- function(x, value) {
  # Add 'value' as a 'test' attribute to 'x' after coercing it to 'svTest'
  attr(x, "test") <- as.svTest(value)
  x
}

#' @export
#' @rdname svTest
makeUnit <- function(x, ...)
  UseMethod("makeUnit")

#' @export
#' @rdname svTest
makeUnit.default <- function(x, name = make.names(deparse(substitute(x))),
dir = tempdir(), objfile = "", codeSetUp = NULL, codeTearDown = NULL, ...) {
  # Take an object and make a unit from the tests it contains
  # It is saved in a file runit<name>.R in 'dir'
  name <- as.character(name)[1]
  name <- sub("^test\\.(.+)\\.$", "\\1", name)
  # Under Windows, we transform \\ into /
  dir <- gsub("\\\\", "/", as.character(dir)[1])
  Unit <- .prepareUnit(name, dir)
  # Just get the test from the object
  Test <- test(x)
  # Make required initialisation to allow locating objects
  .writeSetUp(unit = Unit, file = objfile, code = codeSetUp)
  .writeTearDown(unit = Unit, code = codeTearDown)
  # Write the test function in the file
  .writeTest(unit = Unit, objname = name, obj = x)
  # Return the name of the test function
  Unit
}

#' @export
#' @rdname svTest
makeUnit.svTest <- function(x, name = make.names(deparse(substitute(x))),
dir = tempdir(), objfile = "", codeSetUp = NULL, codeTearDown = NULL, ...) {
  # I know: this is not needed, but it is there in case additional work
  # would be needed in the future, and also to show that makeUnit is
  # designed to work on 'svTest' objects
  makeUnit.default(x, name = name, dir = dir, objfile = objfile,
    codeSetUp = codeSetUp, codeTearDown = codeTearDown, ...)
}

#' @export
#' @rdname svTest
runTest <- function(x, ...)
  UseMethod("runTest")

#' @export
#' @rdname svTest
runTest.default <- function(x, name = deparse(substitute(x)), objfile = "",
tag = "", msg = "", ...) {
  # Run the test for the 'test' attribute of this object
  name <- paste("test(", name, ")", sep = "")
  runTest(test(x), name = name, objfile = objfile, tag = tag, msg = msg, ...)
}

#' @export
#' @rdname svTest
runTest.list <- function(x, ...) {
  # Run each test in x, giving each test the name it has in x
  lapply(names(x), function(name, item=x[[name]]) {
    unit <- ifelse(is.null(attr(item, "unit")), "**root**", attr(item, "unit"))
    runTest(item, name = name, unit = unit, ...)
  })
}

#' @export
#' @rdname svTest
runTest.svTest <- function(x, name = deparse(substitute(x)), objfile = "",
tag = "", msg = "", ...) {
  if (!is.svTest(x))
    stop("'x' must be a 'svTest' object")
  # Names of object and test
  test <- as.character(name)[1]
  test <- .runTest(x, test = test, objfile = objfile, tag = tag, msg = msg, ...)
  .Log <- Log()
  invisible(.Log[[test]])
}

#' Objects of class 'svSuiteData' contain results from running test suites
#'
#' The 'svSuiteData' object contains results of all test run in one or more test
#' suites. The `checkxxx()` functions and the [runTest()] method generate data
#' (objects 'svTestData') contained in the default 'svSuiteData' named `.Log`
#' and located in `.GlobalEnv`. It is then possible to display and report
#' information it contains in various ways to analyze the results.
#'
#' @param x Any kind of object, or a 'svSuiteData' object in the case of
#' [print()].
#' @param object A 'svSuiteData' object.
#' @param fields Character vector. The name of all metadata items you want to
#' extract for the object. The default value is an exhaustive list of all
#' available metadata (i.e., defined by default) in the object, but you can add
#' more: just add a corresponding attribute to your object.
#' @param all Do we print concise report for all test, or only for the tests
#' that fail or produce an error?
#' @param file Character. The path to the file where to write the report.
#' If `file = ""`, the protocol report is output to the console.
#' @param append Do we append to this file?
#' @param type Character. The type of protocol report to create. For the moment,
#' only `type = "text"` and `type = "junit"` are supported, but further types
#' (HTML, LaTeX, Wiki, etc.) could be provided later.
#' @param  ... Further arguments to pass to methods. Not used yet.
#'
#' @return
#' [is.svSuiteData()] returns `TRUE` if the object is an 'svSuiteData'. The
#' various methods serve to extract or print content in the object.
#'
#' @details
#' A 'svSuiteData' is, indeed, an environment. The results for the various tests
#' runs are in non hidden (i.e., names not starting with a dot) objects that are
#' of class 'svTestData' in this environment. Various other objects that control
#' the execution of the test, their context, etc. are contained as hidden objects
#' with name starting with a dot. Note that using an environment instead of a
#' list for this object allows for a call by reference instead of a usual call by
#' value in R, when passing this object to a function. This property is largely
#' exploited in all svUnit functions to make sure results of test runs are
#' centralized in the same log ('svSuiteData' object).
#'
#' @export
#' @name svSuiteData
#' @author Philippe Grosjean; Mario Frasca for the junit protocol.
#' @seealso [svSuite()], [is.svTestData()], [Log()], [checkEquals()]
#' @keywords utilities
#' @concept unit testing
#' @examples
#' clearLog()    # Clear any existing log
#'
#' # Run some tests
#' checkTrue(1 < 2)
#' checkException(log("a"))
#' foo <- function(x, y = 2)
#'   return(x * y)
#' test(foo) <- function() {
#'   checkEqualsNumeric(4, foo(2))
#'   checkEqualsNumeric(6, foo(2, nonexisting))
#'   checkTrue(is.test(foo))
#'   warning("This is a warning")
#'   cat("Youhou from test!\n")  # Don't use, except for debugging!
#'   checkTrue(is.test(test(foo)))
#'   checkIdentical(attr(foo, "test"), test(foo))
#'   checkException(foo(2, nonexisting))
#'   #DEACTIVATED("My deactivation message")
#'   checkException(foo(2))  # This test fails
#' }
#' runTest(foo)
#'
#' # Now inspect the log, which is a 'svSuiteData' object
#' is.svSuiteData(Log())
#' stats(Log())
#' metadata(Log())
#' Log()    # Print method
#' summary(Log())
#'
#' \dontrun{
#' # To get a print of the test protocol on file, use:
#' protocol(Log(), type = "text", file = "RprofProtocol.out")
#' file.show("RprofProtocol.out")
#' unlink("RprofProtocol.out")
#' }
#'
#' rm(foo)
#'
#' \dontrun{
#' # Profiling of very simple test runs
#' library(utils)
#' createLog(description = "test profiling", deleteExisting = TRUE)
#' imax <- 3
#' jmax <- 100
#' l <- 50
#' Rprof()
#' for (i in 1:imax) {
#'   # Change the context for these tests
#'   .Log$..Test <- paste("Test", i, sep = "")
#'   .Log$..Tag <- paste("#", i, sep = "")
#'   res <- system.time({
#'     for (j in 1:jmax) checkTrue(i <= j, "My test")
#'   }, gcFirst = TRUE)[3]
#'   print(res)
#'   flush.console()
#' }
#' Rprof(NULL)
#' # Look at profile
#' summaryRprof()
#' unlink("Rprof.out")
#'
#' # Look at the log
#' summary(Log())
#' }
is.svSuiteData <- function(x) {
  # It this a svSuiteData object
  inherits(x, "svSuiteData")
}

#' @export
#' @rdname svSuiteData
stats.svSuiteData <- function(object, ...) {
  if (!is.svSuiteData(object))
    stop("'object' must inherit from 'svSuiteData'")
  # Get the list of tests
  Tests <- ls(object)
  if (length(Tests) == 0) {
    # The object is empty!
    Res <- data.frame(kind = .kind(logical()), timing = numeric(),
      time = numeric(), unit = character(), tag = character(),
      msg = character(), stringsAsFactors = FALSE)
  } else {
    # Functions to get data for each test
    getKind <- function(x) .kindMax(x$kind)
    getTiming <- function(x) attr(x, "stats")["timing"]
    getTime <- function(x) attr(x, "time")
    getContext <- function(x, item) attr(x, "context")[[item]]
    Res <- data.frame(
      kind = rev(sapply(object, getKind)),
      timing = rev(sapply(object, getTiming)),
      time = structure(rev(sapply(object, getTime)),
        class = c("POSIXt", "POSIXct")),
      unit = rev(sapply(object, getContext, "unit")),
      msg = rev(sapply(object, getContext, "msg")),
        stringsAsFactors = FALSE)
  }
  Res
}

#' @export
#' @rdname svSuiteData
metadata <- function(object, ...)
  UseMethod("metadata")

#' @export
#' @rdname svSuiteData
metadata.svSuiteData <- function(object,
fields = c("R.version", "sessionInfo", "time", "description"), ...) {
  # Extract metadata information from a 'svSuiteData' object
  if (!is.svSuiteData(object))
    stop("'object' must inherit from 'svSuiteData'")
  # Return a list with all metadata elements found
  fields <- paste(".", fields, sep = "")
    Res <- list()
  for (F in fields)
    Res[[F]] <- object[[F]]
  Res
}

#' @export
#' @rdname svSuiteData
print.svSuiteData <- function(x, all = FALSE, file = "", append = FALSE, ...) {
  if (!is.svSuiteData(x))
    stop("'x' must inherit from 'svSuiteData'")
  Tests <- ls(x)
  if (length(Tests) == 0) {
    cat("No test records!\n", file = file, append = append)
  } else {
    # Print general information about the tests
    Stats <- stats(x)
    Tests <- rownames(Stats)  # To make sure we use the same!
    Timing <- .formatTime(sum(Stats$timing, na.rm = TRUE), secDigits = 1)
    cat("= A svUnit test suite", Timing, " with:\n\n", sep = "",
      file = file, append = append)
    cat(paste("* ", Tests, " ... ", as.character(Stats$kind), "",
      sep = "", collapse = "\n"),
      "\n\n", sep = "", file = file, append = TRUE)

    # Print detailed information about each test
    for (Test in Tests)
      print(x[[Test]], all = all, file = file, append = TRUE, ...)
  }
  invisible(x)
}

#' @export
#' @rdname svSuiteData
summary.svSuiteData <- function(object, ...)
    protocol_text.svSuiteData(object, ...)

#' @export
#' @rdname svSuiteData
protocol <- function(object, type = "text", file = "", append = FALSE, ...)
  UseMethod("protocol")

#' @export
#' @rdname svSuiteData
protocol.default <- function(object, type = "text", file = "",
append = FALSE, ...)
  get(paste("protocol", type[1], sep = "_"))(object, file = file,
    append = append, ...)

#' @export
#' @rdname svSuiteData
protocol.svSuiteData <- function(object, type = "text", file = "",
append = FALSE, ...)
  get(paste("protocol", type[1], sep = "_"))(object, file = file,
    append = append, ...)

#' @export
#' @rdname svSuiteData
protocol_text <- function(object, file = "", append = FALSE, ...)
  UseMethod("protocol_text")

#' @export
#' @rdname svSuiteData
protocol_text.svSuiteData <- function(object, file = "",
append = FALSE, ...) {
  if (!is.svSuiteData(object))
    stop("'object' must inherit from 'svSuiteData'")
  Tests <- sort(ls(object))
  if (length(Tests) == 0) {
    cat("No test records!\n", file = file, append = append)
  } else {
    # Print general information about the tests
    Stats <- stats(object)
    Tests <- rownames(Stats)  # To make sure we use the same!
    Timing <- .formatTime(sum(Stats$timing, na.rm = TRUE), secDigits = 1)
    cat("= A svUnit test suite", Timing, " with:\n\n", sep = "",
      file = file, append = append)
    cat(paste("* ", Tests, " ... ", as.character(Stats$kind), "",
      sep = "", collapse = "\n"),
      "\n\n", sep = "", file = file, append = TRUE)

    # Summarize each test
    for (Test in Tests)
      summary(object[[Test]], file = file, append = TRUE)
  }
}

#' @export
#' @rdname svSuiteData
protocol_junit <- function(object, ...)
  UseMethod("protocol_junit")

#' @export
#' @rdname svSuiteData
protocol_junit.svSuiteData <- function(object, file = "", append = FALSE, ...) {
  if (!is.svSuiteData(object))
    stop("'object' must inherit from 'svSuiteData'")
  #if(!require(XML, quietly = TRUE))
  #  return(invisible(FALSE))
  requireNamespace("XML")

  Tests <- sort(ls(object))
  if (length(Tests) > 0 && inherits(object[[Tests[1]]], "svSuiteData")) {
    # This is a set of suites (containing svSuiteData)
    root <- XML::xmlNode('testsuites')
  } else {
    # This is a single suite (containing svTestData)
    root <- XML::xmlNode('testsuite')
  }

  with(stats(object), addAttributes(root, name = NULL, tests = length(Tests),
    errors = sum(kind == '**ERROR**'), failures = sum(kind == '**FAILS**'),
    skip = sum(kind == 'DEACTIVATED')))
  for (Test in Tests)
    root <- XML::addChildren(root,
      kids = list(protocol_junit(object[[Test]], append = TRUE)))

  # Decide whether to return the xml node or write the xml file
  if (isTRUE(append)) {
    return(root)
  } else {
    XML::saveXML(root, file)
    return(invisible(TRUE))
  }
}

# Note that the message in case of error is not explicit enough.Moreover,
# nothing is printed in case of success, while we would like a succinct report
# TODO: check if warnings are not converted into errors during R CMD check tests
library(svUnit)
clearLog()
res <- capture.output(runTest(svSuite("package:svUnit"), "svUnit"))
errorLog()

# Run tests using svUnit
# This is a wrapper to run these from within testthat
# so that RStudio's test function and devtools::test() can be used too
pkgname <- "svUnit"

test_svUnit <- function(pkgname) {
  library(svUnit)
  clearLog()

  # Current dir is /tests/testthat
  basedir <- dirname(dirname(getwd()))

  # Look for our tests in /unitTests or /inst/unitTests
  testdir <- file.path(basedir, "unitTests")
  if (!dir.exists(testdir)) {
    # We must be in a development environment, look at /inst/unitTests
    testdir <- file.path(basedir, "inst", "unitTests")
  }
  if (!dir.exists(testdir)) {
    # We must be in a checking environment, look at /<pkgname>/unitTests
    testdir <- file.path(basedir, pkgname, "unitTests")
  }
  if (!dir.exists(testdir)) {
    # No test directory found
    stop("No svUnit test directories found for ", basename(basedir))
  }

  res <- capture.output(runTest(svSuite(paste0("dir:", testdir)), pkgname))
  summary(Log())
}
test_svUnit(pkgname)

# runit.VirtualClass.R test suite
# Just one (little bit more complex) example taken from RUnit

# Test setup
if (FALSE) {
  # Not really needed, but can be handy when writing tests
  library("svUnit")
}

# Package 'methods' is usually loaded, but make sure it is
if (!require(methods))
  stop("Package 'methods' is required!")

# Define class (not exported yet by the program, and defined in .GlobalEnv!)
className <- "MyVirtualBaseClass"
setClass(className,
  representation("VIRTUAL",
    x = "numeric",
    y = "numeric",
    description = "character"),
  validity = NULL,
  sealed   = FALSE,
  where    = .GlobalEnv)

if (!isGeneric("getX")) {
  setGeneric("getX", function(object, ...) standardGeneric("getX"),
    useAsDefault = TRUE, where = .GlobalEnv, valueClass = "numeric")
}

setMethod("getX", signature = className, function(object) return(object@x),
  where = .GlobalEnv)

if (!isGeneric("setX<-")) {
  setGeneric("setX<-", function(object, value) standardGeneric("setX<-"),
    useAsDefault = TRUE, where = .GlobalEnv)
}

setMethod("setX<-",
  signature = signature(object = className, value = "numeric"),
  function(object, value) {
    if (length(value) < 1)
      stop("value has to contain at least one element.")
    if (any(is.na(value)))
      stop("value may not contain NA(s).")
    object@x <- value
    object
  }, where = .GlobalEnv)


# Test functions
.setUp <- function() {
  # Executed before each test function
  # ...
}

.tearDown <- function() {
  # Executed after each test function
  # ...
}

testCreateClass <- function() {
  setClass("A", contains = "numeric", where = .GlobalEnv)
  a <- new("A")
  checkTrue(validObject(a))
  removeClass("A", where = .GlobalEnv)	# Better to use on.exit() here!
  checkException(new("A"))
}

testMyVirtualBaseClass.getX <- function() {
  testClassName <- "MyDerivedTestClass"
  setClass(testClassName,
    representation("MyVirtualBaseClass"),
    validity = NULL,
    sealed   = FALSE,
    where    = .GlobalEnv)

  on.exit(removeClass(testClassName, where = .GlobalEnv))

  # system constructor
  this <- new(testClassName)

  # constructor call succeeded?
  checkTrue(is(this, testClassName))

  ret <- getX(this)
  checkTrue(is(ret, "numeric"))
  # class default
  checkEquals(numeric(0), ret)
}

testMyVirtualBaseClass.setX <- function() {
  testClassName <- "MyDerivedTestClass"
  setClass(testClassName,
    representation("MyVirtualBaseClass"),
    validity = NULL,
    sealed   = FALSE,
    where    = .GlobalEnv)

  on.exit(removeClass(testClassName, where = .GlobalEnv))

  # System constructor
  this <- new(testClassName)

  # Constructor call succeeded?
  checkTrue(is(this, testClassName))

  testSeq <- 1:23
  setX(this) <- testSeq
  ret <- getX(this)
  checkTrue(is(ret, "numeric"))
  checkEquals(testSeq, ret)

  # Error handling
  checkException(setX(this) <- numeric(0))
  checkException(setX(this) <- as.numeric(NA))
  checkException(setX(this) <- c(1:4, NA))
}

# runitBadTests.R test suite
# by Ph. Grosjean <phgrosjean@sciviews.org>

# Create a few objects we need for tests

# An R object (matrix)
mat <- matrix(rnorm(4), ncol = 2)

# Create very simple test cases for matrix 'mat'
testmat <- svTest(function() {
  checkEqualsNumeric(2, nrow(mat))
  checkTrue(is.numeric(mat))
})

# An example function without test case
foo <- function(x)
  x

# Another function with a test associated
bar <- function(x)
  x^2

testbar <- svTest(function() {
  checkEqualsNumeric(4, bar(2))
  checkException(bar("xx"))
})
test(bar) <- testbar

# The test cases
.setUp <- function() {
  # Executed before each test function
  # ...
}

.tearDown <- function() {
  # Executed after each test function
  # ...
}

testBadTests <- function() {
  # These tests should fail
  checkEquals(c(x = 2), 2, "Check a double is equal to a named double")
  checkEqualsNumeric(2, 3, "Check if two different numbers are equal")
  checkIdentical(2, sqrt(2)^2, "Check a double is exactly the expected value")
  checkTrue(FALSE, "Check if FALSE is TRUE")
  checkException(log(10), "log(10) produces and exception")

  # These tests should generate en error
  checkEquals(fff(2), 2, "Wrong expression in checkEquals()")
  checkEqualsNumeric(fff(2), 2, "Wrong expression in checkEqualsNumeric()")
  checkIdentical(2, fff(2), "Wrong expression in checkIdentical()")
  checkTrue(fff(), "Wrong expression in checkTrue()")
}

# runitsvSuite.R test suite
# by Ph. Grosjean <phgrosjean@sciviews.org>
# Run it simply by example(unitTests.svUnit)

# Create a few objects we need for our tests

# Save current exclusion list and clear it
oex <- getOption("svUnit.excludeList")

# Create a very simple 'svTest' object
test_R <- svTest(function() {
  checkTrue(1 < 2)
})

# The test cases
.setUp <- function() {
  # Executed before each test function
  # Remove temporarily the exclusion list for our tests
  options(svUnit.excludeList = NULL)

  # Create an object with associated tests in .GlobalEnv
  foo <- function(x)
    return(x)
  test(foo) <- function() {
    checkEquals(2, foo(2),            "foo(2) returns 2")
    checkException(foo("x"),          "foo(\"x\") raises an exception")
  }
  svSuite.foo <<- foo   # Place a copy of 'foo' in .GlobalEnv

  # Create an object without associated tests in .GlobalEnv
  svSuite.bar <<- function(x)
    return(x^2)

  # Create an integration test in .globalEnv
  test_svSuite <<- svTest(function() {
    checkTrue(1 == 1,                  "example test: 1 == 1")
    checkException(nonexisting + 1,    "exception when using non existing var")
  })
}

.tearDown <- function() {
  # Executed after each test function
  # Restore previous exclusion list
  options(svUnit.excludeList = oex)
  # Remove our object with tests in .GlobalEnv
  rm(svSuite.foo, svSuite.bar, test_svSuite, envir = .GlobalEnv)
}

testsvSuite <- function() {
  checkTrue(is.svSuite(svSuite("svSuite.foo")),
    "svSuite(\"svSuite.foo\") returns a 'svSuite' object")
  checkTrue(is.svSuite(as.svSuite("svSuite.foo")),
    "as.svSuite(\"svSuite.foo\") returns a 'svSuite' object")
  checkTrue(is.svSuite(svSuite("svSuite.bar")),
    "svSuite(\"svSuite.bar\") returns a 'svSuite' object")
  checkTrue(is.svSuite(svSuite("test_svSuite")),
    "svSuite(\"test_svSuite\") returns a 'svSuite' object")
  checkTrue(is.svSuite(print(svSuite("test_svSuite"))),
    "print(svSuite) returns a 'svSuite' object invisibly")
  checkTrue(is.svSuite(svSuite("nonexisting")),
    "svSuite(\"nonexisting\") returns a 'svSuite' object")
  checkException(svSuite(nonexisting),
    "svSuite(nonexisting) raises an exception")
}

testsvSuiteList <- function() {
  checkTrue(is.svSuite(svSuiteList()),
    "svSuiteList() returns a 'svSuite' object")
  checkTrue("package:svUnit" %in% svSuiteList(),
    "svSuiteList() lists 'svSuite' package")
  checkTrue("package:svUnit (VirtualClass)" %in% svSuiteList(),
    "svSuiteList() lists 'VirtualClass' suite")
  checkTrue("test(svSuite.foo)" %in% svSuiteList(),
    "svSuiteList() lists objects with tests")
  checkTrue("test_svSuite" %in% svSuiteList(),
    "svSuiteList() lists 'svTest' objects")
  checkTrue("test_R" %in% svSuiteList(pos = parent.frame()),
    "svSuiteList() uses 'pos' correctly")
}

# runitsvTest.R test suite
# by Ph. Grosjean <phgrosjean@sciviews.org>
# Run it simply by example(unitTests.svUnit)

# Create a few objects we need for tests

# An R object (matrix)
mat <- matrix(rnorm(4), ncol = 2)

# Create very simple test cases for matrix 'mat'
testmat <- svTest(function() {
  checkEquals(2, nrow(mat), checkNames = FALSE)
  checkEquals(2, ncol(mat), checkNames = TRUE) # Not important here
  checkTrue(is.numeric(mat))
})

# An example function without test case
foo <- function(x)
  x

# Another function with a test associated
bar <- function(x)
  x^2

testbar <- svTest(function() {
  checkEqualsNumeric(4, bar(2))
  checkException(bar("xx"))
})
test(bar) <- testbar

# The test cases
.setUp <- function() {
  # Executed before each test function
  # ...
}

.tearDown <- function() {
  # Executed after each test function
  # ...
}

testis.test <- function() {
  checkTrue(!is.test(foo),             "No associated test cases to 'foo'")
  checkTrue(is.test(testbar),          "Is testbar a 'svTest'?")
  checkTrue(is.test(bar),              "Associated test cases to 'bar'")
  checkTrue(!is.test(mat),             "No associated test cases to 'mat'")
  checkTrue(is.test(testmat),          "Is an 'svTest' object a test?")

  if (exists(".Log"))
    .Log$..Obj <- "test"       # Switch the context to test()
  checkTrue(is.test(test(foo)),        "Return dummy test if no test cases")
  checkIdentical(testbar, test(bar),   "test of 'bar' identical to 'testbar'")

  if (exists(".Log"))
    .Log$..Obj <- "test<-"     # Switch the context to `test<-`()
  checkException(test(foo) <- "x",     "Strange value to assign as 'test'")
  checkException(test(foo) <- function(y) y,
    "Try assign a function with arguments")

  # Add test cases to an object
  mat2 <- mat
  checkTrue(is.test(test(mat2) <- testmat),
    "'mat2' valid test case association")
  checkIdentical(testmat, test(mat2),
    "test of 'mat2' identical to 'testmat'")
  # Strange,... but allowed
  test(testbar) <- testbar
  checkIdentical(testbar, test(testbar),
    "Assigning test cases to oneself")

  if (exists(".Log"))
    .Log$..Obj <- "is.test"    # Switch context back to is.test()
  checkTrue(!is.test("x"),             "'x' is is not a 'svTest' object")
  checkTrue(!is.test(NULL),            "NULL is not a 'svTest' object")
  checkTrue(!is.test(NA),              "NA is not a 'svTest' object")
}

testsvTest <- function() {
  checkException(svTest(foo),          "Functions with arguments not allowed")
  checkException(svTest("x"),          "Strange argument to svTest")
  checkTrue(is.svTest(svTest(function() {})),
    "Creation of a minimal 'svTest' object")

  if (exists(".Log"))
    .Log$..Obj <- "is.svTest"  # Switch context to is.svTest()
  checkTrue(is.svTest(testmat),        "Is testmat a 'svTest' object?")
  checkTrue(is.svTest(testbar),        "Is testbar a 'svTest' object?")
  checkTrue(is.svTest(test(bar)),      "Is test(bar) a 'svTest' object?")
  checkTrue(!is.svTest(foo),           "'foo' is not a 'svTest' object")
  checkTrue(!is.svTest("x"),           "'x' is not a 'svTest' object")
  checkTrue(!is.svTest(NULL),          "NULL is not a 'svTest' object")
  checkTrue(!is.svTest(NA),            "NA is not a 'svTest' object")
  checkTrue(!is.svTest(function() {}), "A function is not a 'svTest' object")

  if (exists(".Log"))
    .Log$..Obj <- "as.svTest"  # Switch context to as.svTest()
  checkTrue(is.svTest(as.svTest(testmat)),
    "Coercion to a 'svTest' object")
  checkException(as.svTest("x"),
    "Try coercion on wrong object")
  checkException(as.svTest(function(y) y),
    "Try coercion on function with arguments")
}

testrunTest <- function() {
  checkTrue(inherits(runTest(testbar), "svTestData"),
    "result of runTest(testbar) is 'svTestData'")
  # Following tests fail currently for reasons I haven't spotted yet,
  # but runTest() works wine outside of these tests... So, I deactivate them
  DEACTIVATED("runTest(bar) does not work inside test functions")
  checkTrue(inherits(runTest(test(bar)), "svTestData"),
    "result of runTest(test(bar)) is 'svTestData'")
  checkTrue(inherits(runTest(bar), "svTestData"),
    "result of runTest(bar) is 'svTestData'")
}

# To be promoted as a demo?
library(svUnit)

options(svUnit.excludeList = NULL)

foo <- function(x, y = 2)
  x * y

#is.test(foo)    # No
# Create test cases for this function
test(foo) <- function() {
  checkEqualsNumeric(4, foo(2))
  checkEqualsNumeric(5, foo(2, 3)) # Should fail
  checkTrue(is.test(foo))
  checkTrue(is.test(test(foo)))
  checkIdentical(attr(foo, "test"), test(foo))
  checkException(foo(2, dfgfg))
  #DEACTIVATED("My deactivation message")
  checkException(foo("bb"))
}

test_simple <- svTest(function() {
  checkTrue(1 == 1, "test1")
  checkTrue(1 == 2, "test2")
  checkTrue(1 == var, "test3")
})

# This is for an example!
clearLog()
imax <- 3
jmax <- 100
l <- 50
Rprof()
for (i in 1:imax) {
  .LogTest <- paste("Test", i, sep = "")
  .LogTag <- paste("#", i, sep = "")
  res <- system.time({
    for (j in 1:jmax)
      checkTrue(i <= j, "My test")
  }, gcFirst = TRUE)[3]
  print(res)
  flush.console()
}
Rprof(NULL)
summaryRprof()$by.self
summary(.Log[[.LogTest]])
rm(.LogTest, .LogTag)
unlink("Rprof.out")

Rprof()
system.time(
  for (i in 1:100)
    runTest(test.bar))[3]
Rprof(NULL)
summaryRprof()
unlink("Rprof.out")

#' @details
#' The SciViews 'svUnit' package defines a framework for testing R code, not
#' unlike jUnit for Java. It is inspired on the `checkxxx()` functions from the
#' 'RUnit' package and the same test unit files should be compatible with both
#' 'svUnit' and 'RUnit'. However, the internal implementation is completely
#' different and svUnit can also be used interactively, while 'RUnit' is only
#' designed to run test units written in files on disks.
#'
#' The test unit framework provided in 'svUnit' is based on tests, also called
#' assertions, implemented in `checkxxx()` functions. For instance, the
#' `checkTrue(expr)` function check if its 'expr' argument returns `TRUE`.
#' Results of these assertions are collected in a centralized logger located in
#' the `.Log` object in `.GlobalEnv`. This is a 'svSuiteData' object with data
#' about the context of the tests (see for instance, [lastTest()], [lastSuite()]
#' or `metadata(.Log)`).
#'
#' Assertions can give three results: (1) `TRUE` if success, (2) `FALSE` in
#' case of failure (in our example, 'expr' in `checkTrue(expr)` did not return
#' `TRUE`), and (3) `NA` if the code in 'expr' cannot be parsed or executed
#' correctly. All these errors or failures are catch and recorded in the logger,
#' as individual 'svTestData' objects.
#'
#' Both the logger ('svSuiteData' object) and test records inside it
#' ('svTestData' objects) have convenient methods to visualize information they
#' contain: [print()], [summary()] and [stats()] methods. Access to the
#' individual test records in the logger is done with list-like instructions:
#' `.Log$mytest` returns the 'svTestData' object named 'mytest', itself the
#' result of running test in the 'mytest' test function (i.e.,
#' `runTest(mytest)`, see hereunder). Assertions run at the command line,
#' outside of specific contexts provided by test functions, test units and
#' test suites (see hereunder) are recorded under the 'eval' 'svTestData' object
#' in the logger (i.e., `.Log$eval`).
#'
#' Since a 'svSuiteData' object (the logger) is also an environment, you can get
#' the list of all test records it contains using `ls(.Log)`, and you can
#' eliminate a given test record using `rm(mytest, envir = .Log)`.
#'
#' Test cases are collections of assertions with the satellite code needed to
#' build example or situations to be tested. They are collected together in
#' argument-less functions with class being 'svTest'. See `?svTest`  for
#' further explanations and a couple of example test cases/test functions.
#'
#' In its simplest instance, a test function is defined as a separate R object
#' loaded in memory (unlike RUnit where all test must be defined in files). You
#' run it simply by using `runTest(mytest)`. A slightly more structured way
#' to work is to attach the test function to the object it testes. You use
#' `test(myobj) <- testmyobj` to do so, and retrieve it with `test(myobj)`. Now,
#' the test function always follows the tested object. Testing the object is
#' still simple by using `runTest(myobj)`, which is totally equivalent to
#' `runTest(test(myobj))`. One can determine if an object has a test function
#' associated, or is a test function itself by using `is.test(myobj)`.
#'
#' Several test functions can be collected together in so-called test units. A
#' test unit only exists on disk. It is a file named 'runit*.R' containing
#' sourceable R code with test functions having names starting with 'test'
#' (unlike 'RUnit', the default convention of file names starting with 'runit'
#' and test function names starting with 'test' is not customizable in
#' 'svUnit'). One can also define special `.setUp()` and `.tearDown()` functions
#' in the test unit. The first function will be run before each test function,
#' and the latter one will be run after it. Test units are created manually, or
#' from a collection of objects with associated test functions loaded in an
#' environment (usually `.GlobalEnv`) thanks to the [makeUnit()] method. These
#' units should be mutually compatible with those used in the 'RUnit' package
#' (at least this is verified with version 0.4-17 of 'RUnit').
#'
#' Test units defined for packages should be located in the package /runitTests
#' subdirectory (/inst/runitTests for source of the package) or one of its
#' subdirectories. That way, they are located automatically by the function
#' [svSuiteList()] that also automatically detects all objects with associated
#' test functions loaded in `.GlobalEnv`. Test suites are 'svSuite'
#' objects with a list of test units or test objects to collect in the suite.
#' Thus, [svSuiteList()] automatically builds such a suite with all tests it
#' finds in R, with many possibilities to filter packages' test units, objects'
#' test functions, or to add non standard directories with test units, for
#' instance. See `?svSuite` for more details on creating and using these
#' suites.
#'
#' A GUI (Graphical User Interface) is provided to automatically build and run
#' tests suites and to get a graphical (tree) interactive report of the results
#' in the Komodo Edit or IDE code editor, together with the SciViews-K
#' extension. If you want to use this (optional) GUI, you have to install
#' required software components on your machine.
#'
#' Finally, the 'svUnit' framework is compatible with `R CMD check` (see the
#' manual "Writing R extensions"). You simply define man pages (.Rd files) with
#' an example section running selected test units from your package. The
#' function [errorLog()] is designed to generate and error if one or more tests
#' failed or raised an error during `R CMD check`, and it should be used at the
#' end of the example that runs your unit test(s). That way, `R CMD check` is
#' interrupted and a detailed report of the tests that failed or raised an error
#' is printed. See an example in `?unitTests.svUnit`.
#'
#' @author Written by Ph. Grosjean, inspired from the general design of the
#' 'RUnit' package by Thomas Konig, Klaus Junemann & Matthias Burger.
#'
#' @references There is a huge literature and unit testing. An easy starting
#' point is: \url{https://en.wikipedia.org/wiki/Unit_test}.
#'
#' @examples
#' # Clear the logger
#' clearLog()
#'
#' # Design and attach a simple test function to an object
#' foo <- function(x, y = 2)
#'   return(x * y)
#' testfoo <- function() {
#'   #DEACTIVATED()	# Use this to deactive the test (notice placed in the log)
#'   checkEqualsNumeric(5, foo(2),  "Check return of foo()")
#'   checkException(foo("b"),       "Wrong first argument")
#'   checkException(foo(2, "a"),    "Wrong second argument")
#' }
#' # Attach this to the foo function
#' test(foo) <- testfoo
#'
#' # Run this test
#' runTest(foo)
#'
#' # Inspect the result
#' ls(.Log)
#' .Log$`test(foo)`
#' # This test fails. You see that the test function requires that foo(2) = 5
#' # and the actual implementation returns 4. This is a trivial, useless example,
#' # but you are supposed to correct the function. For instance:
#' foo <- function(x, y = 2)
#'   return(x * y + 1)
#' test(foo) <- testfoo
#'
#' (runTest(foo))	# Now, that's fine!

#' @keywords internal
"_PACKAGE"

# The following block is used by usethis to automatically manage
# roxygen namespace tags. Modify with care!
## usethis namespace: start
## usethis namespace: end
NULL

---
title: "'SciViews::R' - Unit, Integration and System Testing"
author: "Philippe Grosjean (phgrosjean@sciviews.org)"
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    toc: true
    toc_depth: 3
    fig_caption: yes
vignette: >
  %\VignetteIndexEntry{'SciViews::R' - Unit, Integration and System Testing}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")
```

## Introduction

Unit testing (see https://en.wikipedia.org/wiki/Unit_test) is an approach successfully used to develop software, and to ease code refactoring for keeping bugs to the minimum. It is also the insurance that the software is doing the right calculation (quality insurance). Basically, a test just checks if the code is running and is producing the correct answer/behavior in a given situation. As such, unit tests are build in R package production because all examples in documentation files, and perhaps, test code in `/tests` subdirectory are run during the checking of a package (`R CMD check <pkg>`). However, the R approach lacks a certain number of features to allow optimal use of unit tests as in extreme programming (test first – code second):

* Tests are related to package compilation and cannot easily be run independently (for instance, for functions developed separately).
* Once a test fails, the checking process is interrupted. Thus one has to correct the bug and launch package checking again... and perhaps get caught by the next bug. It is a long and painful process.
* There is no way to choose one or several tests selectively: all are run or not (depending on command line options) during package checking.
* It is very hard, or near to impossible to program in R in a test driven development (*write tests first*) with the standard tools (https://en.wikipedia.org/wiki/Test-driven_development).
* Consequently, the '*test-code-simplify*' cycle is not easily accessible yet to R programmer, because of the lack of an interactive and flexible testing mechanism providing immediate, or quasi immediate feedback about changes made.
* We would like also to emphasize that test suites are not only useful to check code, they can also be used to check data, or the pertinence of analyses.

## Unit testing in R without 'svUnit'

Besides the "regular" testing mechanism of R packages, one can find the 'testthat' and 'RUnit' packages on CRAN (https://CRAN.R-project.org/package=testthat and https://CRAN.R-project.org/package=RUnit, respectively)^[There is also a very simple testing system in 'testit' (https://CRAN.R-project.org/package=testit) using only one assertion: `assert()` for all test. Other interesting R packages on CRAN are 'testthis' (https://CRAN.R-project.org/package=testthis), which adds more tools and an RStudio addins around 'testthat', and 'runittotestthat' (https://CRAN.R-project.org/package=runittotestthat) which, as its name implies, automatically converts 'RUnit' tests into 'testthat' equivalents.]. Another package used to provide an alternate implementation of test unit: 'butler', but it is archived because it is not maintained any more and has given up in favor of 'RUnit'.

The 'testthat' package is heavily used by the UseR community (there are many more packages on CRAN that use it than any other testing system). It implements R testing in a rather original way. 'RUnit', on the other hand sticks with the framework initially defined by junit for Java and largely developed for many different languages. In this document, we will not discuss further about 'testthat'. We will focus on the junit-like framework implemented in 'RUnit' and 'svUnit'. 'RUnit' implements the following features:

* **Assertions**, `checkEquals()`, `checkEqualsNumeric()`, `checkIdentical()` and `checkTrue()` and negative tests
(tests that check error conditions, `checkException()`).
* Assertions are grouped into R functions to form one **test function** that runs a series of related individual tests. It is easy to temporarily inactivate one or more tests by commenting lines in the function. To avoid forgetting tests that are commented out later on, there is special function, named `DEACTIVATED()` that tags the test with a reminder for your deactivated items (i.e., the reminder is written in the test log).
* A series of test functions (whose name typically start with `test....`) are collected together in a source-able R code file (name starting with `runit....`) on disk. This file is called a **test unit**.
* A **test suite** (object `RUnitTestSuite`) is a special object defining a battery of tests. It points to one or several directories containing test units. A test suite is defined by `defineTestSuite()`.
* One or more test suites can be run by calling `runTestSuite()`. There is a shortcut to define and run a test suite constituted by only one test unit by using the function `runTestFile()`. Once the test is run, a `RUnitTestData` object is created that contains all the information collected from the various tests run.
* One can print a synthetic report (how many test units, test functions, number of errors, fails and deactivated item), or get a more extensive `summary()` of the tests with indication about which ones failed or produced errors. The function `printTextProtocol()` does the same, while `printHTMLProtocol()` produces a report in HTML format.
* 'RUnit' contains also functions to determine which code is run in the original function when tested, in order to detect the parts of the code not covered by the test suite (code coverage function
`inspect()` and function `tracker()`).

As complete and nice as 'RUnit' is, there is no tools to integrate the test suite in a given development environment (IDE) or graphical user interface (GUI), as far as we know. In particular, there is no real-time reporting mechanism used to easy the *test-code-simplify* cycle. The way tests are implemented and run is left to the user, but the implementation suggests that the authors of 'RUnit' mainly target batch execution of the tests (for instance, nightly check of code in a server), rather that real-time interaction with the tests.

There is also no integration with the "regular" `R CMD check` mechanism of R in 'RUnit'.

## Unit testing framework for R with 'svUnit'

Our initial goal was to implement a GUI layer on top of 'RUnit', and to integrate test units as smoothly as possible in a code editor, as well as, making tests easily accessible and fully compatible with `R CMD check` on all platforms supported by R. Ultimately, the test suite should be easy to create, to use interactively, and should be able to test functions in a complex set of R packages.

However, we encountered several difficulties while trying to enhance 'RUnit' mechanism. When we started to work on this project, 'RUnit' (version 0.4-17) did not allow to subclass its objects (version 0.4-19 solved this problem). Moreover, its `RUnitTestData` object is optimized for quick testing, but not at all for easy reviewing of its content: it is a list of lists of lists,... requiring embedded for loop and `lapply()` / `sapply()` procedures to extract some content. Moreover, 'RUnit' is implemented as S4 classes, requiring the heavy 'methods' package to be loaded, making it impossible to use in a lightweight environment without it, or to test R without the 'methods' package loaded. Finally, the concept of test units as source-able files on disk is a nice idea, but it is too rigid for quick writing test cases for objects not associated (yet) with an R packages.

We did a first implementation of the 'RUnit' GUI based on these objects, before realizing that it is really not designed for such an use. So, we decide to write a completely different unit testing framework in R: 'svUnit', but we make it test code compatible with 'RUnit' (i.e., the engine and objects used are totally different, but the test code run in 'RUnit' or 'svUnit' is interchangeable).

Finally, 'svUnit' is also designed to be integrated in the SciViews
GUI on top of Komodo IDE (https://docs.activestate.com/komodo/12/), and to approach extreme programming practices with automatic code testing while you write it. A rather simple interface is provided to link and pilot 'svUnit' from any GUI/IDE, and the Komodo IDE implementation could be used as an example to program similar integration panels for other R GUIs. 'svUnit' also formats its report with *creole wiki* syntax. It is directly readable, but it can also be displayed in a much nicer way using any wiki engine compatible with the creole wiki language. It is thus rather easy to write test reports in wiki servers, possibly through nightly automatic process for your code, if you like.

This vignette is a guided tour of 'svUnit', showing its features
and the various ways you can use it to test your R code.

## Installation

The 'svUnit' package is available on CRAN (https://CRAN.R-project.org/package=svUnit), and its latest development version is also available on Github (https://github.com/SciViews/svUnit). You can install it with^[Install the development version with `devtools::install_github("SciViews/svUnit").]:


```{r, eval=FALSE}
install.packages("svUnit")
```

This package has no required dependencies other than R >= 1.9.0 and 'utils' ('XML' is also used for some export formats, but it is not required to run the tests). However, if you would like to use its interactive mode in a GUI editor, you must also install Komodo Edit or Komodo IDE and SciViews-K, or use RStudio. Its 'pkgdown' web site is at https://www.sciviews.org/svUnit/.

Once the 'svUnit' package is installed, you can check it is working correctly on your machine with the following example code:

```{r}
library(svUnit)
Square <- function(x) return(x^2)
test(Square) <- function() {
  checkEqualsNumeric(9, Square(3))
  checkEqualsNumeric(10, Square(3))   # This intentionally fails
  checkEqualsNumeric(9, SSSquare(3))  # This raises error
  checkEqualsNumeric(c(1, 4, 9), Square(1:3))
  checkException(Square("xx"))
}
clearLog()
(runTest(Square))
```

> Although test unit code is compatible with both 'svUnit' and 'RUnit', do not load both packages in R memory at the same
time, or you will badly mix incompatible code!

## Overview of 'svUnit'

You ensure that code you write in your R functions does the expected work by defining a battery of tests that will compare the output of your code with reference values. In 'svUnit', the simplest way to define such a battery of tests is by attaching it to functions loaded in R memory^[In fact, you can attach 'svUnit' tests to any kind of R object, not only function. This could be useful to test S3/S4 objects, or even, datasets.]. Of course, you can also define batteries of tests that are independent of any R object, or that check several of them together (so called,
*integration tests*). Here is a couple of examples:

```{r}
library(svUnit)
# Create two R functions that include their own test cases
Square <- function(x) return(x^2)
test(Square) <- function() {
  checkEqualsNumeric(9, Square(3))
  checkEqualsNumeric(c(4, 9), Square(2:3))
  checkException(Square("xx"))
}

Cube <- function(x) return(x^3)
test(Cube) <- function() {
  checkEqualsNumeric(27, Cube(3))
  checkEqualsNumeric(c(8, 28), Cube(2:3))
  checkException(Cube("xx"))
}

# Add a separate test case
test_Integrate <- svTest(function() {
  checkTrue(1 < 2, "check1")
  v <- c(1, 2, 3)  # The reference
  w <- 1:3         # The value to compare to the reference
  checkEquals(v, w)
})
```

When you run a test in 'svUnit', it logs its results in a centralized logger. The idea is to get a central repository for tests that you can manipulate as you like (print, summarize, convert, search, display in a GUI, etc.). If you want to start new tests, you should first clean this logger by `clearLog()`. At any time, the logger is accessible by `Log()`, and a summary of its content is displayed using `summary(Log()`). So, to run test for your `Square()` function as well as your `test_Integrate` integration test, you simply do the following:

```{r}
clearLog()
runTest(Square)
runTest(test_Integrate)
Log()
```

In this report, you see that all your tests succeed. Note that 'svUnit' is making the distinction between a test that **fails** (the code is run correctly, but the result is different from what was expected) and code that raises **error** (it was not possible to run the test because its code is incorrect, or for some other reasons). Note also that the function `checkException()` is designed to explicitly test code that should `stop()` in R^[`checkException()` can also track `warning()` messages with this little trick: first convert all warnings into errors with `owarn <- options(warn = 2)$warn`, run the code that should generate a warning inside `checkException()`, and then restore default warning behavior with `options(warn = owarn)`.]. So, if that test does not raises an exception, it is considered to have failed. This is useful to check that your functions correctly trap wrong arguments, for instance, like in `checkException(Square("xx"))` here above (a character string is provided where a numerical value is expected).

Now, let's look what happens if we test the `Cube()` function
without clearing the logger:

```{r}
runTest(Cube)
Log()
```

We note this:

1. When a test succeeds, nothing is printed by default (result is returned invisibly). But when a test fails, or raises errors, the guilty test(s) results are printed. We expected `c(8, 28)` (made intentionally wrong for the sake of the demonstration) from `Cube(2:3)` in `checkEqualsNumeric(c(8, 28), Cube(2:3))`, and (of course), we got `c(8, 27)`. This test shows how 'svUnit' presents test failures.
2. The results of the tests on `Cube()` are added to the previous report. So, it is possible to build rather easily reports that summarize tests on several objects, by adding test results in the logger sequentially. 'svUnit' does this naturally and transparently. Starting a new report is equally simple: just use `clearLog()`...

### Assertions in 'svUnit'

The most basic item in a test suite is an **assertion** represented by a `checkXXX()` function in 'svUnit'/'RUnit'. Five such functions are currently defined:

* `checkEquals(current, target)` determines if data in `target` is the same as data in `current`.
* `checkEqualsNumeric(current, target)` does the same but allows for a better comparison for numbers (variation allowed within a tolerance window).
* `checkIdentical(current, target)` checks whether two R objects are strictly identical.
* `checkTrue(expr)` only succeed if `expr` is `TRUE`. Note a difference in 'svUnit' and 'RUnit' (at least, in its version 0.4-17): the 'RUnit' function is not vectorized and `expr` must return a single atomic logical value. The corresponding 'svUnit' function also accepts a vector of logical values. In this case, all elements of the vector must be `TRUE` for the test to succeed. When you make sure that `expr` always returns a single logical value (for instance by using `all(expr)`), both functions should be compatible.
* `checkException(expr)` verifies that a given code raises an exception (in R, it means that a line of code with `stop()` is executed).
* `DEACTIVATED()` makes sure that all tests following this instruction (in a test function, see next paragraph) are deactivated, and inserts a reminder in the logger about the fact that some tests are deactivated in this suite.

For all these functions, you have an additional optional argument `msg =` where you can provide a (short) message to print in front of each text in the report. These functions return invisibly: `TRUE` if the test succeeds, or `FALSE` if it fails (code is executed correctly, but does not pass the test), and `NA` if there was an error (the R code of the test was not executed correctly). Moreover, these functions record the results, the context of the test and the timing in a logger (object `svSuiteData` inheriting from `environment`) called `.Log` and located in the user’s workspace. So, executing a series of assertions and getting a report is simply done as (in its simplest form, you can use the various `checkXXX()` functions directly at the command line):

```{r}
clearLog()
checkEqualsNumeric(1, log(exp(1)))
checkException(log("a"))
checkTrue(1 == 2)
Log()
```

As you can see, the `checkXXX()` functions work hand in hand with the test logger (the `checkXXX()` functions also return the result of the test invisibly, so, you can also assign it to a variable if you like). These function are mainly used for their side-effect of adding an entry to the logger.

The last command `Log()` prints the content of the logger. You see how a report is printed, with a first part being a short summary by categories (assertions run at the command line are placed automatically in the `eval` category: there is no better context known for them. Usually, those assertions should be placed in test functions, or in test units, as we will see later in this manual, and the category will reflect this organization). A detailed report on the tests that failed or raised an error is also printed at the end of the report.

Of course, the same report is much easier to manipulate from within the graphical tree in the Komodo’s **R Unit** tab, but the simple text report in R has the advantage of being independent from any GUI, and from Komodo. It can also be generated in batch mode. Last, but not least, it uses a general Wiki formatting called creole wiki (https://moin-20.readthedocs.io/en/latest/user/creolewiki.html). 

!['svUnit' test report as it appears when inserted in a wiki page (DokuWiki engine with the creole plugin installed). Note the summary of results at the top left of the page, and the clickable table of contents with detailed entries to easily navigate to the test log you are consulting). Timing of the test is also clearly indicated, since it is a complementary but important information (if a test succeeds, but calculation is way too long, it is good to know it)!](svUnit_wikiReport.png)

The figure illustrates the way the same report looks like in DokuWiki with the creole plugin (https://www.dokuwiki.org/plugin:creole) installed. Note the convenient table of content that lists here a clickable list of all tests run. From this point, it is relatively easy to define nightly cron task jobs on a server to run a script that executes these tests and update a wiki page (look at your particular wiki engine documentation to determine how you can access wiki pages on the command line).

### Manipulating the logger data

The 'svUnit' package provides a series of functions to manipulate the logger from the command line, in particular, `stats()`,`summary()`, `metadata()` and `ls()`:

```{r}
# Clear test exclusion list for running all test suites
options(svUnit.excludeList = NULL)
# Clear the logger
clearLog()
# Run all currently defined tests
runTest(svSuiteList(), name = "AllTests")
# Get some statistics
stats(Log())[, 1:3]
# A slightly different presentation than with print
summary(Log())
# Metadata collected on the machine where tests are run
metadata(Log())
# List content of the log
ls(Log())
```

As you can see, `ls()` lists all components recorded in the test suite. Each component is a `svTestData` object inheriting from `data.frame`, and it can be easily accessed through the `$` operator. There are, of course similar methods defined for those `svTestData` objects, like `print()`, `summary()` and `stats()`:

```{r}
myTest <- Log()$testCube
class(myTest)
myTest
summary(myTest)
stats(myTest)
```

As the logger inherits from `environment`, you can manage individual test data the same way as objects in any other environment. For instance, if you want to delete a particular test data without touching to the rest, you can use:

```{r}
ls(Log())
rm(test_R, envir = Log())
ls(Log())
```

As we will see in the following section, 'svUnit' proposes several means to organize individual assertions in modules: **test functions**, **test units** and **test suites**. This organization is inspired from 'RUnit', but with additional ways of using tests in interactive sessions (for instance, the ability to attach a test to the objects to be tested).

### Test function

The first organization level for grouping assertions together is the **test function**. A test function is a function without arguments whose name must start with `test`. It typically contains a series of assertions applied to one object, method, or function to be checked (this is not obligatory, assertions are not restricted to one object, but good practices strongly suggest such a restriction). Here is an example:

```{r}
test_function <- function() {
  checkTrue(1 < 2, "check1")
  v <- c(1, 2, 3)  # The reference
  w <- 1:3         # The object to compare to the reference
  checkEqualsNumeric(v, w)
}
# Turn this function into a test
test_function <- as.svTest(test_function)
is.svTest(test_function)
```

A test function should be made a special object called `svTest`, so that 'svUnit' can recognize it. This `svTest` object, is allowed to live on its own (for instance, in the user's workspace, or anywhere you like). It can be defined in a R script, be saved in a `.RData` file, etc... Note that this is very different from 'RUnit' where test must always be located in a unit test file on disk). In 'svUnit' (not 'RUnit'), you run such a test simply by using `runTest()`, which returns the results invisibly and add it to the logger:

```{r}
clearLog()
runTest(test_function)
Log()
```

Now, a test function is most likely designed to test an R object. The 'svUnit' package also provides facilities to attach the test function to the object to be tested. Hence, the test cases and the tested object conveniently form a single entity that one can manipulate, copy, save, reload, etc. with all the usual tools in R. This association is simply made using `test(myobj) <-`:

```{r}
# A very simple function
Square <- function(x) return(x^2)

# A test case to associate with the Square() function
test(Square) <- function() {
  checkEqualsNumeric(9, Square(3))
  checkEqualsNumeric(c(1, 4, 9), Square(1:3))
  checkException(Square("xx"))
}
is.test(Square)  # Does this object contain tests?
```

One can retrieve the test associated with the object by using:

```{r}
test(Square)
```

And of course, running the test associated with an object is as easy as:

```{r}
runTest(Square)
Log()  # Remember we didn't clear the log!
```

Now that you master test functions, you will discover how you can group them in logical **units**, and associate them to R packages.

### Test units

An **unit** is a coherent piece of software that can be tested separately from the rest. Typically, a R package is a structured way to compile and distribute such code units in R. Hence, we need a mean to organize tests related to this "unit" conveniently.

Since a package can contain several functions, data frames, or other objects, our unit should collect together individual test functions related to each of these objects that compose our package. Also, the test unit should accommodate the well-define organization of a package, and should integrate in the already existing testing features of R, in particular, `R CMD check`. In both 'RUnit', and 'svUnit', one can define such test units, and they are made code compatible between the two implementations.

A test unit is a source-able text file that contains one or more test functions, plus possibly `.setUp()` and `.tearDown()` functions (see the online help for further information on these special functions). In 'RUnit', you must write such test unit files from scratch. With 'svUnit', you can "promote" one or several test functions (associated to other objects, or "living" alone as separate `svTest` objects) by using `makeUnit()`. Here is how you promote the test associated with our `Square()` function to a simple test unit containing only one test function:

```{r, eval=FALSE}
# Create a test unit on disk and view its content
unit <- makeUnit(Square)
file.show(unit, delete.file = TRUE)
```

You got the following file whose name must start with `runit`, with an `.R` extension (`runit*.R`), and located by default in the temporary directory of R. Specify another directory with the `dir =` argument of `makeUnit()` for a more permanent record of this test unit file. Note also that `.setUp()` and `.tearDown()` functions are constructed automatically for you. They specify the context of these tests. This context is used, for instance, by the GUI in Komodo Edit/IDE to locate the test function and the code being tested.

```
## Test unit 'Square'
.setUp <- function() {
  # Specific actions for svUnit: prepare context
  if ("package:svUnit" %in% search()) {
    .Log <- Log() # Make sure .Log is created
    .Log$..Unit <- "/tmp/RtmpBoZnId/runitSquare.R"
    .Log$..File <- ""
    .Log$..Obj <- ""
    .Log$..Tag <- ""
    .Log$..Msg <- ""
    rm(..Test, envir = .Log)
  }
}

.tearDown <- function() {
  # Specific actions for svUnit: clean up context
  if ("package:svUnit" %in% search()) {
    .Log$..Unit <- ""
    .Log$..File <- ""
    .Log$..Obj <- ""
    .Log$..Tag <- ""
    .Log$..Msg <- ""
    rm(..Test, envir = .Log)
  }
}

testSquare <- function() {
  checkEqualsNumeric(9, Square(3))
  checkEqualsNumeric(c(1, 4, 9), Square(1:3))
  checkException(Square("xx"))
}
```

Compatibility of these test unit files between 'RUnit' and 'svUnit' was a major concern in the design of 'svUnit'. Consequently, code specific to 'svUnit' (for managing the context of the tests) is embedded in a `if ("package:svUnit" %in% search())` construct. That way, if 'svUnit' is not loaded in memory, this code is not executed. *Note that you should avoid loading in memory both 'svUnit' and 'RUnit' at the same time! If you do so, you will most likely crash your tests.*

You will see further that it is possible to write much more complex test units with the same `makeUnit()` function. But for the moment, let’s discuss a little bit how such test units should be organized in R package.

If you intend to associate test units to your R package, you should respect the following conventions:

* Name your test units `runit*.R`.
* Place them in the `/inst/unitTests` subdirectory of the package sources, or in one of its subdirectories. If you place them in a subdirectory of `/inst/unitTests`, then you define secondary unit tests for (optional) detailed testing of specific item in your package. Always keep in mind that all `runit*.R` files in a directory will be run one after the other. So, if you want to make subgroups you would like to dissociate them, and locate them is separate subdirectories.
* When the package will be compiled, all these test units will be located in `/unitTests` in the compiled/installed version of your R package.

If you respect these conventions, 'svUnit' knows where package unit tests are located and will be able to find and run them quite easily. See, for instance, the examples in the 'svUnit' package.

So, with test units associated to packages, you have a very convenient way to run these tests, including from the Komodo GUI. With just a little bit more coding you can also include these test units in the `R CMD check` process of your packages. You do this by means of examples in a help page (we prefer to use **examples**, instead of `/tests` in the `R CMD check` process, because examples offer a more flexible way to run tests and you can also run them in interactive sessions through the `example()` R function, which is not the case for code located in the `/tests` subdirectory of your package). Here is what you do to associate some or all of your unit tests to the `R CMD check` process (illustrated with the case of the 'svUnit' package itself):

* Define a `.Rd` help file in the `/man` subdirectory called `unitTests.<mypackage>.Rd` where <mypackage> is the name of your package (or whatever name you prefer).
* Fill the `.Rd` file, making sure that you define an alias as `unitTests.<mypackage>`. Also place a little bit of information telling how users can run your test in an interactive session.
* The important part of this file is, of course, the `\examples{}` section. You must first clear the log, then run each test, and then, call the `errorLog()` function. That function looks if one or more tests failed or raised an error. In this case, it stops execution of the example and causes a dump of the test log in the `R CMD check` process. That way, providing that you have the 'svUnit' package installed in the machine where you run `R CMD check`, your test units will be included nicely in the checking process of your packages, that is, they will run silently each time you check your package if no error occurs, but will produce a detailed report in case of problems.
* Here is how your `.Rd` file should looks like (example of the 'svUnit' package):

```
\name{unitTests.svUnit}
\alias{unitTests.svUnit}
\title{ Unit tests for the package 'svUnit' }
\description{ Performs unit tests defined in this
  package by running \code{example(unitTests.svUnit)}.
  Tests are in \code{runit*.R} files Located in the
  ’/unitTests’ subdirectory or one of its
  subdirectories (’/inst/unitTests’ and subdirectories
  in package sources).
}
\author{Philippe Grosjean}
\keyword{utilities}
\examples{
if (require(svUnit)) {
  clearLog()
  runTest(svSuite("package:svUnit"), "svUnit")
  \donttest{
  # Tests to run with example() but not with R CMD check
  runTest(svSuite("package:svUnit (VirtualClass)"), "VirtualClass")
  }
  \dontrun{
  # Tests to present in ?unitTests.svUnit but not run automatically
  # Run all currently loaded packages test cases and test suites
  runTest(svSuiteList(), "AllTests")
  }
  \dontshow{
  # Put here test units you want to run during R CMD check but
  # don't want to show or run with example(unitTests.svUnit)
  }
  # Check errors at the end (needed to interrupt R CMD check)
  errorLog()
}
}
```

Alternatively, if you use 'roxygen2' to build your documentation, you can add a `unitTests.<pkgname>.R` file in the `/R` subdirectory containing ROxygen code to generate the adequate documentation. Here is the `unitTests.svUnit.R` file:

```r
#' Unit tests for the package 'svUnit'
#'
#' Performs unit tests defined in this package by running
#' `example(unitTests.svUnit)`. Tests are in `runit*.R` files located in the
#' '/unitTests' subdirectory or one of its subdirectories ('/inst/unitTests' and
#' subdirectories in package sources).
#'
#' @name unitTests.svUnit
#' @author Philippe Grosjean
#' @keywords utilities
#' @examples
#' if (require(svUnit)) {
#'   clearLog()
#'   runTest(svSuite("package:svUnit"), "svUnit")
#'   \donttest{
#'   # Tests to run with example() but not with R CMD check
#'   runTest(svSuite("package:svUnit (VirtualClass)"), "VirtualClass")
#'   }
#'   \dontrun{
#'   # Tests to present in ?unitTests.svUnit but not run automatically
#'   # Run all currently loaded packages test cases and test suites
#'   runTest(svSuiteList(), "AllTests")
#'   }
#'   \dontshow{
#'   # Put here test units you want to run during R CMD check but
#'   # don't want to show or run with example(unitTests.svUnit)
#'   }
#'   # Check errors at the end (needed to interrupt R CMD check)
#'   errorLog()
#' }
NULL
```

Note, however, that if the package 'svUnit' is not available on the computer where you run `R CMD check`, your tests are silently ignored (`require()` issues a warning, but that does not prevent the checking process to continue). This is an intended feature in order to allow compilation of your package without requiring 'svUnit'. Hence, dependence to 'svUnit' is less strict and also allows you to check your tests using 'RUnit' (but you have to write a dedicated function for that). Still to keep such a less strict dependence on 'svUnit', you should add `svUnit` in the `Suggests:` field in the `DESCRIPTION` file of your package, not in `Depends:` or `Imports:` fields (except if you use 'svUnit' for other purposes that testing your package using the mechanism discussed here, of course).

Also, this approach, associated with examples, provides a very convenient and easy way to test a package from the command line in an interactive session by running:

```{r}
example(unitTests.svUnit)
```

In the present case, the `errorLog()` instruction in the examples returns nothing, because all tests succeed. If there is an error somewhere, you will see it printed at the end of this example.

### Test suites: collections of test functions and units

The highest level of organization of your tests is the **test suite**. A test suite is an unordered collection of test functions and test units. You can select test units associated with R package in a very convenient way: just specify `package:myPkg` and all test units in the `/unitTests` subdirectory of the package myPkg will be included ('svUnit' does all the required work to map these to actual directories where the test unit files are located). Also, if you specify `package:myPkg(subgroup)`, you will include the test units defined in `/unitTests/subgroup` in the package myPkg. Of course, you will be able to also add test units defined in custom directories, outside of R packages (for instance for integration of *harness tests* that check cross-packages, or multi-packages features of your application).

Test functions associated to your test suite receive a special treatment. Unlike `runTest()` applied to a single test function, or to an object that has an associated test function, these tests are not run from the version loaded in memory. Instead, they are first collected together in a test unit file on disk (located in the R temporary directory, by default), and run from there. Hence, building a more complex test unit file by collecting together several test functions is just a question of constructing a test suite, and then, applying the `makeUnit()` function to this `svSuite` object.

Before we apply all this, you should also know the existence of one more function: `svSuiteList()`. This function lists all test units and test functions available in your system at a given time. So, you don’t need to manually create lists of components. You are better to list them automatically. Of course, this function has a lot of arguments for listing only test units in packages, only test functions, specifying where (in which environment) the test functions are located, adding custom directories where to look for test units, etc. See the online help of this function for the description of all these arguments. One argument is particularly important: `excludeList =`. This argument defines one or several regular expressions that are used as filters to hide items from the list. This is required, since you will certainly not want to run again and again, let’s say, the example tests associated with the 'svUnit' package ('svUnit' must be loaded in memory to run the tests, so its tests examples will always be listed by `svSuiteList()`, ... unless you define an adequate filter expression that will exclude them from your list)! As the default argument suggests it, the regular expression for list exclusion could also be recorded in `options(svUnit.excludeList = ...)`.
Here is how it works:

```{r}
# Reset default exclusion list
options(svUnit.excludeList = c("package:sv", "package:RUnit"))
# List all currently available tests
svSuiteList()
```

Thus, every entry matching the regular expressions `package:sv` and `package:RUnit` are excluded from the list. The entries `package:svUnit` and `package:svUnit (VirtualClass)` match first pattern and are thus excluded. Now, let's clear the exclusion list to see what happens:

```{r}
# Clear exclusion list
options(svUnit.excludeList = NULL)
svSuiteList()
```

The test units associated with the package 'svUnit' are now listed. You have noticed that `svSuiteList()` can also find automatically `svTest` objects, as well as tests attached to objects in the user's workspace. You can create a suite by collecting all these items together very easily:

```{r}
(mySuite <- svSuiteList())
```

Now let’s make a test unit using tests collected in this suite:

```{r, eval=FALSE}
myUnit <- makeUnit(mySuite, name = "ExampleTests")
file.show(myUnit, delete.file = TRUE)
```

This produces a file named `runitExampleTests.R` located (by default) in the R temporary directory, and which collects together all tests in the user's workspace (either as `svTest` objects, or as tests attached to other objects), plus tests suites in packages that are **not** in the exclusion list. Running all tests in your suite is very simple. You still use `runTest()` as usual,
but this time, you apply it to your `svSuite` object:

```{r}
clearLog()
runTest(mySuite)
summary(Log())
```

There are many other tools to manipulate `svSuite` objects in the 'svUnit' package, including functions to define the content of the suite manually (see online help).

### Using 'svUnit' with RStudio

Except by integrating your tests in a packages as explained above in this document, nothing is required to make your tests run in RStudio. The `Build -> Test Package` menu entry should run your tests with 'svUnit' once this integration is done.

### Using 'svUnit' with SciViews Komodo

If you use the SciViews Komodo GUI, you can integrate 'svUnit' tests in this IDE and display reports in a convenient hierarchical tree presentation.

If R is started from within Komodo Edit or IDE (with the SciViews-K and SciViews-K Unit plugins installed), then, loading the 'svUnit' package in R automatically installs the **R Unit** side panel in Komodo at right. Its use should be straightforward:

![Komodo Edit with SciViews-K and SciViews-K Unit after running tests. At right: the **R Unit** panel that display (at the top) the list of available tests, and test units where you can select the ones to run, and at the bottom, a tree with the results from last tests run. The stripe at the very top is green if all tests succeed, and red (as here), if at least one tests failed or raised an error.](svUnit_KomodoRUnit.png)

* Select the tests you want to run in the top part,
* Click the `Run` button each time you want to refresh the test
tree,
* Browse the tree for failures or errors (the color at the top of the panel immediately indicates if there is a problem somewhere: green -> everything is fine, red -> there is a problem).
* If you have failures or errors, move the mouse on top of the corresponding item in the tree, and you got more information displayed in a tooltip,
* Click on an item to open the test unit at that place in a buffer in Komodo.

The `Auto` mode, when activated, sources R files currently edited in Komodo whenever you save them, and then, refreshes the test report tree. This mode allows you to run automatically your tests in the background while you edit your R code!

> If you want to implement such a side panel in another GUI, make sure to look at the `koUnit\_XXX()` functions in the 'svUnit' package. These functions allow to control the GUI in Komodo remotely from within R, and similar functions should not be too difficult to implement for other GUIs.

## References

* Grosjean, Ph., 2003. SciViews: an object-oriented abstraction layer to design GUIs on top of various calculation kernels [online: https://www.r-project.org/conferences/DSC-2003/]

* IEEE Standards Boards, 1993. IEEE standard for software unit testing. ANSI/IEEE Std 1008-1987. 24 pp.

* Ihaka R. & R. Gentleman, 1996. R: a language for data analysis and graphics. *J. Comput. Graphic. Stat.*, 5:299-314.

* Jeffries, R., 2006. Extreme programming, web site at: https://ronjeffries.com/xprog/what-is-extreme-programming/.

* König, T., K. Jünemann & M. Burger, 2007. RUnit – A unit test framework for R. Vignette of the package RUnit available on CRAN. 11 pp.

* R Core Team, 2019. R: A language and environment for statistical computing. R Foundation for Statistical Computing, Vienna, Austria. [online: https://www.R-project.org]

---
title: "svUnit - A framework for unit testing in R"
author: "Philippe Grosjean (phgrosjean@sciviews.org)"
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    fig_caption: yes
vignette: >
  %\VignetteIndexEntry{svUnit - A framework for unit testing in R}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

## Introduction

Unit testing (see https://en.wikipedia.org/wiki/Unit_test) is an approach successfully used to develop software, and to ease code refactoring for keeping bugs to the minimum. It is also the insurance that the software is doing the right calculation (quality insurance). Basically, a test just checks if the code is running and is producing the correct answer/behavior in a given situation. As such, unit tests are build in R package production because all examples in documentation files, and perhaps, test code in `/tests` subdirectory are run during the checking of a package (`R CMD check <pkg>`). However, the R approach lacks a certain number of features to allow optimal use of unit tests as in extreme programming (test first – code second):

* Tests are related to package compilation and cannot easily be run independently (for instance, for functions developed separately).
* Once a test fails, the checking process is interrupted. Thus one has to correct the bug and launch package checking again... and perhaps get caught by the next bug. It is a long and painful process.
* There is no way to choose one or several tests selectively: all are run or not (depending on command line options) during package checking.
* It is very hard, or near to impossible to program in R in a test driven development (*write tests first*) with the standard tools (https://en.wikipedia.org/wiki/Test-driven_development).
* Consequently, the '*test-code-simplify*' cycle is not easily accessible yet to R programmer, because of the lack of an interactive and flexible testing mechanism providing immediate, or quasi immediate feedback about changes made.
* We would like also to emphasize that test suites are not only useful to check code, they can also be used to check data, or the pertinence of analyses.

## Unit testing in R without 'svUnit'

Besides the "regular" testing mechanism of R packages, one can find the 'testthat' and 'RUnit' packages on CRAN (https://CRAN.R-project.org/package=testthat and https://CRAN.R-project.org/package=RUnit, respectively)^[There is also a very simple testing system in 'testit' (https://CRAN.R-project.org/package=testit) using only one assertion: `assert()` for all test. Other interesting R packages on CRAN are 'testthis' (https://CRAN.R-project.org/package=testthis), which adds more tools and an RStudio addins around 'testthat', and 'runittotestthat' (https://CRAN.R-project.org/package=runittotestthat) which, as its name implies, automatically converts 'RUnit' tests into 'testthat' equivalents.]. Another package used to provide an alternate implementation of test unit: 'butler', but it is archived because it is not maintained any more and has given up in favor of 'RUnit'.

The 'testthat' package is heavily used by the UseR community (there are many more packages on CRAN that use it than any other testing system). It implements R testing in a rather original way. 'RUnit', on the other hand sticks with the framework initially defined by junit for Java and largely developed for many different languages. In this document, we will not discuss further about 'testthat'. We will focus on the junit-like framework implemented in 'RUnit' and 'svUnit'. 'RUnit' implements the following features:

* **Assertions**, `checkEquals()`, `checkEqualsNumeric()`, `checkIdentical()` and `checkTrue()` and negative tests
(tests that check error conditions, `checkException()`).
* Assertions are grouped into R functions to form one **test function** that runs a series of related individual tests. It is easy to temporarily inactivate one or more tests by commenting lines in the function. To avoid forgetting tests that are commented out later on, there is special function, named `DEACTIVATED()` that tags the test with a reminder for your deactivated items (i.e., the reminder is written in the test log).
* A series of test functions (whose name typically start with `test....`) are collected together in a source-able R code file (name starting with `runit....`) on disk. This file is called a **test unit**.
* A **test suite** (object `RUnitTestSuite`) is a special object defining a battery of tests. It points to one or several directories containing test units. A test suite is defined by `defineTestSuite()`.
* One or more test suites can be run by calling `runTestSuite()`. There is a shortcut to define and run a test suite constituted by only one test unit by using the function `runTestFile()`. Once the test is run, a `RUnitTestData` object is created that contains all the information collected from the various tests run.
* One can print a synthetic report (how many test units, test functions, number of errors, fails and deactivated item), or get a more extensive `summary()` of the tests with indication about which ones failed or produced errors. The function `printTextProtocol()` does the same, while `printHTMLProtocol()` produces a report in HTML format.
* 'RUnit' contains also functions to determine which code is run in the original function when tested, in order to detect the parts of the code not covered by the test suite (code coverage function
`inspect()` and function `tracker()`).

As complete and nice as 'RUnit' is, there is no tools to integrate the test suite in a given development environment (IDE) or graphical user interface (GUI), as far as we know. In particular, there is no real-time reporting mechanism used to easy the *test-code-simplify* cycle. The way tests are implemented and run is left to the user, but the implementation suggests that the authors of 'RUnit' mainly target batch execution of the tests (for instance, nightly check of code in a server), rather that real-time interaction with the tests.

There is also no integration with the "regular" `R CMD check` mechanism of R in 'RUnit'.

## Unit testing framework for R with 'svUnit'

Our initial goal was to implement a GUI layer on top of 'RUnit', and to integrate test units as smoothly as possible in a code editor, as well as, making tests easily accessible and fully compatible with `R CMD check` on all platforms supported by R. Ultimately, the test suite should be easy to create, to use interactively, and should be able to test functions in a complex set of R packages.

However, we encountered several difficulties while trying to enhance 'RUnit' mechanism. When we started to work on this project, 'RUnit' (version 0.4-17) did not allow to subclass its objects (version 0.4-19 solved this problem). Moreover, its `RUnitTestData` object is optimized for quick testing, but not at all for easy reviewing of its content: it is a list of lists of lists,... requiring embedded for loop and `lapply()` / `sapply()` procedures to extract some content. Moreover, 'RUnit' is implemented as S4 classes, requiring the heavy 'methods' package to be loaded, making it impossible to use in a lightweight environment without it, or to test R without the 'methods' package loaded. Finally, the concept of test units as source-able files on disk is a nice idea, but it is too rigid for quick writing test cases for objects not associated (yet) with an R packages.

We did a first implementation of the 'RUnit' GUI based on these objects, before realizing that it is really not designed for such an use. So, we decide to write a completely different unit testing framework in R: 'svUnit', but we make it test code compatible with 'RUnit' (i.e., the engine and objects used are totally different, but the test code run in 'RUnit' or 'svUnit' is interchangeable).

Finally, 'svUnit' is also designed to be integrated in the SciViews
GUI on top of Komodo IDE (https://www.activestate.com/products/komodo-ide/), and to approach extreme programming practices with automatic code testing while you write it. A rather simple interface is provided to link and pilot 'svUnit' from any GUI/IDE, and the Komodo IDE implementation could be used as an example to program similar integration panels for other R GUIs. 'svUnit' also formats its report with *creole wiki* syntax. It is directly readable, but it can also be displayed in a much nicer way using any wiki engine compatible with the creole wiki language. It is thus rather easy to write test reports in wiki servers, possibly through nightly automatic process for your code, if you like.

This vignette is a guided tour of 'svUnit', showing its features
and the various ways you can use it to test your R code.

## Installation

The 'svUnit' package is available on CRAN (https://CRAN.R-project.org/package=svUnit), and its latest development version is also available on Github (https://github.com/SciViews/svUnit). You can install it with^[Install the development version with `devtools::install_github("SciViews/svUnit").]:


```{r, eval=FALSE}
install.packages("svUnit")
```

This package has no required dependencies other than R >= 1.9.0 and 'utils' ('XML' is also used for some export formats, but it is not required to run the tests). However, if you would like to use its interactive mode in a GUI editor, you must also install Komodo Edit or Komodo IDE and SciViews-K, or use RStudio. Its 'pkgdown' web site is at https://www.sciviews.org/svUnit/.

Once the 'svUnit' package is installed, you can check it is working correctly on your machine with the following example code:

```{r}
library(svUnit)
Square <- function(x) return(x^2)
test(Square) <- function() {
  checkEqualsNumeric(9, Square(3))
  checkEqualsNumeric(10, Square(3))   # This intentionally fails
  checkEqualsNumeric(9, SSSquare(3))  # This raises error
  checkEqualsNumeric(c(1, 4, 9), Square(1:3))
  checkException(Square("xx"))
}
clearLog()
(runTest(Square))
```

> Although test unit code is compatible with both 'svUnit' and 'RUnit', do not load both packages in R memory at the same
time, or you will badly mix incompatible code!

## Overview of 'svUnit'

You ensure that code you write in your R functions does the expected work by defining a battery of tests that will compare the output of your code with reference values. In 'svUnit', the simplest way to define such a battery of tests is by attaching it to functions loaded in R memory^[In fact, you can attach 'svUnit' tests to any kind of R object, not only function. This could be useful to test S3/S4 objects, or even, datasets.]. Of course, you can also define batteries of tests that are independent of any R object, or that check several of them together (so called,
*integration tests*). Here is a couple of examples:

```{r}
library(svUnit)
# Create two R functions that include their own test cases
Square <- function(x) return(x^2)
test(Square) <- function() {
  checkEqualsNumeric(9, Square(3))
  checkEqualsNumeric(c(4, 9), Square(2:3))
  checkException(Square("xx"))
}

Cube <- function(x) return(x^3)
test(Cube) <- function() {
  checkEqualsNumeric(27, Cube(3))
  checkEqualsNumeric(c(8, 28), Cube(2:3))
  checkException(Cube("xx"))
}

# Add a separate test case
test_Integrate <- svTest(function() {
  checkTrue(1 < 2, "check1")
  v <- c(1, 2, 3)  # The reference
  w <- 1:3         # The value to compare to the reference
  checkEquals(v, w)
})
```

When you run a test in 'svUnit', it logs its results in a centralized logger. The idea is to get a central repository for tests that you can manipulate as you like (print, summarize, convert, search, display in a GUI, etc.). If you want to start new tests, you should first clean this logger by `clearLog()`. At any time, the logger is accessible by `Log()`, and a summary of its content is displayed using `summary(Log()`). So, to run test for your `Square()` function as well as your `test_Integrate` integration test, you simply do the following:

```{r}
clearLog()
runTest(Square)
runTest(test_Integrate)
Log()
```

In this report, you see that all your tests succeed. Note that 'svUnit' is making the distinction between a test that **fails** (the code is run correctly, but the result is different from what was expected) and code that raises **error** (it was not possible to run the test because its code is incorrect, or for some other reasons). Note also that the function `checkException()` is designed to explicitly test code that should `stop()` in R^[`checkException()` can also track `warning()` messages with this little trick: first convert all warnings into errors with `owarn <- options(warn = 2)$warn`, run the code that should generate a warning inside `checkException()`, and then restore default warning behavior with `options(warn = owarn)`.]. So, if that test does not raises an exception, it is considered to have failed. This is useful to check that your functions correctly trap wrong arguments, for instance, like in `checkException(Square("xx"))` here above (a character string is provided where a numerical value is expected).

Now, let's look what happens if we test the `Cube()` function
without clearing the logger:

```{r}
runTest(Cube)
Log()
```

We note this:

1. When a test succeeds, nothing is printed by default (result is returned invisibly). But when a test fails, or raises errors, the guilty test(s) results are printed. We expected `c(8, 28)` (made intentionally wrong for the sake of the demonstration) from `Cube(2:3)` in `checkEqualsNumeric(c(8, 28), Cube(2:3))`, and (of course), we got `c(8, 27)`. This test shows how 'svUnit' presents test failures.
2. The results of the tests on `Cube()` are added to the previous report. So, it is possible to build rather easily reports that summarize tests on several objects, by adding test results in the logger sequentially. 'svUnit' does this naturally and transparently. Starting a new report is equally simple: just use `clearLog()`...

### Assertions in 'svUnit'

The most basic item in a test suite is an **assertion** represented by a `checkXXX()` function in 'svUnit'/'RUnit'. Five such functions are currently defined:

* `checkEquals(current, target)` determines if data in `target` is the same as data in `current`.
* `checkEqualsNumeric(current, target)` does the same but allows for a better comparison for numbers (variation allowed within a tolerance window).
* `checkIdentical(current, target)` checks whether two R objects are strictly identical.
* `checkTrue(expr)` only succeed if `expr` is `TRUE`. Note a difference in 'svUnit' and 'RUnit' (at least, in its version 0.4-17): the 'RUnit' function is not vectorized and `expr` must return a single atomic logical value. The corresponding 'svUnit' function also accepts a vector of logical values. In this case, all elements of the vector must be `TRUE` for the test to succeed. When you make sure that `expr` always returns a single logical value (for instance by using `all(expr)`), both functions should be compatible.
* `checkException(expr)` verifies that a given code raises an exception (in R, it means that a line of code with `stop()` is executed).
* `DEACTIVATED()` makes sure that all tests following this instruction (in a test function, see next paragraph) are deactivated, and inserts a reminder in the logger about the fact that some tests are deactivated in this suite.

For all these functions, you have an additional optional argument `msg =` where you can provide a (short) message to print in front of each text in the report. These functions return invisibly: `TRUE` if the test succeeds, or `FALSE` if it fails (code is executed correctly, but does not pass the test), and `NA` if there was an error (the R code of the test was not executed correctly). Moreover, these functions record the results, the context of the test and the timing in a logger (object `svSuiteData` inheriting from `environment`) called `.Log` and located in the user’s workspace. So, executing a series of assertions and getting a report is simply done as (in its simplest form, you can use the various `checkXXX()` functions directly at the command line):

```{r}
clearLog()
checkEqualsNumeric(1, log(exp(1)))
checkException(log("a"))
checkTrue(1 == 2)
Log()
```

As you can see, the `checkXXX()` functions work hand in hand with the test logger (the `checkXXX()` functions also return the result of the test invisibly, so, you can also assign it to a variable if you like). These function are mainly used for their side-effect of adding an entry to the logger.

The last command `Log()` prints the content of the logger. You see how a report is printed, with a first part being a short summary by categories (assertions run at the command line are placed automatically in the `eval` category: there is no better context known for them. Usually, those assertions should be placed in test functions, or in test units, as we will see later in this manual, and the category will reflect this organization). A detailed report on the tests that failed or raised an error is also printed at the end of the report.

Of course, the same report is much easier to manipulate from within the graphical tree in the Komodo’s **R Unit** tab, but the simple text report in R has the advantage of being independent from any GUI, and from Komodo. It can also be generated in batch mode. Last, but not least, it uses a general Wiki formatting called creole wiki (http://www.wikicreole.org/wiki/Creole1.0). 

!['svUnit' test report as it appears when inserted in a wiki page (DokuWiki engine with the creole plugin installed). Note the summary of results at the top left of the page, and the clickable table of contents with detailed entries to easily navigate to the test log you are consulting). Timing of the test is also clearly indicated, since it is a complementary but important information (if a test succeeds, but calculation is way too long, it is good to know it)!](svUnit_wikiReport.png)

The figure illustrates the way the same report looks like in DokuWiki with the creole plugin (http://www.wikicreole.org/wiki/DokuWiki) installed. Note the convenient table of content that lists here a clickable list of all tests run. From this point, it is relatively easy to define nightly cron task jobs on a server to run a script that executes these tests and update a wiki page (look at your particular wiki engine documentation to determine how you can access wiki pages on the command line).

### Manipulating the logger data

The 'svUnit' package provides a series of functions to manipulate the logger from the command line, in particular, `stats()`,`summary()`, `metadata()` and `ls()`:

```{r}
# Clear test exclusion list for running all test suites
options(svUnit.excludeList = NULL)
# Clear the logger
clearLog()
# Run all currently defined tests
runTest(svSuiteList(), name = "AllTests")
# Get some statistics
stats(Log())[, 1:3]
# A slightly different presentation than with print
summary(Log())
# Metadata collected on the machine where tests are run
metadata(Log())
# List content of the log
ls(Log())
```

As you can see, `ls()` lists all components recorded in the test suite. Each component is a `svTestData` object inheriting from `data.frame`, and it can be easily accessed through the `$` operator. There are, of course similar methods defined for those `svTestData` objects, like `print()`, `summary()` and `stats()`:

```{r}
myTest <- Log()$testCube
class(myTest)
myTest
summary(myTest)
stats(myTest)
```

As the logger inherits from `environment`, you can manage individual test data the same way as objects in any other environment. For instance, if you want to delete a particular test data without touching to the rest, you can use:

```{r}
ls(Log())
rm(test_R, envir = Log())
ls(Log())
```

As we will see in the following section, 'svUnit' proposes several means to organize individual assertions in modules: **test functions**, **test units** and **test suites**. This organization is inspired from 'RUnit', but with additional ways of using tests in interactive sessions (for instance, the ability to attach a test to the objects to be tested).

### Test function

The first organization level for grouping assertions together is the **test function**. A test function is a function without arguments whose name must start with `test`. It typically contains a series of assertions applied to one object, method, or function to be checked (this is not obligatory, assertions are not restricted to one object, but good practices strongly suggest such a restriction). Here is an example:

```{r}
test_function <- function() {
  checkTrue(1 < 2, "check1")
  v <- c(1, 2, 3)  # The reference
  w <- 1:3         # The object to compare to the reference
  checkEqualsNumeric(v, w)
}
# Turn this function into a test
test_function <- as.svTest(test_function)
is.svTest(test_function)
```

A test function should be made a special object called `svTest`, so that 'svUnit' can recognize it. This `svTest` object, is allowed to live on its own (for instance, in the user's workspace, or anywhere you like). It can be defined in a R script, be saved in a `.RData` file, etc... Note that this is very different from 'RUnit' where test must always be located in a unit test file on disk). In 'svUnit' (not 'RUnit'), you run such a test simply by using `runTest()`, which returns the results invisibly and add it to the logger:

```{r}
clearLog()
runTest(test_function)
Log()
```

Now, a test function is most likely designed to test an R object. The 'svUnit' package also provides facilities to attach the test function to the object to be tested. Hence, the test cases and the tested object conveniently form a single entity that one can manipulate, copy, save, reload, etc. with all the usual tools in R. This association is simply made using `test(myobj) <-`:

```{r}
# A very simple function
Square <- function(x) return(x^2)

# A test case to associate with the Square() function
test(Square) <- function() {
  checkEqualsNumeric(9, Square(3))
  checkEqualsNumeric(c(1, 4, 9), Square(1:3))
  checkException(Square("xx"))
}
is.test(Square)  # Does this object contain tests?
```

One can retrieve the test associated with the object by using:

```{r}
test(Square)
```

And of course, running the test associated with an object is as easy as:

```{r}
runTest(Square)
Log()  # Remember we didn't clear the log!
```

Now that you master test functions, you will discover how you can group them in logical **units**, and associate them to R packages.

### Test units

An **unit** is a coherent piece of software that can be tested separately from the rest. Typically, a R package is a structured way to compile and distribute such code units in R. Hence, we need a mean to organize tests related to this "unit" conveniently.

Since a package can contain several functions, data frames, or other objects, our unit should collect together individual test functions related to each of these objects that compose our package. Also, the test unit should accommodate the well-define organization of a package, and should integrate in the already existing testing features of R, in particular, `R CMD check`. In both 'RUnit', and 'svUnit', one can define such test units, and they are made code compatible between the two implementations.

A test unit is a source-able text file that contains one or more test functions, plus possibly `.setUp()` and `.tearDown()` functions (see the online help for further information on these special functions). In 'RUnit', you must write such test unit files from scratch. With 'svUnit', you can "promote" one or several test functions (associated to other objects, or "living" alone as separate `svTest` objects) by using `makeUnit()`. Here is how you promote the test associated with our `Square()` function to a simple test unit containing only one test function:

```{r, eval=FALSE}
# Create a test unit on disk and view its content
unit <- makeUnit(Square)
file.show(unit, delete.file = TRUE)
```

You got the following file whose name must start with `runit`, with an `.R` extension (`runit*.R`), and located by default in the temporary directory of R. Specify another directory with the `dir =` argument of `makeUnit()` for a more permanent record of this test unit file. Note also that `.setUp()` and `.tearDown()` functions are constructed automatically for you. They specify the context of these tests. This context is used, for instance, by the GUI in Komodo Edit/IDE to locate the test function and the code being tested.

```
## Test unit 'Square'
.setUp <- function() {
  # Specific actions for svUnit: prepare context
  if ("package:svUnit" %in% search()) {
    .Log <- Log() # Make sure .Log is created
    .Log$..Unit <- "/tmp/RtmpBoZnId/runitSquare.R"
    .Log$..File <- ""
    .Log$..Obj <- ""
    .Log$..Tag <- ""
    .Log$..Msg <- ""
    rm(..Test, envir = .Log)
  }
}

.tearDown <- function() {
  # Specific actions for svUnit: clean up context
  if ("package:svUnit" %in% search()) {
    .Log$..Unit <- ""
    .Log$..File <- ""
    .Log$..Obj <- ""
    .Log$..Tag <- ""
    .Log$..Msg <- ""
    rm(..Test, envir = .Log)
  }
}

testSquare <- function() {
  checkEqualsNumeric(9, Square(3))
  checkEqualsNumeric(c(1, 4, 9), Square(1:3))
  checkException(Square("xx"))
}
```

Compatibility of these test unit files between 'RUnit' and 'svUnit' was a major concern in the design of 'svUnit'. Consequently, code specific to 'svUnit' (for managing the context of the tests) is embedded in a `if ("package:svUnit" %in% search())` construct. That way, if 'svUnit' is not loaded in memory, this code is not executed. *Note that you should avoid loading in memory both 'svUnit' and 'RUnit' at the same time! If you do so, you will most likely crash your tests.*

You will see further that it is possible to write much more complex test units with the same `makeUnit()` function. But for the moment, let’s discuss a little bit how such test units should be organized in R package.

If you intend to associate test units to your R package, you should respect the following conventions:

* Name your test units `runit*.R`.
* Place them in the `/inst/unitTests` subdirectory of the package sources, or in one of its subdirectories. If you place them in a subdirectory of `/inst/unitTests`, then you define secondary unit tests for (optional) detailed testing of specific item in your package. Always keep in mind that all `runit*.R` files in a directory will be run one after the other. So, if you want to make subgroups you would like to dissociate them, and locate them is separate subdirectories.
* When the package will be compiled, all these test units will be located in `/unitTests` in the compiled/installed version of your R package.

If you respect these conventions, 'svUnit' knows where package unit tests are located and will be able to find and run them quite easily. See, for instance, the examples in the 'svUnit' package.

So, with test units associated to packages, you have a very convenient way to run these tests, including from the Komodo GUI. With just a little bit more coding you can also include these test units in the `R CMD check` process of your packages. You do this by means of examples in a help page (we prefer to use **examples**, instead of `/tests` in the `R CMD check` process, because examples offer a more flexible way to run tests and you can also run them in interactive sessions through the `example()` R function, which is not the case for code located in the `/tests` subdirectory of your package). Here is what you do to associate some or all of your unit tests to the `R CMD check` process (illustrated with the case of the 'svUnit' package itself):

* Define a `.Rd` help file in the `/man` subdirectory called `unitTests.<mypackage>.Rd` where <mypackage> is the name of your package (or whatever name you prefer).
* Fill the `.Rd` file, making sure that you define an alias as `unitTests.<mypackage>`. Also place a little bit of information telling how users can run your test in an interactive session.
* The important part of this file is, of course, the `\examples{}` section. You must first clear the log, then run each test, and then, call the `errorLog()` function. That function looks if one or more tests failed or raised an error. In this case, it stops execution of the example and causes a dump of the test log in the `R CMD check` process. That way, providing that you have the 'svUnit' package installed in the machine where you run `R CMD check`, your test units will be included nicely in the checking process of your packages, that is, they will run silently each time you check your package if no error occurs, but will produce a detailed report in case of problems.
* Here is how your `.Rd` file should looks like (example of the 'svUnit' package):

```
\name{unitTests.svUnit}
\alias{unitTests.svUnit}
\title{ Unit tests for the package 'svUnit' }
\description{ Performs unit tests defined in this
  package by running \code{example(unitTests.svUnit)}.
  Tests are in \code{runit*.R} files Located in the
  ’/unitTests’ subdirectory or one of its
  subdirectories (’/inst/unitTests’ and subdirectories
  in package sources).
}
\author{Philippe Grosjean}
\keyword{utilities}
\examples{
if (require(svUnit)) {
  clearLog()
  runTest(svSuite("package:svUnit"), "svUnit")
  \donttest{
  # Tests to run with example() but not with R CMD check
  runTest(svSuite("package:svUnit (VirtualClass)"), "VirtualClass")
  }
  \dontrun{
  # Tests to present in ?unitTests.svUnit but not run automatically
  # Run all currently loaded packages test cases and test suites
  runTest(svSuiteList(), "AllTests")
  }
  \dontshow{
  # Put here test units you want to run during R CMD check but
  # don't want to show or run with example(unitTests.svUnit)
  }
  # Check errors at the end (needed to interrupt R CMD check)
  errorLog()
}
}
```

Alternatively, if you use 'roxygen2' to build your documentation, you can add a `unitTests.<pkgname>.R` file in the `/R` subdirectory containing ROxygen code to generate the adequate documentation. Here is the `unitTests.svUnit.R` file:

```r
#' Unit tests for the package 'svUnit'
#'
#' Performs unit tests defined in this package by running
#' `example(unitTests.svUnit)`. Tests are in `runit*.R` files located in the
#' '/unitTests' subdirectory or one of its subdirectories ('/inst/unitTests' and
#' subdirectories in package sources).
#'
#' @name unitTests.svUnit
#' @author Philippe Grosjean
#' @keywords utilities
#' @examples
#' if (require(svUnit)) {
#'   clearLog()
#'   runTest(svSuite("package:svUnit"), "svUnit")
#'   \donttest{
#'   # Tests to run with example() but not with R CMD check
#'   runTest(svSuite("package:svUnit (VirtualClass)"), "VirtualClass")
#'   }
#'   \dontrun{
#'   # Tests to present in ?unitTests.svUnit but not run automatically
#'   # Run all currently loaded packages test cases and test suites
#'   runTest(svSuiteList(), "AllTests")
#'   }
#'   \dontshow{
#'   # Put here test units you want to run during R CMD check but
#'   # don't want to show or run with example(unitTests.svUnit)
#'   }
#'   # Check errors at the end (needed to interrupt R CMD check)
#'   errorLog()
#' }
NULL
```

Note, however, that if the package 'svUnit' is not available on the computer where you run `R CMD check`, your tests are silently ignored (`require()` issues a warning, but that does not prevent the checking process to continue). This is an intended feature in order to allow compilation of your package without requiring 'svUnit'. Hence, dependence to 'svUnit' is less strict and also allows you to check your tests using 'RUnit' (but you have to write a dedicated function for that). Still to keep such a less strict dependence on 'svUnit', you should add `svUnit` in the `Suggests:` field in the `DESCRIPTION` file of your package, not in `Depends:` or `Imports:` fields (except if you use 'svUnit' for other purposes that testing your package using the mechanism discussed here, of course).

Also, this approach, associated with examples, provides a very convenient and easy way to test a package from the command line in an interactive session by running:

```{r}
example(unitTests.svUnit)
```

In the present case, the `errorLog()` instruction in the examples returns nothing, because all tests succeed. If there is an error somewhere, you will see it printed at the end of this example.

### Test suites: collections of test functions and units

The highest level of organization of your tests is the **test suite**. A test suite is an unordered collection of test functions and test units. You can select test units associated with R package in a very convenient way: just specify `package:myPkg` and all test units in the `/unitTests` subdirectory of the package myPkg will be included ('svUnit' does all the required work to map these to actual directories where the test unit files are located). Also, if you specify `package:myPkg(subgroup)`, you will include the test units defined in `/unitTests/subgroup` in the package myPkg. Of course, you will be able to also add test units defined in custom directories, outside of R packages (for instance for integration of *harness tests* that check cross-packages, or multi-packages features of your application).

Test functions associated to your test suite receive a special treatment. Unlike `runTest()` applied to a single test function, or to an object that has an associated test function, these tests are not run from the version loaded in memory. Instead, they are first collected together in a test unit file on disk (located in the R temporary directory, by default), and run from there. Hence, building a more complex test unit file by collecting together several test functions is just a question of constructing a test suite, and then, applying the `makeUnit()` function to this `svSuite` object.

Before we apply all this, you should also know the existence of one more function: `svSuiteList()`. This function lists all test units and test functions available in your system at a given time. So, you don’t need to manually create lists of components. You are better to list them automatically. Of course, this function has a lot of arguments for listing only test units in packages, only test functions, specifying where (in which environment) the test functions are located, adding custom directories where to look for test units, etc. See the online help of this function for the description of all these arguments. One argument is particularly important: `excludeList =`. This argument defines one or several regular expressions that are used as filters to hide items from the list. This is required, since you will certainly not want to run again and again, let’s say, the example tests associated with the 'svUnit' package ('svUnit' must be loaded in memory to run the tests, so its tests examples will always be listed by `svSuiteList()`, ... unless you define an adequate filter expression that will exclude them from your list)! As the default argument suggests it, the regular expression for list exclusion could also be recorded in `options(svUnit.excludeList = ...)`.
Here is how it works:

```{r}
# Reset default exclusion list
options(svUnit.excludeList = c("package:sv", "package:RUnit"))
# List all currently available tests
svSuiteList()
```

Thus, every entry matching the regular expressions `package:sv` and `package:RUnit` are excluded from the list. The entries `package:svUnit` and `package:svUnit (VirtualClass)` match first pattern and are thus excluded. Now, let's clear the exclusion list to see what happens:

```{r}
# Clear exclusion list
options(svUnit.excludeList = NULL)
svSuiteList()
```

The test units associated with the package 'svUnit' are now listed. You have noticed that `svSuiteList()` can also find automatically `svTest` objects, as well as tests attached to objects in the user's workspace. You can create a suite by collecting all these items together very easily:

```{r}
(mySuite <- svSuiteList())
```

Now let’s make a test unit using tests collected in this suite:

```{r, eval=FALSE}
myUnit <- makeUnit(mySuite, name = "ExampleTests")
file.show(myUnit, delete.file = TRUE)
```

This produces a file named `runitExampleTests.R` located (by default) in the R temporary directory, and which collects together all tests in the user's workspace (either as `svTest` objects, or as tests attached to other objects), plus tests suites in packages that are **not** in the exclusion list. Running all tests in your suite is very simple. You still use `runTest()` as usual,
but this time, you apply it to your `svSuite` object:

```{r}
clearLog()
runTest(mySuite)
summary(Log())
```

There are many other tools to manipulate `svSuite` objects in the 'svUnit' package, including functions to define the content of the suite manually (see online help).

### Using 'svUnit' with RStudio

Except by integrating your tests in a packages as explained above in this document, nothing is required to make your tests run in RStudio. The `Build -> Test Package` menu entry should run your tests with 'svUnit' once this integration is done.

### Using 'svUnit' with SciViews Komodo

If you use the SciViews Komodo GUI, you can integrate 'svUnit' tests in this IDE and display reports in a convenient hierarchical tree presentation.

If R is started from within Komodo Edit or IDE (with the SciViews-K and SciViews-K Unit plugins installed), then, loading the 'svUnit' package in R automatically installs the **R Unit** side panel in Komodo at right. Its use should be straightforward:

![Komodo Edit with SciViews-K and SciViews-K Unit after running tests. At right: the **R Unit** panel that display (at the top) the list of available tests, and test units where you can select the ones to run, and at the bottom, a tree with the results from last tests run. The stripe at the very top is green if all tests succeed, and red (as here), if at least one tests failed or raised an error.](svUnit_KomodoRUnit.png)

* Select the tests you want to run in the top part,
* Click the `Run` button each time you want to refresh the test
tree,
* Browse the tree for failures or errors (the color at the top of the panel immediately indicates if there is a problem somewhere: green -> everything is fine, red -> there is a problem).
* If you have failures or errors, move the mouse on top of the corresponding item in the tree, and you got more information displayed in a tooltip,
* Click on an item to open the test unit at that place in a buffer in Komodo.

The `Auto` mode, when activated, sources R files currently edited in Komodo whenever you save them, and then, refreshes the test report tree. This mode allows you to run automatically your tests in the background while you edit your R code!

> If you want to implement such a side panel in another GUI, make sure to look at the `koUnit\_XXX()` functions in the 'svUnit' package. These functions allow to control the GUI in Komodo remotely from within R, and similar functions should not be too difficult to implement for other GUIs.

## References

* Grosjean, Ph., 2003. SciViews: an object-oriented abstraction layer to design GUIs on top of various calculation kernels [online: http://www.ci.tuwien.ac.at/Conferences/DSC-2003/]

* IEEE Standards Boards, 1993. IEEE standard for software unit testing. ANSI/IEEE Std 1008-1987. 24 pp.

* Ihaka R. & R. Gentleman, 1996. R: a language for data analysis and graphics. *J. Comput. Graphic. Stat.*, 5:299-314.

* Jeffries, R., 2006. Extreme programming, web site at: https://ronjeffries.com/xprog/what-is-extreme-programming/.

* König, T., K. Jünemann & M. Burger, 2007. RUnit – A unit test framework for R. Vignette of the package RUnit available on CRAN. 11 pp.

* R Core Team, 2019. R: A language and environment for statistical computing. R Foundation for Statistical Computing, Vienna, Austria. [online: https://www.R-project.org]

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/check.R
\name{check}
\alias{check}
\alias{checkEquals}
\alias{checkEqualsNumeric}
\alias{checkIdentical}
\alias{checkTrue}
\alias{checkException}
\alias{DEACTIVATED}
\title{SciViews-R Unit assertions (check functions)}
\usage{
checkEquals(
  target,
  current,
  msg = "",
  tolerance = .Machine$double.eps^0.5,
  checkNames = TRUE,
  ...
)

checkEqualsNumeric(
  target,
  current,
  msg = "",
  tolerance = .Machine$double.eps^0.5,
  ...
)

checkIdentical(target, current, msg = "")

checkTrue(expr, msg = "")

checkException(expr, msg = "", silent = getOption("svUnit.silentException"))

DEACTIVATED(msg = "")
}
\arguments{
\item{target}{a target object as reference for comparison.}

\item{current}{An object created for comparison (not an S4 class object).}

\item{msg}{An optional (short!) message to document a test. This message is
stored in the log and printed in front of each test report.}

\item{tolerance}{numeric >= 0. A numeric check does not fail if differences
are smaller than 'tolerance'.}

\item{checkNames}{Flag, if \code{FALSE} the names attributes are set to \code{NULL} for
both \code{current} and \code{target} before performing the check.}

\item{...}{Optional arguments passed to \code{\link[=all.equal]{all.equal()}} or
\code{\link[=all.equal.numeric]{all.equal.numeric()}}.}

\item{expr}{Syntactically valid R expression which can be evaluated and must
return a logical vector (\code{TRUE}|\code{FALSE}). A named expression is also allowed
but the name is disregarded. In \code{\link[=checkException]{checkException()}}, expr is supposed to
generate an error to pass the test.}

\item{silent}{Flag passed on to try, which determines if the error message
generated by the checked function is displayed at the R console. By default,
it is \code{FALSE}.}
}
\value{
These function return \code{TRUE} if the test succeeds, \code{FALSE} if it fails,
possibly with a 'result' attribute containing more information about the
problem. This is very different from corresponding functions in 'RUnit' that
stop with an error in case of test failure. Consequently, current functions
do not require the complex evaluation framework designed in 'RUnit' for that
reason.
}
\description{
These functions define the assertions in test functions. They are designed to
check the result of some test calculation.
}
\details{
These check functions are equivalent to various methods of the class
junit.framework.Assert of Java junit framework. They should be
code-compatible with functions of same name in 'RUnit' 0.4.17, except
for \code{\link[=checkTrue]{checkTrue()}} that is vectorized here, but accept only a scalar result in
'RUnit'. For scalar test, the behavior of the function is the same in both
packages.
See \code{\link[=svTest]{svTest()}} for examples of use of these functions in
actual test cases attached to R objects.
See also the note about S4 objects in the \code{\link[RUnit:checkTrue]{RUnit::checkTrue()}} online help of the
'RUnit' package.
}
\examples{
clearLog()     # Clear the svUnit log

# All these tests are correct
(checkEquals(c("A", "B", "C"), LETTERS[1:3]))
(checkEqualsNumeric(1:10, seq(1, 10)))
(checkIdentical(iris[1:50, ], iris[iris$Species == "setosa",]))
(checkTrue(1 < 2))
(checkException(log("a")))
Log()    # See what's recorded in the log

# ... but these ones fail
(checkEquals("A", LETTERS[1:3]))
(checkEqualsNumeric(2:11, seq(1, 10)))
(checkIdentical(iris[1:49, ], iris[iris$Species == "setosa",]))
(checkTrue(1 > 2))
(checkException(log(1)))
Log()    # See what's recorded in the log

# Create a test function and run it
foo <- function(x, y = 2)
  return(x * y)
test(foo) <- function() {
  #DEACTIVATED()
  checkEqualsNumeric(5, foo(2))
  checkEqualsNumeric(6, foo(2, 3))
  checkTrue(is.test(foo))
  checkTrue(is.test(test(foo)))
  checkIdentical(test(foo), attr(foo, "test"))
  checkException(foo("b"))
  checkException(foo(2, "a"))
}
(runTest(foo))

# Of course, everything is recorded in the log
Log()

clearLog()
}
\seealso{
\code{\link[=svTest]{svTest()}}, \code{\link[=Log]{Log()}}, \code{\link[=guiTestReport]{guiTestReport()}}, \link[RUnit:checkTrue]{RUnit::checkTrue}
}
\author{
Written by Ph. Grosjean, inspired from the general design of the
'RUnit' package by Thomas Konig, Klaus Junemann & Matthias Burger.
}
\concept{unit testing}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/unitTests.svUnit.R
\name{unitTests.svUnit}
\alias{unitTests.svUnit}
\title{Unit tests for the package 'svUnit'}
\description{
Performs unit tests defined in this package by running
\code{example(unitTests.svUnit)}. Tests are in \code{runit*.R} files located in the
'/unitTests' subdirectory or one of its subdirectories ('/inst/unitTests' and
subdirectories in package sources).
}
\examples{
if (require(svUnit)) {
  clearLog()
  runTest(svSuite("package:svUnit"), "svUnit")
  \donttest{
  # Tests to run with example() but not with R CMD check
  runTest(svSuite("package:svUnit (VirtualClass)"), "VirtualClass")
  }
  \dontrun{
  # Tests to present in ?unitTests.svUnit but not run automatically
  # Run all currently loaded packages test cases and test suites
  runTest(svSuiteList(), "AllTests")
  }
  \dontshow{
  # Put here test units you want to run during R CMD check but
  # don't want to show or run with example(unitTests.svUnit)
  }
  # Check errors at the end (needed to interrupt R CMD check)
  errorLog()
}
}
\author{
Philippe Grosjean
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/koUnit.R
\name{koUnit}
\alias{koUnit}
\alias{koUnit_setAutoTest}
\alias{koUnit_isAutoTest}
\alias{koUnit_runTest}
\alias{koUnit_showRUnitPane}
\alias{koUnit_version}
\title{Interact with the test unit GUI in Komodo/SciViews-K}
\usage{
koUnit_setAutoTest(state)

koUnit_isAutoTest()

koUnit_runTest()

koUnit_showRUnitPane(state)

koUnit_version()
}
\arguments{
\item{state}{\code{TRUE} or \code{FALSE}, or missing for \code{koUnit_showRUnitPane()}, in
this case, the R Unit pane visibility is toggled.}
}
\value{
\code{\link[=koUnit_isAutoTest]{koUnit_isAutoTest()}} returns \code{TRUE} if the test unit is in auto mode in
Komodo (the selected tests are run automatically each time a .R file
edited in Komodo is saved).

\code{\link[=koUnit_version]{koUnit_version()}} returns the version for which the SciViews-K Unit
extension was designed for. This allow to check if this version is compatible
with current 'svUnit' R package version, and to propose to update the Komodo
extension if needed (this mechanism is not running currently, but it will be
implemented in the future to avoid or limit incompatibilities between
respective R and Komodo extensions).

The other functions are invoked for their side-effect and they return
nothing. Note, however, that correct execution of this code in Komodo is
verified, and the functions issue an error in R if they fail to execute
correctly in Komodo.
}
\description{
These functions allow controlling the test unit module (R Unit tab at right)
in Komodo with SciViews-K and SciViews-K Unit extensions. R must be correctly
connected to Komodo, meaning that the 'svGUI' package must be loaded with
proper configuration of client/server socket connections between R and
Komodo. See the manual about SciViews-K for more information. The functions
defined here are the same as JavaScript functions defined in the 'sv.r.unit'
namespace in Komodo/SciViews-K Unit. For instance, \code{\link[=koUnit_runTest]{koUnit_runTest()}} is
equivalent to \code{sv.r.unit.runTest();} in a Javascript macro in Komodo.
}
\examples{
\dontrun{
# Make sure R is communicating with Komodo before use, see ?koCmd in svGUI
koUnit_version()

# Toggle visibility of the R Unit pane in Komodo twice
koUnit_showRUnitPane()
koUnit_showRUnitPane()

# Make sure that the R Unit pane is visible
koUnit_showRUnitPane(TRUE)

# Is the test unit running in auto mode?
koUnit_isAutoTest()

# Toggle auto test mode off
koUnit_setAutoTest(FALSE)

# Run the test units from within Komodo
koUnit_runTest()
}
}
\seealso{
\code{\link[=guiTestReport]{guiTestReport()}}
}
\author{
Philippe Grosjean
}
\concept{unit testing}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Log.R
\name{Log}
\alias{Log}
\alias{createLog}
\alias{clearLog}
\alias{errorLog}
\alias{lastTest}
\alias{lastSuite}
\title{SciViews-R log management functions}
\usage{
Log(description = NULL)

createLog(description = NULL, deleteExisting = FALSE)

clearLog()

errorLog(stopit = TRUE, summarize = TRUE)

lastTest()

lastSuite()
}
\arguments{
\item{description}{A (short) character string describing this test suite log.}

\item{deleteExisting}{Do we delete an existing a \code{.Log} object already
defined in \code{.GlobalEnv} (no, by default)?}

\item{stopit}{Do we issue an error (\code{\link[=stop]{stop()}} in case of any error or failure?
This is particularly useful if you want to interrupt R CMD check on packages,
when you included one or more test suites in examples (see \code{?unitTests}).}

\item{summarize}{Should the summary of the log be printed in case we stop
execution of the code when an error is found (see \verb{stopit =} argument. It is,
indeed, useful to indicate at this time which tests failed or raised an
error. So, this argument should usually be left at its default value.}
}
\value{
\code{\link[=Log]{Log()}} and \code{\link[=createLog]{createLog()}} return the \code{.Log} object defined in \code{.GlobalEnv} by
reference (it is indeed an environment). So, you can use its content (and
change it, if you write functions to manipulate this log).

\code{\link[=clearLog]{clearLog()}} return invisibly \code{TRUE} or \code{FALSE}, depending if an existing log
object was deleted or not.

\code{\link[=errorLog]{errorLog()}} is mainly used for its side-effect of stopping code execution
and/or printing a summary of the test runs in the context of example
massaging in R CMD check (see the "Writing R extensions" manual). However,
this function also returns invisibly a contingency table with the number of
successes, failures, errors and deactivated tests recorded so far.

\code{\link[=lastTest]{lastTest()}} and \code{\link[=lastSuite]{lastSuite()}} recall results of last test and last suite
run, respectively.
}
\description{
These functions define the code of test functions. They are designed to check
the result of some test calculation.
}
\details{
svUnit records results of assertions (using the checkxxx() functions) in a
'svSuiteData' object named \code{.Log} and located in .GlobalEnv. Hence, this log
is easy to access. However, in order to avoid errors in your code in case
this object was deleted, or not created, it is better to access it using
\code{\link[=Log]{Log()}} which take care to create the object if it is missing.
}
\examples{
clearLog()           # Clear the svUnit log

# Two correct tests
(checkTrue(1 < 2))
(checkException(log("a")))
errorLog()           # Nothing, because there is no error

\dontrun{
(checkTrue(1 > 2))    # This test fails
lastTest()            # Print results of last test
errorLog()            # Stop and summarize the tests run so far
}

clearLog()
}
\seealso{
\code{\link[=svSuiteData]{svSuiteData()}}, \code{\link[=svSuite]{svSuite()}}, \code{\link[=svTest]{svTest()}}, \code{\link[=checkEquals]{checkEquals()}}
}
\author{
Philippe Grosjean
}
\concept{unit testing}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/svSuiteData.R
\name{svSuiteData}
\alias{svSuiteData}
\alias{is.svSuiteData}
\alias{stats.svSuiteData}
\alias{metadata}
\alias{metadata.svSuiteData}
\alias{print.svSuiteData}
\alias{summary.svSuiteData}
\alias{protocol}
\alias{protocol.default}
\alias{protocol.svSuiteData}
\alias{protocol_text}
\alias{protocol_text.svSuiteData}
\alias{protocol_junit}
\alias{protocol_junit.svSuiteData}
\title{Objects of class 'svSuiteData' contain results from running test suites}
\usage{
is.svSuiteData(x)

\method{stats}{svSuiteData}(object, ...)

metadata(object, ...)

\method{metadata}{svSuiteData}(
  object,
  fields = c("R.version", "sessionInfo", "time", "description"),
  ...
)

\method{print}{svSuiteData}(x, all = FALSE, file = "", append = FALSE, ...)

\method{summary}{svSuiteData}(object, ...)

protocol(object, type = "text", file = "", append = FALSE, ...)

\method{protocol}{default}(object, type = "text", file = "", append = FALSE, ...)

\method{protocol}{svSuiteData}(object, type = "text", file = "", append = FALSE, ...)

protocol_text(object, file = "", append = FALSE, ...)

\method{protocol_text}{svSuiteData}(object, file = "", append = FALSE, ...)

protocol_junit(object, ...)

\method{protocol_junit}{svSuiteData}(object, file = "", append = FALSE, ...)
}
\arguments{
\item{x}{Any kind of object, or a 'svSuiteData' object in the case of
\code{\link[=print]{print()}}.}

\item{object}{A 'svSuiteData' object.}

\item{...}{Further arguments to pass to methods. Not used yet.}

\item{fields}{Character vector. The name of all metadata items you want to
extract for the object. The default value is an exhaustive list of all
available metadata (i.e., defined by default) in the object, but you can add
more: just add a corresponding attribute to your object.}

\item{all}{Do we print concise report for all test, or only for the tests
that fail or produce an error?}

\item{file}{Character. The path to the file where to write the report.
If \code{file = ""}, the protocol report is output to the console.}

\item{append}{Do we append to this file?}

\item{type}{Character. The type of protocol report to create. For the moment,
only \code{type = "text"} and \code{type = "junit"} are supported, but further types
(HTML, LaTeX, Wiki, etc.) could be provided later.}
}
\value{
\code{\link[=is.svSuiteData]{is.svSuiteData()}} returns \code{TRUE} if the object is an 'svSuiteData'. The
various methods serve to extract or print content in the object.
}
\description{
The 'svSuiteData' object contains results of all test run in one or more test
suites. The \code{checkxxx()} functions and the \code{\link[=runTest]{runTest()}} method generate data
(objects 'svTestData') contained in the default 'svSuiteData' named \code{.Log}
and located in \code{.GlobalEnv}. It is then possible to display and report
information it contains in various ways to analyze the results.
}
\details{
A 'svSuiteData' is, indeed, an environment. The results for the various tests
runs are in non hidden (i.e., names not starting with a dot) objects that are
of class 'svTestData' in this environment. Various other objects that control
the execution of the test, their context, etc. are contained as hidden objects
with name starting with a dot. Note that using an environment instead of a
list for this object allows for a call by reference instead of a usual call by
value in R, when passing this object to a function. This property is largely
exploited in all svUnit functions to make sure results of test runs are
centralized in the same log ('svSuiteData' object).
}
\examples{
clearLog()    # Clear any existing log

# Run some tests
checkTrue(1 < 2)
checkException(log("a"))
foo <- function(x, y = 2)
  return(x * y)
test(foo) <- function() {
  checkEqualsNumeric(4, foo(2))
  checkEqualsNumeric(6, foo(2, nonexisting))
  checkTrue(is.test(foo))
  warning("This is a warning")
  cat("Youhou from test!\n")  # Don't use, except for debugging!
  checkTrue(is.test(test(foo)))
  checkIdentical(attr(foo, "test"), test(foo))
  checkException(foo(2, nonexisting))
  #DEACTIVATED("My deactivation message")
  checkException(foo(2))  # This test fails
}
runTest(foo)

# Now inspect the log, which is a 'svSuiteData' object
is.svSuiteData(Log())
stats(Log())
metadata(Log())
Log()    # Print method
summary(Log())

\dontrun{
# To get a print of the test protocol on file, use:
protocol(Log(), type = "text", file = "RprofProtocol.out")
file.show("RprofProtocol.out")
unlink("RprofProtocol.out")
}

rm(foo)

\dontrun{
# Profiling of very simple test runs
library(utils)
createLog(description = "test profiling", deleteExisting = TRUE)
imax <- 3
jmax <- 100
l <- 50
Rprof()
for (i in 1:imax) {
  # Change the context for these tests
  .Log$..Test <- paste("Test", i, sep = "")
  .Log$..Tag <- paste("#", i, sep = "")
  res <- system.time({
    for (j in 1:jmax) checkTrue(i <= j, "My test")
  }, gcFirst = TRUE)[3]
  print(res)
  flush.console()
}
Rprof(NULL)
# Look at profile
summaryRprof()
unlink("Rprof.out")

# Look at the log
summary(Log())
}
}
\seealso{
\code{\link[=svSuite]{svSuite()}}, \code{\link[=is.svTestData]{is.svTestData()}}, \code{\link[=Log]{Log()}}, \code{\link[=checkEquals]{checkEquals()}}
}
\author{
Philippe Grosjean; Mario Frasca for the junit protocol.
}
\concept{unit testing}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/svSuite.R
\name{svSuite}
\alias{svSuite}
\alias{as.svSuite}
\alias{is.svSuite}
\alias{print.svSuite}
\alias{svSuiteList}
\alias{makeUnit.svSuite}
\alias{runTest.svSuite}
\title{Create and run test suites by collecting together unit tests and function
tests defined in objects}
\usage{
svSuite(tests)

as.svSuite(x)

is.svSuite(x)

\method{print}{svSuite}(x, ...)

svSuiteList(
  packages = TRUE,
  objects = TRUE,
  dirs = getOption("svUnit.dirs"),
  excludeList = getOption("svUnit.excludeList"),
  pos = .GlobalEnv,
  loadPackages = FALSE
)

\method{makeUnit}{svSuite}(
  x,
  name = make.names(deparse(substitute(x))),
  dir = tempdir(),
  objfile = "",
  codeSetUp = NULL,
  codeTearDown = NULL,
  pos = .GlobalEnv,
  ...
)

\method{runTest}{svSuite}(x, name = make.names(deparse(substitute(x))), unitname = NULL, ...)
}
\arguments{
\item{tests}{A character string with items to include in the test suite. It
could be 'package:PKG' for including test units located in the /unitTests
subdirectory of the package PGK, or 'package:PKG (SUITE)' for test units
located in the subdirectory /unitTests/SUITE of package PKG, or 'dir:MYDIR'
for including test units in MYDIR, or 'test(OBJ)' for tests embedded in an
object, or 'OBJ' for 'svTest' object directly.}

\item{x}{Any kind of object.}

\item{...}{Further arguments to pass to \code{\link[=makeUnit]{makeUnit()}} or \code{\link[=runTest]{runTest()}} (not used
yet).}

\item{packages}{Do we list test units available in loaded packages?
Alternatively one can provide a character vector of package names, and it
will be used to filter packages (take care: in this case it will look at
installed packages, not only loaded packages)!}

\item{objects}{Do we list test available in objects? Alternatively, one can
provide a character vector of object names, and it will filter objects in
'pos' according to this vector.}

\item{dirs}{An additional list of directories where to look for more test
units. For convenience, this list can simply be saved as an 'svUnit.dirs'
options.}

\item{excludeList}{A list of items to exclude from the listing. The function
uses regular expression to match the exclusions. So, for instance, specifying
\code{"package:MYPKG"} will exclude all items from package 'MYPKG', while using
\code{"package:MYPKG$"} will exclude only tests suites defined in the
.../MYPKG/unitTests directory, bur not in its subdirectories. For
convenience, it can be saved in a 'svUnit.excludeList' option. By default,
all tests for packages whose name start with 'sv' or 'RUnit' are excluded,
that is, \code{c("package:sv", "package:RUnit")}.}

\item{pos}{The environment to look for 'objects' (environment, character
string with name of an environment, or integer with position of the
environment in the search path.}

\item{loadPackages}{In the case a list of packages is provided in
\verb{packages =}, do we make sure that these packages are loaded? If yes, the
function will try to load all packages in that list that are not loaded yet
and will issue a warning for the packages not found. Default, \code{FALSE}.}

\item{name}{The name of the test suite to build.}

\item{dir}{The directory where to create the test unit file.}

\item{objfile}{The path to the file containing the original source code of
the object being tested. This argument is used to bring a context for a
test and allow a GUI to automatically open the source file for edition when
the user clicks on a test that failed or raised an error.}

\item{codeSetUp}{An expression with some code you want to add to the
\code{.setUp()} function in your unit file (this function is executed before each
test.}

\item{codeTearDown}{An expression with some code you want to add to the
\code{.tearDown()} function in your unit file (this function is executed after
each test.}

\item{unitname}{The name of a unit to run inside the suite. If \code{NULL} (by
default), all units are run.}
}
\value{
\code{\link[=svSuite]{svSuite()}}, \code{\link[=as.svSuite]{as.svSuite()}} and \code{\link[=svSuiteList]{svSuiteList()}} return a 'svSuite' object.
\code{\link[=is.svSuite]{is.svSuite()}} returns \code{TRUE} if the object is an 'svSuite'.

\code{\link[=makeUnit]{makeUnit()}} creates a test unit file on disk, and \code{\link[=runTest]{runTest()}} runs the tests
in such a file. They are used for their side-effect, but the first one also
returns the file created, and the second one returns invisibly the list of
all test unit files that where sourced ans run.
}
\description{
A 'svSuite' object is essentially a list of test units directories (or
packages, in this case, corresponding directories are PKG/unitTests and its
subdirectories), and of object names containing tests to add temporarily to
the test suite. These must be formatted in a concise way as described for the
'tests' argument.
}
\details{
\code{\link[=svSuiteList]{svSuiteList()}} lists all loaded packages having /unitTests/runit*.R files
(or similar files in subdirectories), and all objects in the user workspace
that have a 'test' attribute, or are 'svTest' objects (by default). It is a
rather exhaustive list of all test items currently available in the current R
session, but restricted by \code{getOption("svUnit.excludeList")}.

\code{\link[=makeUnit]{makeUnit()}} writes a test unit on disk with the tests from the objects
listed in the 'svSuite' object that do not belong yet to a test unit.
\code{\link[=runTest]{runTest()}} runs all the test in packages, directories and objects listed in
the 'svSuite' object.

Thanks to the variety of sources allowed for tests, it is possible to define
these tests in a structured way, inside packages, like for the 'RUnit'
package (but with automatic recognition of test units associated to packages,
in the present case). It is also easy to define tests more loosely by just
attaching those tests to the objects you want to check. Whenever there objects
are loaded in the user's workspace, their tests are available. In both cases,
a test unit file on disk is sourced in a local environment and test functions
are run (same approach as in the 'RUnit' package, and the same test unit files
should be compatibles with both 'RUnit' and 'svUnit' packages), but in the
case of a loosely definition of the tests by attachment to objects, the test
unit file is created on the fly in the temporary directory (by default).

At any time, you can transform a series of tests loosely attached to objects
into a test unit file by applying \code{\link[=makeUnit]{makeUnit()}} to a 'svSuite' object,
probably specifying another directory than the (default) temporary dir for
more permanent storage of your test unit file. The best choice is the
'/inst/unitTests' directory of a package source, or one of its
subdirectories. That way, your test unit file(s) will be automatically listed
and available each time you load the compiled package in R (if you list them
using \code{\link[=svSuiteList]{svSuiteList()}}). Of course, you still can exclude tests from given
packages by adding 'package:PKG' in the exclusion list with something
like: \code{options(svUnit.excludeList = c(getOption("svUnit.excludeList"), "package:PKG"))}.
}
\examples{
svSuiteList() # List all currently available test units and test cases
# Exclusion list is used (regular expression filtering!). It contains:
(oex <- getOption("svUnit.excludeList"))
# Clear it, and relist available test units
options(svUnit.excludeList = NULL)
svSuiteList()

# Two functions that include their test cases
Square <- function(x)
  return(x^2)
test(Square) <- function() {
  checkEquals(9, Square(3))
  checkEquals(c(1, 4, 9), Square(1:3))
  checkException(Square("xx"))
}

Cube <- function(x)
  return(x^3)
test(Cube) <- function() {
  checkEquals(27, Cube(3))
  checkEquals(c(1, 8, 28), Cube(1:3))
  checkException(Cube("xx"))
}

# A separate test case object (not attached to a particular object)
# This is the simplest way to define quick and durty integration tests
test_Integrate <- svTest(function() {
  checkTrue(1 < 2, "check1")
  v <- 1:3 	# The reference
  w <- 1:3 	# The value to compare to the reference
  checkEquals(v, w)
})

# A function without test cases (will be filtered out of the suite list)
foo <- function(x)
  return(x)

# Look now which tests are available
svSuiteList()

# Only objects, no package units
svSuiteList(packages = FALSE)

\dontrun{
# Create the test unit file for all objects with tests in .GlobalEnv
myunit <- makeUnit(svSuiteList(), name = "AllTests")
file.show(myunit, delete.file = TRUE)
}

# Filter objects using a list (object with/without tests and a nonexisting obj)
svSuiteList(packages = FALSE, objects = c("Cube", "foo", "bar"))

# Create another svSuite object with selected test items
(mysuite <- svSuite(c("package:svUnit (VirtualClass)", "test(Cube)")))
is.svSuite(mysuite)	# Should be!

\dontrun{
# Run all the tests currently available
(runTest(svSuiteList(), name = "AllTests"))
summary(Log())
}

# Restore previous exclusion list, and clean up the environment
options(svUnit.excludeList = oex)
rm(Square, Cube, foo, test_Integrate, mysuite, myunit, oex)
}
\seealso{
\code{\link[=svSuiteData]{svSuiteData()}}, \code{\link[=svTest]{svTest()}}, \code{\link[=Log]{Log()}}, \code{\link[=checkEquals]{checkEquals()}}, \code{\link[RUnit:checkEquals]{RUnit::checkEquals()}}
}
\author{
Philippe Grosjean
}
\concept{unit testing}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/svUnit-package.R
\docType{package}
\name{svUnit-package}
\alias{svUnit}
\alias{svUnit-package}
\title{svUnit: 'SciViews' - Unit, Integration and System Testing}
\description{
A complete unit test system and functions to implement its GUI part.
}
\details{
The SciViews 'svUnit' package defines a framework for testing R code, not
unlike jUnit for Java. It is inspired on the \code{checkxxx()} functions from the
'RUnit' package and the same test unit files should be compatible with both
'svUnit' and 'RUnit'. However, the internal implementation is completely
different and svUnit can also be used interactively, while 'RUnit' is only
designed to run test units written in files on disks.

The test unit framework provided in 'svUnit' is based on tests, also called
assertions, implemented in \code{checkxxx()} functions. For instance, the
\code{checkTrue(expr)} function check if its 'expr' argument returns \code{TRUE}.
Results of these assertions are collected in a centralized logger located in
the \code{.Log} object in \code{.GlobalEnv}. This is a 'svSuiteData' object with data
about the context of the tests (see for instance, \code{\link[=lastTest]{lastTest()}}, \code{\link[=lastSuite]{lastSuite()}}
or \code{metadata(.Log)}).

Assertions can give three results: (1) \code{TRUE} if success, (2) \code{FALSE} in
case of failure (in our example, 'expr' in \code{checkTrue(expr)} did not return
\code{TRUE}), and (3) \code{NA} if the code in 'expr' cannot be parsed or executed
correctly. All these errors or failures are catch and recorded in the logger,
as individual 'svTestData' objects.

Both the logger ('svSuiteData' object) and test records inside it
('svTestData' objects) have convenient methods to visualize information they
contain: \code{\link[=print]{print()}}, \code{\link[=summary]{summary()}} and \code{\link[=stats]{stats()}} methods. Access to the
individual test records in the logger is done with list-like instructions:
\code{.Log$mytest} returns the 'svTestData' object named 'mytest', itself the
result of running test in the 'mytest' test function (i.e.,
\code{runTest(mytest)}, see hereunder). Assertions run at the command line,
outside of specific contexts provided by test functions, test units and
test suites (see hereunder) are recorded under the 'eval' 'svTestData' object
in the logger (i.e., \code{.Log$eval}).

Since a 'svSuiteData' object (the logger) is also an environment, you can get
the list of all test records it contains using \code{ls(.Log)}, and you can
eliminate a given test record using \code{rm(mytest, envir = .Log)}.

Test cases are collections of assertions with the satellite code needed to
build example or situations to be tested. They are collected together in
argument-less functions with class being 'svTest'. See \code{?svTest}  for
further explanations and a couple of example test cases/test functions.

In its simplest instance, a test function is defined as a separate R object
loaded in memory (unlike RUnit where all test must be defined in files). You
run it simply by using \code{runTest(mytest)}. A slightly more structured way
to work is to attach the test function to the object it testes. You use
\code{test(myobj) <- testmyobj} to do so, and retrieve it with \code{test(myobj)}. Now,
the test function always follows the tested object. Testing the object is
still simple by using \code{runTest(myobj)}, which is totally equivalent to
\code{runTest(test(myobj))}. One can determine if an object has a test function
associated, or is a test function itself by using \code{is.test(myobj)}.

Several test functions can be collected together in so-called test units. A
test unit only exists on disk. It is a file named 'runit*.R' containing
sourceable R code with test functions having names starting with 'test'
(unlike 'RUnit', the default convention of file names starting with 'runit'
and test function names starting with 'test' is not customizable in
'svUnit'). One can also define special \code{.setUp()} and \code{.tearDown()} functions
in the test unit. The first function will be run before each test function,
and the latter one will be run after it. Test units are created manually, or
from a collection of objects with associated test functions loaded in an
environment (usually \code{.GlobalEnv}) thanks to the \code{\link[=makeUnit]{makeUnit()}} method. These
units should be mutually compatible with those used in the 'RUnit' package
(at least this is verified with version 0.4-17 of 'RUnit').

Test units defined for packages should be located in the package /runitTests
subdirectory (/inst/runitTests for source of the package) or one of its
subdirectories. That way, they are located automatically by the function
\code{\link[=svSuiteList]{svSuiteList()}} that also automatically detects all objects with associated
test functions loaded in \code{.GlobalEnv}. Test suites are 'svSuite'
objects with a list of test units or test objects to collect in the suite.
Thus, \code{\link[=svSuiteList]{svSuiteList()}} automatically builds such a suite with all tests it
finds in R, with many possibilities to filter packages' test units, objects'
test functions, or to add non standard directories with test units, for
instance. See \code{?svSuite} for more details on creating and using these
suites.

A GUI (Graphical User Interface) is provided to automatically build and run
tests suites and to get a graphical (tree) interactive report of the results
in the Komodo Edit or IDE code editor, together with the SciViews-K
extension. If you want to use this (optional) GUI, you have to install
required software components on your machine.

Finally, the 'svUnit' framework is compatible with \verb{R CMD check} (see the
manual "Writing R extensions"). You simply define man pages (.Rd files) with
an example section running selected test units from your package. The
function \code{\link[=errorLog]{errorLog()}} is designed to generate and error if one or more tests
failed or raised an error during \verb{R CMD check}, and it should be used at the
end of the example that runs your unit test(s). That way, \verb{R CMD check} is
interrupted and a detailed report of the tests that failed or raised an error
is printed. See an example in \code{?unitTests.svUnit}.
}
\examples{
# Clear the logger
clearLog()

# Design and attach a simple test function to an object
foo <- function(x, y = 2)
  return(x * y)
testfoo <- function() {
  #DEACTIVATED()	# Use this to deactive the test (notice placed in the log)
  checkEqualsNumeric(5, foo(2),  "Check return of foo()")
  checkException(foo("b"),       "Wrong first argument")
  checkException(foo(2, "a"),    "Wrong second argument")
}
# Attach this to the foo function
test(foo) <- testfoo

# Run this test
runTest(foo)

# Inspect the result
ls(.Log)
.Log$`test(foo)`
# This test fails. You see that the test function requires that foo(2) = 5
# and the actual implementation returns 4. This is a trivial, useless example,
# but you are supposed to correct the function. For instance:
foo <- function(x, y = 2)
  return(x * y + 1)
test(foo) <- testfoo

(runTest(foo))	# Now, that's fine!
}
\references{
There is a huge literature and unit testing. An easy starting
point is: \url{https://en.wikipedia.org/wiki/Unit_test}.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/SciViews/svUnit}
  \item \url{https://www.sciviews.org/svUnit/}
  \item Report bugs at \url{https://github.com/SciViews/svUnit/issues}
}

}
\author{
Written by Ph. Grosjean, inspired from the general design of the
'RUnit' package by Thomas Konig, Klaus Junemann & Matthias Burger.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/guiTestReport.R
\name{guiTestReport}
\alias{guiTestReport}
\alias{guiSuiteList}
\alias{guiSuiteAutoList}
\alias{guiTestFeedback}
\title{Report or give feedback to the GUI client about running test units}
\usage{
guiTestReport(object, sep = "\\t", path = NULL, ...)

guiSuiteList(sep = "\\t", path = NULL, compare = TRUE)

guiSuiteAutoList(...)

guiTestFeedback(object, path = NULL, ...)
}
\arguments{
\item{object}{a 'svUnitData' object.}

\item{sep}{Field separator to use in the results.}

\item{path}{Path where to write a 'Suites.txt' file with the list of
currently available test suites (to be used by the GUI client). If \code{NULL},
no file is written (by default).}

\item{...}{Not used currently.}

\item{compare}{Do we compare the list of available test suite and return
something to the GUI client only if there are changes in the list? This is
used (when \code{TRUE}) to avoid unnecessary multiple processing of the same list
by the GUI client.}
}
\value{
\code{\link[=guiSuiteList]{guiSuiteList()}} returns the list of available test suites invisibly.
\code{\link[=guiSuiteAutoList]{guiSuiteAutoList()}} is used to establish a callback to automatically list
the available test suites in the GUI. It is not intended to be called
directly by the user. The other functions just return \code{TRUE} invisibly.They
are used for their side effect of sending data to compatible GUI clients.
}
\description{
These functions are usually not called from the command line. They return
data to compatible GUI clients, like Komodo Edit with the SciViews-K
extension.
}
\seealso{
\code{\link[=svTest]{svTest()}}, \code{\link[=svSuite]{svSuite()}}, \code{\link[=koUnit_version]{koUnit_version()}}
}
\author{
Philippe Grosjean
}
\concept{unit testing}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/runExamples.R, R/svTest.R
\name{makeTestListFromExamples}
\alias{makeTestListFromExamples}
\alias{svTest}
\alias{print.svTest}
\alias{as.svTest}
\alias{is.svTest}
\alias{is.test}
\alias{test}
\alias{test<-}
\alias{makeUnit}
\alias{makeUnit.default}
\alias{makeUnit.svTest}
\alias{runTest}
\alias{runTest.default}
\alias{runTest.list}
\alias{runTest.svTest}
\title{Create, attach to and manipulate test functions in R objects}
\usage{
makeTestListFromExamples(packageName, manFilesDir, skipFailing = FALSE)

svTest(testFun)

\method{print}{svTest}(x, ...)

as.svTest(x)

is.svTest(x)

is.test(x)

test(x)

test(x) <- value

makeUnit(x, ...)

\method{makeUnit}{default}(
  x,
  name = make.names(deparse(substitute(x))),
  dir = tempdir(),
  objfile = "",
  codeSetUp = NULL,
  codeTearDown = NULL,
  ...
)

\method{makeUnit}{svTest}(
  x,
  name = make.names(deparse(substitute(x))),
  dir = tempdir(),
  objfile = "",
  codeSetUp = NULL,
  codeTearDown = NULL,
  ...
)

runTest(x, ...)

\method{runTest}{default}(
  x,
  name = deparse(substitute(x)),
  objfile = "",
  tag = "",
  msg = "",
  ...
)

\method{runTest}{list}(x, ...)

\method{runTest}{svTest}(
  x,
  name = deparse(substitute(x)),
  objfile = "",
  tag = "",
  msg = "",
  ...
)
}
\arguments{
\item{packageName}{A character string identifying the package from which to
extract examples.}

\item{manFilesDir}{A character string identifying the directory holding the
manual pages and examples.}

\item{skipFailing}{A logical indicating whether missing or failing
documentation examples should be marked as \code{skipped} instead of as \code{failure}.}

\item{testFun}{A function without arguments defining assertions (using
\code{checkxxx()} functions) for tests to be transformed into a 'svTest' object.}

\item{x}{Any kind of object.}

\item{...}{Further arguments to the method (not used yet).}

\item{value}{The tests to place in the object (as 'test' attribute); could be
a 'svTest' object, or a function without arguments with
assertions (\code{checkxxx()} functions).}

\item{name}{The name of a test.}

\item{dir}{The directory where to create the test unit file.}

\item{objfile}{The path to the file containing the original source code of
the object being tested. This argument is used to bring a context for a test
and allow a GUI to automatically open the source file for edition when the
user clicks on a test that failed or raised an error.}

\item{codeSetUp}{An expression with some code you want to add to the
\code{.setUp()} function in your unit file (this function is executed before each
test.}

\item{codeTearDown}{An expression with some code you want to add to the
\code{.tearDown()} function in your unit file (this function is executed after
each test.}

\item{tag}{A tag is a character string identifying a location in source code
files (either a test unit file, or the original source code of the tested
objects defined in \verb{objfile =}. This character string will be searched by the
text editor for easy location of the cursor near the corresponding test
command, or near the location in the original object that is concerned by
this test. Use any string you want to uniquely identify your tag, both in
your files, and in this argument.}

\item{msg}{A message you want to associate with this test run.}
}
\value{
A 'svTest' object for \code{\link[=svTest]{svTest()}}, \code{\link[=as.svTest]{as.svTest()}} and \code{\link[=test]{test()}}. Function
\code{\link[=is.svTest]{is.svTest()}} returns \code{TRUE} if 'x' is a 'svTest' object, and \code{is.test()}
does the same but also looks in the 'test' attribute if the class of 'x' is
not 'svTest' and returns \code{TRUE} if it finds something there.

\code{\link[=makeUnit]{makeUnit()}} takes an object, extract its test function and write it in a
sourceable test unit on the disk (it should be compatible with 'RUnit' test
unit files too).

\code{\link[=runTest]{runTest()}} returns invisibly a 'svTestData' object with all results after running specified tests.
}
\description{
Test functions are functions without arguments with class 'svTest' containing
one or more assertions using \code{checkxxx()} functions. They can be
attached to any object as a 'test' attribute. They can also be transferred
into a more formal test unit file on disk by applying the \code{\link[=makeUnit]{makeUnit()}} method.
}
\examples{
clearLog()    # Clear the log file

foo <- function(x, y = 2)
  return(x * y)
is.test(foo)    # No
# Create test cases for this function
test(foo) <- function() {
  checkEqualsNumeric(4, foo(2))
  checkEqualsNumeric(6, foo(2, 3))
  checkTrue(is.test(foo))
  checkTrue(is.test(test(foo)))
  checkIdentical(attr(foo, "test"), test(foo))
  checkException(foo(2, "aa"))
  checkException(foo("bb"))
}
is.test(foo)    # Yes

\dontrun{
# Create a test unit on disk and view it
unit <- makeUnit(foo)
file.show(unit, delete.file = TRUE)
}

# Run the test
(runTest(foo))
# Same as...
bar <- test(foo)
(runTest(bar))

# How fast can we run 100 times such kind of tests (700 test in total)?
# (just an indication because in real situation with test unit files, we
# have also the time required to source the units!)
system.time(for (i in 1:100) runTest(foo))[3]

is.svTest(test(foo))    # Yes, of course!
# When an object without associated test is passed to runTest(),
# a simple test containing only a DEACTIVATED entry is build
x <- 1:10
summary(runTest(x))

summary(Log())

rm(foo, bar, x)
}
\seealso{
\code{\link[=svSuite]{svSuite()}}, \code{\link[=is.svTestData]{is.svTestData()}}, \code{\link[=Log]{Log()}}, \code{\link[=checkEquals]{checkEquals()}}
}
\author{
Philippe Grosjean
}
\concept{unit testing}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/svTestData.R
\name{svTestData}
\alias{svTestData}
\alias{is.svTestData}
\alias{stats}
\alias{stats.svTestData}
\alias{print.svTestData}
\alias{summary.svTestData}
\alias{protocol_junit.svTestData}
\title{Objects of class 'svTestData' contain results from running a test}
\usage{
is.svTestData(x)

stats(object, ...)

\method{stats}{svTestData}(object, ...)

\method{print}{svTestData}(x, all = FALSE, header = TRUE, file = "", append = FALSE, ...)

\method{summary}{svTestData}(object, header = TRUE, file = "", append = FALSE, ...)

\method{protocol_junit}{svTestData}(object, ...)
}
\arguments{
\item{x}{Any kind of object, or a 'svTestData' object in the case of
\code{\link[=print]{print()}}.}

\item{object}{A 'svTestData' object.}

\item{...}{Further arguments to pass to methods. Not used yet.}

\item{all}{Do we print concise report for all test, or only for the tests
that fail or produce an error?}

\item{header}{Do we print a header or not?}

\item{file}{Character. The path to the file where to write the report. If
\code{file = ""}, the report is output to the console.}

\item{append}{Do we append to this file?}
}
\value{
\code{\link[=is.svTestData]{is.svTestData()}} returns \code{TRUE} if the object is an 'svTestData'. The
various methods serve to extract or print content in the object.
}
\description{
The 'svTestData' contains results of test run. The \code{checkxxx()} functions and
the \code{runTest()} method generate one such object which is located in the
\code{.Log} object in \code{.GlobalEnv}. It is then possible to display and report
information it contains in various ways to analyze the results.
}
\examples{
foo <- function(x, y = 2)
  return(x * y)
is.test(foo)  # No
# Create test cases for this function
test(foo) <- function() {
  checkEqualsNumeric(4, foo(2))
  checkEqualsNumeric(5, foo(2, 3))
  checkEqualsNumeric(5, foo(nonexists))
}
# Generate a 'svTestData' object by running the test
obj <- runTest(foo)  # Equivalent to runTest(test(foo)), but shorter
obj
summary(obj)
stats(obj)
is.svTestData(obj)

rm(foo, obj)
}
\seealso{
\code{\link[=svTest]{svTest()}}, \code{\link[=is.svSuiteData]{is.svSuiteData()}}, \code{\link[=Log]{Log()}}, \code{\link[=checkEquals]{checkEquals()}}
}
\author{
Philippe Grosjean
}
\concept{unit testing}
\keyword{utilities}
